# -*- coding: utf-8 -*-
=begin
  Copyright (C) 2011-2015 Takashi SUGA

  You may use and/or modify this file according to the license described in the LICENSE.txt file included in this archive.
=end

require 'when_exe/region/chinese'
require 'when_exe/region/chinese/calendars'

module When
  class TM::CalendarEra

    #
    # 中国の年号
    #
    Chinese = [{'V'=>{'0618'=>{'A'=>'0618', 'B'=>'0618B', 'C'=>'0618C'}}}, self, [
      "locale:[=ja:, en=en:]",
      'area:[中国#{?V=V}=ja:%%<元号>#%.<中国>,China#{?V=V}=en:Chinese_era_name, zh:中國#{?V=V}=zh:%%<年号>#%.<中國>]',
      [self,
	"period:[秦]",
	["[<始皇帝>]25*",			"@F"	,	"name=[始皇帝];-221*10-01^Chinese_221"],# 歳首は１０月
	["[<二世皇帝>]1*",			"@A"	,	"name=[二世皇帝];-208*10-01"],
	["[<子嬰>]1.08",			"@A"	,	"name=[子嬰];-206-08", "-205*10="]
      ],
      [self,
	"period:[前漢]",
	["[<高祖>=ja:%%<劉邦>,alias:高帝]1*",	"@F"	,	"name=[高祖=ja:%%<劉邦>];-205*10^Chinese_221"],		# 顓頊暦で推算
	["[<恵帝>=ja:%%<恵帝_(漢)>]1*",		"@A"	,	"name=[恵帝=ja:%%<恵帝_(漢)>];-193*10-01"],		# 前年４月甲辰高祖崩ず
	["[<高后>]1*",				"@A"	,	"name=[高后];-186*10-01"],				# 前年８月戊寅恵帝崩ず
	["[文帝#前元=zh:%%<前元>#%.<汉文帝>]1*","@A"	,	"name=[文帝=ja:%%<文帝_(漢)>];-178*10-01"],		# 前年７月辛巳高后崩ず
	["[文帝#後元=zh:%%<後元>#%.<汉文帝>]1*",""	,	"	     -162*10-01"],
	["[景帝#前元=zh:%%<前元>#%.<汉景帝>]1*","@A"	,	"name=[景帝=ja:%%<景帝_(漢)>];-155*10-01"],		# 前年６月己亥文帝崩ず
	["[中元=ja:%%<景帝_(漢)>]1*",		""	,	"	     -148*10-01"],
	["[景帝#後元=zh:%%<後元>#%.<汉景帝>]1*",""	,	"	     -142*10-01"],
	["[建元=ja:%%<建元_(漢)>]1*",		"@A"	,	"name=[武帝=ja:%%<武帝_(漢)>];-139*10-01"],		# 前年正月甲子景帝崩ず
	["[元光=ja:%%<元光_(漢)>]1*",		""	,	"	     -133*10-01"],
	["[元朔]1*",				""	,	"	     -127*10-01"],
	["[元狩]1*",				""	,	"	     -121*10-01"],
	["[元鼎]1*",				""	,	"	     -115*10-01"],
	["[元封=ja:%%<元封_(漢)>]1*",		""	,	"	     -109*10-01"],				# 以前は秦にひきつづいて１０月を歳首とする
	["[太初=ja:%%<太初_(漢)>]1*",		""	,	"	     -103*10-01",
									    "-103-05-01^Chinese_103", ""],		# 太初元年５月から太初暦を使用
															# 正月を歳首とする
	["[天漢=ja:%%<天漢_(漢)>]1"	,	""	,	"	     -99-01-01"],
	["[太始=ja:%%<太始_(漢)>]1"	,	""	,	"	     -95-01-01"],
	["[征和]1"	,			""	,	"	     -91-01-01"],
	["[武帝#後元=ja:%%<後元>]1"	,	""	,	"	     -87-01-01"],
	["[始元=ja:%%<始元_(漢)>]1"	,	"@A"	,	"name=[昭帝=ja:%%<昭帝_(漢)>];-85-01-01"],		# 前年２月丁卯武帝崩ず
	["[元鳳]1"	,			""	,	"	     -79-08"],
	["[元平]1"	,			""	,	"	     -73-01-01"],
	["[本始]1"	,			"@A"	,	"name=[宣帝=ja:%%<宣帝_(漢)>];-72-01-01"],		# 前年４月癸未昭帝崩ず、７月庚申宣帝即位
	["[地節]1"	,			""	,	"	     -68-01-01"],
	["[元康=ja:%%<元康_(漢)>]1"	,	""	,	"	     -64-01-01"],
	["[神爵]1"	,			""	,	"	     -60-03"],
	["[五鳳=ja:%%<五鳳_(漢)>]1"	,	""	,	"	     -56-01-01"],
	["[甘露=ja:%%<甘露_(漢)>]1"	,	""	,	"	     -52-01-01"],
	["[黄龍=ja:%%<黄龍_(漢)>]1"	,	""	,	"	     -48-01-01"],
	["[初元]1"	,			"@A"	,	"name=[元帝=ja:%%<元帝_(漢)>];-47-01-01"],		# 前年１２月甲戌宣帝崩ず
	["[永光=ja:%%<永光_(漢)>]1"	,	""	,	"	     -42-01-01"],
	["[建昭]1"	,			""	,	"	     -37-01-01"],
	["[竟寧]1"	,			""	,	"	     -32-01-01"],
	["[建始=ja:%%<建始_(漢)>]1"	,	"@A"	,	"name=[成帝=ja:%%<成帝_(漢)>];-31-01-01"],		# 前年５月壬辰元帝崩ず
	["[河平]1"	,			""	,	"	     -27-03"],
	["[陽朔]1"	,			""	,	"	     -23-01-01"],
	["[鴻嘉]1"	,			""	,	"	     -19-01-01"],
	["[永始=ja:%%<永始_(漢)>]1"	,	""	,	"	     -15-01-01"],
	["[元延]1"	,			""	,	"	     -11-01-01"],
	["[綏和]1"	,			""	,	"	     -7-01-01"],
	["[建平=ja:%%<建平_(漢)>]1"	,	"@A"	,	"name=[哀帝=ja:%%<哀帝_(漢)>];-5-01-01"],		# 前年３月丙戌成帝崩ず
	["[太初元将]1"	,			""	,	"	     -4-06"],
	["[建平=ja:%%<建平_(漢)>]2.08",		""	,	"	     -4-08"],
	["[元寿=ja:%%<元寿_(漢)>]1"	,	""	,	"	     -1-01-01"],
	["[元始=ja:%%<元始_(漢)>]1"	,	"@A"	,	"name=[平帝=ja:%%<平帝_(漢)>];1-01-01"],		# 前年６月戊午哀帝崩ず
	["[居摂]1"	,			"@A"	,	"name=[孺子嬰];6-01-01"],				# 前年１２月丙午王莽が平帝を毒殺す
	["[初始]1"	,			""	,	"	     8-11", "9*12-01"],
	[self,
	  "period:[新]",
	  ["[始建国]1*12.01",			"@F"	,	"name=[王莽];9*12-01^Chinese0008"],			# １２月を歳首とする
	  ["[天鳳]1*12.01",			""	,	"	     14*12-01"],
	  ["[地皇]1*12.01",			""	,	"	     20*12-01", "23-09-03="]
	],
	[self,
	  "period:[漢=ja:%%<更始帝>]",
	  ["[更始=ja:%%<更始_(漢)>]1*"	,	"@F"	,	"name=[更始帝];23-02-01^Chinese0008",			# 辛巳
									      "23-09-04^Chinese_103", "25-09-07"],	# 本年９月３日庚戌王莽亡
	],
	[self,
	  "period:[赤眉]",
	  ["[建世]1"	,			"@F"	,	"name=[劉盆子];25-06^Chinese_103", "27-01?"]
	],
	[self,
	  "period:[成家=ja:%%<公孫述>]",
	  ["[龍興=ja:%%<龍興_(公孫述)>]1",	"@F"	,	"name=[公孫述];25-04^Chinese_103", "36-11="]
	],
	[self,
	  "period:[隴=ja:%%<隗囂>]",
	  ["[漢復]1*"	,			"@F"	,	"name=[隗囂];23-07^Chinese0008",
								"name=[隗囂];23-09-04^Chinese_103",
								"name=[隗純];033", "34-10="]
	]
      ],
      [self,
	"period:[後漢]",
	["[建武=ja:%%<建武_(漢)>]1"	,	"@F"	,	"name=[光武帝];25-06-22^Chinese_103"],			# 己未
	["[建武中元]1"	,			""	,	"	     56-04-11"],				# 己卯
	["[永平]1"	,			"@A"	,	"name=[明帝=ja:%%<明帝_(漢)>];58-01-01"],		# 己未
	["[建初]1"	,			"@A"	,	"name=[章帝];76-01-01"],				# 甲辰
	["[元和=ja:%%<元和_(漢)>]1"	,	""	,	"	     84-08-20"],				# 癸酉 翌元和２年から四分暦を使用
	["[章和=ja:%%<章和_(漢)>]1"	,	""	,	"	     87-07-27"],				# 壬戌
	["[永元=ja:%%<永元_(漢)>]1"	,	"@A"	,	"name=[和帝=ja:%%<和帝_(漢)>];89-01-01"],		# 戊子
	["[元興=ja:%%<元興_(漢)>]1"	,	""	,	"	     105-05-18"],				# 庚午?
	["[延平=ja:%%<延平_(漢)>]1"	,	"@A"	,	"name=[殤帝=ja:%%<殤帝_(漢)>];106-01-01"],		# 己卯
	["[永初=ja:%%<永初_(漢)>]1"	,	"@A"	,	"name=[安帝=ja:%%<安帝_(漢)>];107-01-01"],		# 癸酉
	["[元初]1"	,			""	,	"	     114-01-02"],				# 甲子
	["[永寧=ja:%%<永寧_(漢)>]1"	,	""	,	"	     120-04-11"],				# 丙寅
	["[建光=ja:%%<建光_(漢)>]1"	,	""	,	"	     121-07-01"],				# 己卯
	["[延光]1"	,			""	,	"	     122-03-02"],				# 丙午
	["[永建=ja:%%<永建_(漢)>]1"	,	"@A"	,	"name=[順帝=ja:%%<順帝_(漢)>];126-01-01"],		# 癸丑
	["[陽嘉]1"	,			""	,	"	     132-03-13"],				# 庚寅
	["[永和=ja:%%<永和_(漢)>]1"	,	""	,	"	     136-01-15"],				# 己巳
	["[漢安]1"	,			""	,	"	     142-01-14"],				# 癸巳
	["[建康=ja:%%<建康_(漢)>]1"	,	""	,	"	     144-04-15"],				# 辛巳
	["[永嘉]1"	,			"@A"	,	"name=[冲帝];145-01-01"],				# 癸巳
	["[本初]1"	,			"@A"	,	"name=[質帝];146-01-01"],				# 丁亥
	["[建和=ja:%%<建和_(漢)>]1"	,	"@A"	,	"name=[桓帝=ja:%%<桓帝_(漢)>];147-01-01"],		# 辛亥
	["[和平=ja:%%<和平_(漢)>]1"	,	""	,	"	     150-01-01"],				# 甲子
	["[元嘉=ja:%%<元嘉_(漢)>]1"	,	""	,	"	     151-01-16"],				# 癸酉
	["[永興=ja:%%<永興_(漢)>]1"	,	""	,	"	     153-05-22"],				# 丙申
	["[永寿=ja:%%<永寿_(漢)>]1"	,	""	,	"	     155-01-14"],				# 戊申
	["[延熹]1"	,			""	,	"	     158-06-04"],				# 戊寅
	["[永康=ja:%%<永康_(漢)>]1"	,	""	,	"	     167-06-08"],				# 庚申
	["[建寧=ja:%%<建寧_(漢)>]1"	,	"@A"	,	"name=[霊帝=ja:%%<霊帝_(漢)>];168-01-21"],		# 庚子
	["[熹平]1"	,			""	,	"	     172-05-16"],				# 己巳
	["[光和]1"	,			""	,	"	     178-03-21"],				# 辛丑
	["[中平]1"	,			""	,	"	     184-12-29"],				# 己巳
	["[光熹]1"	,			"@A"	,	"name=[弘農王=ja:%%<少帝弁>];189-04-13"],		# 戊午
	["[昭寧]1"	,			""	,	"	     189-08-28"],				# 辛未
	["[永漢=ja:%%<永漢_(漢)>]1"	,	"@A"	,	"name=[献帝=ja:%%<献帝_(漢)>];189-09-01"],		# 甲戌
	["[中平]6"	,			""	,	"	     189-12=27"],				# 戊戌?
	["[初平]1"	,			""	,	"	     190-01-01"],				# 壬寅
	["[興平=ja:%%<興平_(漢)>]1"	,	""	,	"	     194-01-13"],				# 辛酉
	["[建安=ja:%%<建安_(漢)>]1"	,	""	,	"	     196-01-07"],				# 癸酉
	["[延康=ja:%%<延康_(漢)>]1"	,	""	,	"	     220-03", "220-10-29"]
      ],
      [self,
	"period:[三国=ja:%%<三国時代_(中国)>, zh:三國=zh:%%<三国>]",
	[self,
	  "period:[蜀漢]",
	  ["[章武]1"	,			"@A"	,	"name=[昭烈帝=ja:%%<劉備>];221-04-06^Chinese_103"],	# 劉備 丙午 2185-4 四分暦を使用
	  ["[建興=ja:%%<建興_(蜀)>]1"	,	"@A"	,	"name=[後主=ja:%%<劉禅>];223-05"],			# 劉禅 2214-6
	  ["[延熙=ja:%%<延熙_(蜀)>]1"	,	""	,	"	     238-01"],					# 三国志 897
	  ["[景耀]1"	,			""	,	"	     258-04"],					# ？
	  ["[炎興]1"	,			""	,	"	     263-08", "263-11="]			# 2467-3
	],
	[self,
	  "period:[呉=ja:%%<呉_(三国)>]",
	  ["[黄武]1"	,			"@F"	,	"name=[大帝=ja:%%<孫権>];222-10^Chinese0223"],		# 孫権 2208-18 翌黄武２年乾象暦使用
	  ["[黄龍=ja:%%<黄龍_(呉)>]1"	,	""	,	"	     229-04-13"],				# 丙申 2252-2
	  ["[嘉禾]1"	,			""	,	"	     232-01-01"],				# 2274-13
	  ["[赤烏]1"	,			""	,	"	     238-08"],					# 2338-8
	  ["[太元=ja:%%<太元_(呉)>]1"	,	""	,	"	     251-05"],					# 2391-6
	  ["[神鳳=ja:%%<神鳳_(呉)>]1"	,	""	,	"	     252-02"],					# 2394-4 これは大帝の年号
	  ["[建興=ja:%%<建興_(呉)>]1"	,	"@A"	,	"name=[会稽王=ja:%%<孫亮>];252-04"],			# 孫亮 2394-4
	  ["[五鳳=ja:%%<五鳳_(呉)>]1"	,	""	,	"	     254-01-01"],				# 三国志 1152
	  ["[太平=ja:%%<太平_(呉)>]1"	,	""	,	"	     256-10-06"],				# 乙酉 2434-10
	  ["[永安=ja:%%<永安_(呉)>]1"	,	"@A"	,	"name=[景帝=ja:%%<孫休>];258-10-18"],			# 孫休 三国志 1155 通鑑 2447-8 戊午変
	  ["[元興=ja:%%<元興_(呉)>]1"	,	"@A"	,	"name=[烏程公=ja:%%<孫皓>];264-07-25"],			# 孫皓 癸未 2487-15
	  ["[甘露=ja:%%<甘露_(呉)>]1"	,	""	,	"	     265-04"],					# 2491-2
	  ["[宝鼎]1"	,			""	,	"	     266-08"],					# 2498-11
	  ["[建衡]1"	,			""	,	"	     269-10"],					# 2510-8
	  ["[鳳凰=ja:%%<鳳凰_(呉)>]1",		"",	"	     272-01-01"],					# 2518-20
	  ["[天冊]1"	,			""	,	"	     275-01"],					# 2541-2
	  ["[天璽=ja:%%<天璽_(呉)>]1"	,	""	,	"	     276-07"],					# 2542-3
	  ["[天紀]1"	,	""	,	"	     277-01-01", "280-04-16="]					# 三国志 1171
	],
	[self,
	  "period:[燕=ja:%%<公孫淵>]",
	  ["[紹漢]1*"	,			"@F"	,	"name=[公孫淵];237-07^Chinese0237", "238-08-23="]	# 2319-8
	],
	[self,
	  "period:[魏=ja:%%<魏_(三国)>]",
	  ["[黄初]1"	,			"@F"	,	"name=[文帝=ja:%%<曹丕>];220-10-29^Chinese_103"],	# 辛未
	  ["[太和=ja:%%<太和_(魏)>]1"	,	"@A"	,	"name=[明帝];227-01-01"],				# 丁酉
	  ["[青龍=ja:%%<青龍_(魏)>]1"	,	""	,	"	     233-02-06"],				# 丁酉
	  ["[景初]1*"	,			""	,	"name=[明帝=ja:%%<曹叡>];237-03^Chinese0237",		# 改元と同時に景初暦使用、１２月を歳首とする
								"name=[明帝];239-12-01^Chinese0239",""],		# 正月を歳首にもどす
	  ["[正始=ja:%%<正始_(魏)>]1"	,	"@A"	,	"name=[斉王=ja:%%<曹芳>];240-01-01"],			# 辛亥
	  ["[嘉平=ja:%%<嘉平_(魏)>]1"	,	""	,	"	     249-04-08"],				# 乙丑
	  ["[正元=ja:%%<正元_(魏)>]1"	,	"@A"	,	"name=[高貴郷公=ja:%%<曹髦>];254-10-28"],		# 癸丑
	  ["[甘露=ja:%%<甘露_(魏)>]1"	,	""	,	"	     256-06-01"],				# 丙午
	  ["[景元]1"	,			"@A"	,	"name=[元帝=ja:%%<曹奐>];260-06-02"],			# 甲寅
	  ["[咸熙]1"	,			""	,	"	     264-05-15", "265-12-17"]			# 甲戌
	]
      ],
      [self,
	"period:[晋=ja:%%<晋_(王朝)>]",
	[self,
	  "period:[西晋]",
	  ["[泰始=ja:%%<泰始_(晋)>]1"	,	"@F"	,	"name=[武帝=ja:%%<司馬炎>];265-12-17^Chinese0239"],	# 丙寅
	  ["[咸寧=ja:%%<咸寧_(晋)>]1"	,	""	,	"	     275-01-01"],				# 戊午
	  ["[太康=ja:%%<太康_(晋)>]1"	,	""	,	"	     280-04-29"],				# 乙酉
	  ["[太熙]1"	,	""	,	"	     290-01-01"],						# 辛酉
	  ["[永熙=ja:%%<永熙_(晋)>]1"	,	"@A"	,	"name=[恵帝=ja:%%<恵帝_(西晋)>];290-04-20"],		# 己酉
	  ["[永平=ja:%%<永平_(晋)>]1"	,	""	,	"	     291-01-01"],				# 乙酉
	  ["[元康=ja:%%<元康_(晋)>]1"	,	""	,	"	     291-03-09"],				# 壬辰
	  ["[永康=ja:%%<永康_(晋)>]1"	,	""	,	"	     300-01-01"],				# 癸亥
	  ["[永寧=ja:%%<永寧_(晋)>]1"	,	""	,	"	     301-04-09"],				# 癸亥
	  ["[太安=ja:%%<太安_(晋)>]1"	,	""	,	"	     302-12-22"],				# 丁卯
	  ["[永安=ja:%%<永安_(晋)>]1"	,	""	,	"	     304-01-26"],				# 甲子
	  ["[建武=ja:%%<建武_(西晋)>]1"	,	""	,	"	     304-07-25"],				# 庚申
	  ["[永安=ja:%%<永安_(晋)>]1"	,	""	,	"	     304-11-12"],				# 丙午
	  ["[永興=ja:%%<永興_(晋)>]1"	,	""	,	"	     304-12-24"],				# 丁亥
	  ["[光熙]1"	,	""	,	"	     306-06-16"],						# 辛未
	  ["[永嘉=ja:%%<永嘉_(晋)>]1"	,	"@A"	,	"name=[懐帝=ja:%%<懐帝_(西晋)>];307-01-02"],		# 癸丑
	  ["[建興=ja:%%<建興_(晋)>]1"	,	"@A"	,	"name=[愍帝=ja:%%<愍帝_(西晋)>];313-04-27", "317-03-09"] # 壬申
	],
	[self,
	  "period:[東晋]",
	  ["[建武=ja:%%<建武_(東晋)>]1"	,	"@A"	,	"name=[元帝=ja:%%<元帝_(東晋)>];317-03-09^Chinese0239"], # 辛卯
	  ["[大興=ja:%%<大興_(東晋)>,alias:太興=]1",""	,	"	     318-03-10"],				# 丙辰
	  ["[永昌=ja:%%<永昌_(東晋)>]1"	,	""	,	"	     322-01-01"],				# 乙卯
	  ["[太寧=ja:%%<太寧_(東晋)>,alias:泰寧=]1",""	,	"name=[明帝=ja:%%<明帝_(東晋)>];323-03-01"],		# 戊寅
	  ["[咸和=ja:%%<咸和_(東晋)>]1"	,	"@A"	,	"name=[成帝=ja:%%<成帝_(東晋)>];326-02-27"],		# 丁亥
	  ["[咸康=ja:%%<咸康_(東晋)>]1"	,	""	,	"	     335-01-01"],				# 庚午
	  ["[建元=ja:%%<建元_(東晋)>]1"	,	"@A"	,	"name=[康帝=ja:%%<康帝_(東晋)>];343-01-01"],		# 癸丑
	  ["[永和=ja:%%<永和_(東晋)>]1"	,	"@A"	,	"name=[穆帝=ja:%%<穆帝_(東晋)>];345-01-01"],		# 辛未
	  ["[升平]1"	,			""	,	"	     357-01-01"],				# 壬戌
	  ["[隆和]1"	,			"@A"	,	"name=[哀帝=ja:%%<哀帝_(東晋)>];362-01-20"],		# 壬子
	  ["[興寧]1"	,			""	,	"	     363-03-14"],				# 己亥
	  ["[太和=ja:%%<太和_(東晋)>]1"	,	"@A"	,	"name=[廃帝=ja:%%<廃帝_(東晋)>];366-01-01"],		# 庚子
	  ["[咸安=ja:%%<咸安_(東晋)>]1"	,	"@A"	,	"name=[簡文帝=ja:%%<簡文帝_(東晋)>];371-11-15"],	# 己酉
	  ["[寧康]1"	,			"@A"	,	"name=[孝武帝=ja:%%<孝武帝_(東晋)>];373-01-01"],	# 己丑
	  ["[太元=ja:%%<太元_(東晋)>]1"	,	""	,	"	     376-01-03"],		# 甲辰
	  ["[崇安=]1"	,			""	,	"name=[安帝=ja:%%<安帝_(東晋)>];397-01-01"],
	  ["[隆安]1"	,			"@A"	,	"	     397-01-01"],				# 己亥
	  ["[元興=ja:%%<元興_(東晋)>]1"	,	""	,	"	     402-01-01"],				# 庚午
	  ["[隆安]6.03.04",			""	,	"	     402-03-04"],				# 壬寅
	  ["[大亨]1"	,			""	,	"	     402-03-05"],				# 癸酉
	  ["[元興=ja:%%<元興_(東晋)>]2"	,	""	,	"	     403-01-01"],				# 乙丑
	  ["[義熙]1"	,			""	,	"	     405-01-16"],				# 戊戌
	  ["[元熙=ja:%%<元熙_(東晋)>]1"	,	"@A"	,	"name=[恭帝=ja:%%<恭帝_(東晋)>];419-01-01", "420-06-14"] # 壬辰
	],
	[self,
	  "period:[楚=ja:%%<桓玄>]",
	  ["[永始=ja:%%<永始_(桓玄)>]1"	,	"@F"	,	"name=[桓玄];403-12-03^Chinese0239", "404-05-26="],	# 壬辰 3555-19, 壬午 3571-28
	  ["[天康=zh:%%<天康_(桓谦)>]1"	,	""	,	"name=[桓謙=zh:%%<桓谦>];404-05=", "405-02="],		# 資治通鑑には見つからない
	],
      ],
      [self,
	"period:[南朝=ja:%%<南北朝時代_(中国)>#%.<南朝>]",
	[self,
	  "period:[宋=ja:%%<宋_(南朝)>]",
	  ["[永初=ja:%%<永初_(南朝宋)>]1",	"@F"	,	"name=[武帝=ja:%%<劉裕>];420-06-14^Chinese0239"],	# 丁卯
	  ["[景平]1"	,			"@A"	,	"name=[少帝=ja:%%<少帝_(南朝宋)>];423-01-01"],		# 己亥
	  ["[元嘉=ja:%%<元嘉_(南朝宋)>]1",	"@A"	,	"name=[文帝=ja:%%<文帝_(南朝宋)>];424-08-09", "445-01-01^Chinese0445", ""],# 丁酉 元嘉２２年から元嘉暦を使用
	# ["[太初_(南朝宋)]1"	,		"@A"	,	"name=[劉劭=ja:%%<劉劭_(南朝宋)>];453-02"],		# Wikipedia
	# ["[元嘉=ja:%%<元嘉_(南朝宋)>]30",	""	,	"name=[文帝=ja:%%<文帝_(南朝宋)>];453-04="],		# Wikipedia
	  ["[孝建]1"	,			"@A"	,	"name=[孝武帝=ja:%%<孝武帝_(南朝宋)>];454-01-01"],	# 己亥
	  ["[大明=ja:%%<大明_(南朝宋)>]1",	""	,	"	     457-01-01"],				# 辛亥
	  ["[永光=ja:%%<永光_(南朝宋)>]1",	"@A"	,	"name=[前廃帝=ja:%%<前廃帝_(南朝宋)>];465-01-01"],	# 乙未
	  ["[景和]1"	,			""	,	"	     465-08-13"],				# 癸酉
	  ["[泰始=ja:%%<泰始_(南朝宋)>]1",	"@A"	,	"name=[明帝=ja:%%<明帝_(南朝宋)>];465-12-07"],		# 丙寅 本年１２月に大明に戻した後、ただちに改元
	  ["[泰豫=ja:%%<泰予>,alias:泰予=ja:%%<泰予>]1",""	,	"	     472-01-01"],			# 甲寅
	  ["[元徽=ja:%%<元徽_(南朝宋)>]1",	"@A"	,	"name=[後廃帝=ja:%%<後廃帝_(南朝宋)>];473-01-01"],	# 戊寅
	  ["[昇明]1"	,			"@A"	,	"name=[順帝=ja:%%<順帝_(南朝宋)>];477-07-11", "479-04-23"] # 壬辰
	],
	[self,
	  "period:[斉=ja:%%<斉_(南朝)>]",
	  ["[建元=ja:%%<建元_(南朝斉)>]1",	"@F"	,	"name=[高帝=ja:%%<蕭道成>];479-04-23^Chinese0445"],	# 甲午
	  ["[永明]1"	,			"@A"	,	"name=[武帝=ja:%%<武帝_(南朝斉)>];483-01-02"],		# 辛亥
	  ["[隆昌]1"	,			"@A"	,	"name=[欝林王=ja:%%<蕭昭業>];494-01-01"],		# 丁未
	  ["[延興=ja:%%<延興_(南朝斉)>]1",	"@A"	,	"name=[海陵王=ja:%%<蕭昭文>];494-07-25"],		# 丁酉
	  ["[建武=ja:%%<建武_(南朝斉)>]1",	"@A"	,	"name=[明帝=ja:%%<明帝_(南朝斉)>];494-10-22"],		# 癸亥
	  ["[永泰=ja:%%<永泰_(南朝斉)>]1",	""	,	"	     498-04-03"],				# 甲寅
	  ["[永元=ja:%%<永元_(南朝斉)>]1",	"@A"	,	"name=[東昏侯=ja:%%<蕭宝巻>];499-01-01"],		# 戊寅
	  ["[中興=ja:%%<中興_(南朝斉)>]1",	"@A"	,	"name=[和帝=ja:%%<和帝_(南朝斉)>];501-03-11", "502-04-08"] # 乙巳
	],
	[self,
	  "period:[梁=ja:%%<梁_(南朝)>]",
	  ["[天監]1"	,			"@F"	,	"name=[武帝=ja:%%<蕭衍>];502-04-08^Chinese0445"],	# 丙寅 天監９年から大明暦を使用
	  ["[普通=ja:%%<普通_(梁)>]1"	,	""	,	"	     520-01-01"],				# 乙亥
	  ["[大通=ja:%%<大通_(梁)>]1"	,	""	,	"	     527-03-11"],				# 甲戌
	  ["[中大通]1"	,			""	,	"	     529-10-01"],				# 己酉
	  ["[大同=ja:%%<大同_(梁)>]1"	,	""	,	"	     535-01-01"],				# 戊申
	  ["[中大同]1"	,			""	,	"	     546-04-14"],				# 丙戌
	  ["[太清]1"	,			""	,	"	     547-04-21"],				# 丁亥
	  ["[大宝=ja:%%<大宝_(梁)>]1"	,	"@A"	,	"name=[簡文帝=ja:%%<簡文帝_(南朝梁)>];550-01-01"],	# 辛亥
	  ["[天正=ja:%%<天正_(梁)>]1"	,	"@A"	,	"name=[豫章王=ja:%%<蕭棟>];551-08-21"],			# 壬戌 5072-29
	  ["[承聖]1"	,			"@A"	,	"name=[元帝=ja:%%<元帝_(南朝梁)>];552-11-12"],		# 丙子 太清に戻した後、ただちに改元
	  ["[天成=ja:%%<天成_(梁)>]1"	,	"@A"	,	"name=[貞陽侯=ja:%%<蕭淵明>];555-05-27"],		# 丙午
	  ["[紹泰]1"	,			"@A"	,	"name=[敬帝=ja:%%<敬帝_(南朝梁)>];555-10-02"],		# 己酉
	  ["[太平=ja:%%<太平_(梁)>]1"	,	""	,	"	     556-09-01", "557-10-10"],			# 壬寅
	  [self,
	    "period:[<臨賀王>=ja:%%<梁_(南朝)>]",
	    ["[正平=ja:%%<正平_(蕭正徳)>]1",	"@A"	,"name=[臨賀王=ja:%%<蕭正徳>];548-11-01^Chinese0445", "549-03="] # 朔 4989-21
	  ],
	  [self,
	    "period:[<武陵王>=ja:%%<梁_(南朝)>]",
	    ["[天正=ja:%%<天正_(蕭紀)>]1"	,	"@A"	,"name=[武陵王=ja:%%<蕭紀>];552-04-08^Chinese0445", "553-08-08="] # 乙巳 5084-13
	  ],
	  [self,
	    "period:[<永嘉王>=ja:%%<梁_(南朝)>]",
	    ["[天啓=ja:%%<天啓_(梁)>]1"	,	"@A"	,"name=[永嘉王=ja:%%<蕭荘>];558-03^Chinese0445", "560-02="]	# 5174-11
	  ]
	],
	[self,
	  "period:[陳=ja:%%<陳_(南朝)>]",
	  ["[永定=ja:%%<永定_(陳)>]1"	,	"@F"	,	"name=[武帝=ja:%%<陳霸先>];557-10-10^Chinese0445"],	# 乙亥
	  ["[天嘉=ja:%%<天嘉_(陳)>]1"	,	"@A"	,	"name=[文帝=ja:%%<文帝_(陳)>];560-01-01"],		# 癸丑
	  ["[天康=ja:%%<天康_(陳)>]1"	,	""	,	"	     566-02-29"],				# 丙子
	  ["[光大]1"	,	"@A"	,	"name=[廃帝=ja:%%<廃帝_(陳)>];567-01-03"],				# 乙亥
	  ["[太建]1"	,	"@A"	,	"name=[宣帝=ja:%%<宣帝_(陳)>];569-01-04"],				# 甲午
	  ["[至徳=ja:%%<至徳_(陳)>]1"	,	"@A"	,	"name=[後主=ja:%%<後主_(陳)>];583-01-03"],		# 壬寅
	  ["[禎明]1"	,			""	,	"	     587-01-03", "589-01="]			# 戊寅
	],
	[self,
	  "period:[後梁=ja:%%<後梁_(南朝)>]",
	  ["[大定=ja:%%<大定_(南朝後梁)>]1",	"@A"	,	"name=[宣帝=ja:%%<蕭サツ>];555-01-01^Chinese0445"],
	  ["[天保=ja:%%<天保_(南朝後梁)>]1",	"@A"	,	"name=[孝明帝=ja:%%<蕭キ>];562-01"],
	  ["[広運=ja:%%<広運_(南朝後梁)>]1",	"@A"	,	"name=[蕭琮=ja:%%<蕭ソウ>];586-01", "587-09="]
	],
	[self,
	  "period:[蜀=zh:%%<程道養>]",
	  ["[泰始=ja:%%<泰始_(程道養)>]1",	"@F"	,"name=[程道養=zh:%%<程道養>];432-09^Chinese0239", "437-04="]	# 3841-20
	],
	[self,
	  "period:[秦=ja:%%<仇池>]",
	  ["[建義=ja:%%<建義_(仇池)>]1"	,	"@F"	,	"name=[楊難当];436-03^Chinese0239", "442-05?"]		# 3861-6
	],
	[self,
	  "period:[漢=ja:%%<侯景>]",
	  ["[太始=ja:%%<太始_(侯景)>]1"	,	"@F"	,	"name=[侯景];551-11-19^Chinese0445", "552-04-18="]	# 己丑 5075-36
	]
      ],
      [self,
	"period:[五胡十六国=ja:%%<五胡十六国時代>, zh:五胡十六國=zh:%%<五胡十六国>]",
	[self,
	  "period:[成漢]",
	  ["[建初=ja:%%<建初_(成漢)>]1"	,	"@F"	,	"name=[李特];303-01^Chinese0239",			# 景帝 2677-1 考異多
								"name=[李流];303-02", ""],				# 秦文王 2678-1
	  ["[建興=ja:%%<建興_(成漢)>]1"	,	"@A"	,	"name=[李雄=ja:%%<李雄_(成漢)>];304-10"],		# 武帝 2702-15
	  ["[晏平]1"	,			""	,	"	     306-06"],					# 2720-8
	  ["[玉衡]1"	,			""	,	"	     311-01-17",				# 乙亥 2758-3
								"name=[李班];334-06-25",				# 哀帝 丁卯 2996-11
								"name=[李期];334-10-24", ""],				# 幽帝 甲子 2997-15
	  ["[玉恒]1"	,			""	,	"	     335-01-01"],				# 庚午朔? 3000-2
	  ["[漢興]1"	,			"@A"	,	"name=[李寿];338-04"],					# 昭文帝 3017-3
	  ["[太和=ja:%%<太和_(成漢)>]1"	,	"@A"	,	"name=[李勢];344-01"],					# 3057-2
	  ["[嘉寧]1"	,	""	,			"	     346-10", "347-03="]			# 3072-10
	],
	[self,
	  "period:[前趙]",
	  ["[元熙=ja:%%<元熙_(前趙)>]1"	,	"@F"	,	"name=[劉淵];304-10^Chinese0239"],			# 光文帝 2702-16
	  ["[永鳳]1"	,			""	,	"	     308-10-03"],				# 甲戌 2738-13
	  ["[河瑞]1"	,			""	,	"	     309-01-01",				# 辛丑朔 2740-1
								"name=[劉和];310-07-18", ""],				# 己卯 2750-12
	  ["[光興=ja:%%<光興_(前趙)>]1"	,	"@A"	,	"name=[劉聡];310-07-24"],				# 昭武帝 乙酉 2751-12
	  ["[嘉平=ja:%%<嘉平_(前趙)>]1"	,	""	,	"	     311-06-21"],				# 丁未 2765-18
	  ["[建元=ja:%%<建元_(前趙)>]1"	,	""	,	"	     315-03"],					# 2820-8
	  ["[麟嘉=ja:%%<麟嘉_(前趙)>]1"	,	""	,	"	     316-11-18"],				# 壬寅 2835-12
	  ["[漢昌]1"	,			"@A"	,	"name=[劉粲];318-07-20"],				# 隠帝 甲子 2861-18
	  ["[光初]1"	,			"@A"	,	"name=[劉曜];318-10",					# 2862-18
								"name=[劉熙=ja:%%<劉煕>];328-12-25", "329-09="]		# 己亥 2965-11
	],
	[self,
	  "period:[後趙]",
	  ["[<石勒>]1"	,			"@F"	,	"name=[石勒];319-11^Chinese0239"],			# 明帝 2871-15
	  ["[太和=ja:%%<太和_(後趙)>]1"	,	""	,	"	     328-02"],					# 2953-2
	  ["[建平=ja:%%<建平_(後趙)>]1"	,	""	,	"	     330-09",					# 2977-7
								"name=[石弘];333-07-21", ""],				# 戊辰 2986-5
	  ["[延熙=ja:%%<延熙_(後趙)>]1"	,	""	,	"	     334-01-01",				# 乙亥 2993-1
								"name=[石虎];334-11", ""],				# 武帝 2998-16
	  ["[建武=ja:%%<建武_(後趙)>]1"	,	""	,	"	     335-01-01"],				# 庚午朔? 3000-2
	  ["[太寧=ja:%%<太寧_(後趙)>]1"	,	""	,	"	     349-01-01",				# 辛未朔? 2085-2
								"name=[石世];349-04-23",				# 己巳 3089-7
								"name=[石遵];349-05-15",				# 庚寅 3090-7
								"name=[石鑒];349-11", ""],				# 3097-14
	  ["[青龍=ja:%%<青龍_(後趙)>]1"	,	""	,	"	     350-01"],					# 3100-1
	  ["[永寧=ja:%%<永寧_(後趙)>]1"	,	"@A"	,	"name=[石祗];350-01=", "351-04="]			# 3102-1
	],
	[self,
	  "period:[冉魏]",
	  ["[永興=ja:%%<永興_(冉魏)>]1"	,	"@A"	,	"name=[冉閔];350-02=^Chinese0239", "352-04="]		# 3102-1 考異あり
	],
	[self,
	  "period:[翟魏=ja:%%<テキ魏>]",
	  ["[建光=ja:%%<建光_(テキ魏)>]1",	"@F"	,	"name=[翟遼=ja:%%<テキ遼>];388-02^Chinese0239"],	# 3382-3
	  ["[定鼎,alias:神鼎=ja:%%<定鼎>]1",	"@A"	,	"name=[翟釗=ja:%%<テキショウ>];391-10", "392-6="]	# 3402-17 -3405-9
	],
	[self,
	  "period:[前燕]",
	# ["[燕元=]0.11.26",			"@F"	,	"name=[慕容儁];348-11-26^Chinese0239"],			# 景昭帝 甲辰 3085-6 (即位記事のみ)
	  ["[元璽]1"	,			""	,	"name=[慕容儁];352-11-13^Chinese0239"],			# 戊辰 3131-27
	  ["[光寿]1"	,			""	,	"	     357-02-23"],				# 癸丑 3161-3
	  ["[建熙]1"	,			"@A"	,	"name=[慕容暐];360-01-15", "370-11="]			# 幽帝 戊子 3179-1
	],
	[self,
	  "period:[後燕]",
	  ["[燕元]1"	,			"@F"	,	"name=[慕容垂];384-01-26^Chinese0239"],			# 成武帝 庚戌 3323-1
	  ["[建興=ja:%%<建興_(後燕)>]1"	,	""	,	"	     386-02"],					# 3359-7
	  ["[永康=ja:%%<永康_(後燕)>]1"	,	"@A"	,	"name=[慕容宝];396-04-29", "398-07-21="],		# 恵愍帝 壬寅 3426-4
	  ["[青龍=ja:%%<青龍_(後燕)>]1"	,	"@A"	,	"name=[蘭汗];398-04-25"],				# 丁亥 3469-13 
	  ["[建平=ja:%%<建平_(後燕)>]1"	,	"@A"	,	"name=[慕容盛];398-07-21"],				# 辛亥 3472-18 昭武帝 10-17 丙子 3481-31
	  ["[長楽=ja:%%<長楽_(後燕)>]1"	,	""	,	"	     399-01-26"],				# 癸未 3487-5
	  ["[光始]1"	,			"@A"	,	"name=[慕容熙];401-08=24"],				# 昭文帝 丙寅 3527-18
	  ["[建始=ja:%%<建始_(慕容熙)>]1",	""	,	"	     407-01-01"],				# 辛丑朔 3594-1
	  ["[正始=ja:%%<正始_(北燕)>]1"	,	"@A"	,	"name=[慕容雲=ja:%%<高雲>];407-07-28", "409-10-13"],	# 恵懿帝 乙丑 3599-14
	],
	[self,
	  "period:[西燕]",
	  ["[燕興=ja:%%<更始_(西燕)>]1"	,	"@F"	,	"name=[慕容泓];384-04^Chinese0239"],			# 済北王 3328-14
	  ["[更始=ja:%%<更始_(西燕)>]1"	,	"@A"	,	"name=[慕容冲=ja:%%<慕容沖>];385-01"],			# 威帝 3339-2
	  ["[昌平]1"	,			"@A"	,	"name=[段隨=ja:%%<段随>];386-02"],			# 3359-8
	  ["[建明=ja:%%<建明_(西燕)>]1"	,	"@A"	,	"name=[慕容顗];386-03"],				# 3363-15
	  ["[建平=ja:%%<建平_(西燕)>]1"	,	"@A"	,	"name=[慕容瑶=ja:%%<慕容瑤>];386-03"],			# 3363-15
	  ["[建武=ja:%%<建武_(西燕)>]1"	,	"@A"	,	"name=[慕容忠];386-03"],				# 3363-15
	  ["[中興=ja:%%<中興_(西燕)>]1"	,	"@A"	,	"name=[慕容永];386-09", "394-08="]			# 3369-42
	],
	[self,
	  "period:[南燕]",
	  ["[燕平]1"	,			"@F"	,	"name=[慕容徳];398-01^Chinese0239"],			# 献武帝 3461-1
	  ["[建平=ja:%%<建平_(南燕)>]1"	,	""	,	"	     400"],					# 3516-40
	  ["[太上]1"	,			"@A"	,	"name=[慕容超];405-08-10", "410-02="]			# 己未 3586-17
	],
	[self,
	  "period:[北燕]",
	  ["[太平=ja:%%<太平_(北燕)>]1"	,	"@F"	,	"name=[馮跋];409-10-13^Chinese0239"],			# 文成帝 戊辰 3621-21
	  ["[太興]1"	,			"@A"	,	"name=[馮弘];431-01-01", "436-05="]			# 昭成帝 壬午朔 3828-1
	],
	[self,
	  "period:[前秦]",
	  ["[皇始=ja:%%<皇始_(前秦)>]1"	,	"@F"	,	"name=[苻健];351-01-20^Chinese0239"],			# 明帝 丙辰 3111-2
	  ["[寿光]1"	,			"@A"	,	"name=[苻生];355-06-16"],				# 廃帝 丙戌 3147-8
	  ["[永興=ja:%%<永興_(前秦)>]1"	,	"@A"	,	"name=[苻堅];357-06"],					# 宣昭帝 3165-8
	  ["[甘露=ja:%%<甘露_(前秦)>]1"	,	""	,	"	     359-06"],					# 3175-6
	  ["[建元=ja:%%<建元_(前秦)>]1"	,	""	,	"	     365-02"],					# 3198-4
	  ["[太安=ja:%%<太安_(前秦)>]1"	,	"@A"	,	"name=[苻丕];385-08"],					# 哀平帝 3349-33
	  ["[太初=ja:%%<太初_(前秦)>]1"	,	"@A"	,	"name=[苻登];386-10"],					# 高帝 3369-42
	  ["[延初]1"	,			"@A"	,	"name=[苻崇];394-07", "394-10="]			# 3415-10
	],
	[self,
	  "period:[後秦]",
	  ["[白雀]1"	,			"@F"	,	"name=[姚萇];384-04^Chinese0384"],			# 武昭帝 3328-13
	  ["[建初=ja:%%<建初_(後秦)>]1"	,	""	,	"	     386-04"],					# 3364-24
	  ["[皇初]1"	,			"@A"	,	"name=[姚興];394-05"],					# 文桓帝 3414-7
	  ["[弘始]1"	,			""	,	"	     399-09"],					# 3496-30
	  ["[永和=ja:%%<永和_(後秦)>]1"	,	"@A"	,	"name=[姚泓];416-01", "417-08-24="]			# 3685-5
	],
	[self,
	  "period:[西秦]",
	  ["[建義=ja:%%<建義_(西秦)>]1"	,	"@F"	,	"name=[乞伏国仁];385-09^Chinese0239"],			# 宣烈王 3354-38
	  ["[太初=ja:%%<太初_(西秦)>]1"	,	"@A"	,	"name=[乞伏乾帰];388-06", "400-08="],			# 武元王 3384-13
	  ["[更始=ja:%%<更始_(西秦)>]1"	,	""	,	"	     409-07"],					# 3619-14
	  ["[永康=ja:%%<永康_(西秦)>]1"	,	"@A"	,	"name=[乞伏熾磐];412-08"],				# 文昭王 3651-8
	  ["[建弘]1"	,			""	,	"	     420-01"],					# 3732-2
	  ["[永弘]1"	,			"@A"	,	"name=[乞伏暮末];428-05", "431-01="]			# 3801-8
	],
	[self,
	  "period:[前涼]",
	  ["[建興=ja:%%<建興_(晋)>#%.<明王・張寔>]5",		""	,	"name=;317-03-09^Chinese0239",		# 辛卯 (317.03.09)
						"name=[張寔];317-12-21", ""],						# 昭公 己亥
	  ["[永元=ja:%%<建興_(晋)>#%.<成王・張茂>]1"	,	"@A"	,	"name=[張茂];320-06"],			# 成公 2878-9
	  ["[太元=ja:%%<建興_(晋)>#%.<成王・張駿>]1"	,	"@A"	,	"name=[張駿];324-05-14"],		# 文公 甲申 2922-4
	  ["[永楽=ja:%%<建興_(晋)>#%.<成王・張重華>]1"	,	"@A"	,	"name=[張重華];346-05-23",		# 桓公 丙戌 3070-7
										"name=[張耀霊];353-11-18", ""],		# 哀公 丁卯 3136-14
	  ["[和平=ja:%%<和平_(前涼)>]1"	,			"@A"	,	"name=[張祚];354-01"],			# 威王 3137-1
	  ["[太始=ja:%%<建興_(晋)>#%.<冲王・張玄靚>]1"	,	"@A"	,	"name=[張玄靚=ja:%%<張玄ケン>];355-09="], # 冲公 この年の閏月は 6 月のはず
	  ["[建興=ja:%%<建興_(晋)>#%.<冲王・張玄靚>]43"	,	""	,	"	     355-09="],			# 3150-14
	  ["[升平]5"	,					""	,	"	     361-12", "name=[張天錫];363-08=", "376-08-27="], # 3187-17
	  ["[太清?=zh:%%<升平>#%.<前凉悼公张天錫>]1"	,	"@A"	,		    "363-08=", "376-08-27="]	# 悼公 3193-8
	],
	[self,
	  "period:[後涼]",
	  ["[太安=ja:%%<太安_(後涼)>]1"	,	"@F"	,	"name=[呂光];386-12^Chinese0239"],			# 懿武帝 3371-51
	  ["[麟嘉=ja:%%<麟嘉_(後涼)>]1"	,	""	,	"	     389-02"],					# 3387-5
	  ["[龍飛=ja:%%<龍飛_(後涼)>]1"	,	""	,	"	     396-06",					# 3429-11
								"name=[呂紹];399-12", ""],				# 隠王 3504-41
	  ["[咸寧=ja:%%<咸寧 (後涼)>]1"	,	"@A"	,	"name=[呂纂];399-12"],					# 霊帝 3506-41
	  ["[神鼎]1"	,			"@A"	,	"name=[呂隆];401-02", "403-08="]			# 3520-4
	],
	[self,
	"period:[南涼]",
	  ["[太初=ja:%%<太初_(南涼)>]1"	,	"@F"	,	"name=[禿髪烏孤];397-01^Chinese0239"],			# 武王 3439-4
	  ["[建和=ja:%%<建和_(南涼)>]1"	,	"@A"	,	"name=[禿髪利鹿孤];400-01"],				# 康王 3507-5
	  ["[弘昌]1"	,			"@A"	,"name=[禿髪傉檀=ja:%%<禿髪ジョク檀>];402-03"],			# 景王 3542-15
	  ["[嘉平=ja:%%<嘉平_(南涼)>]1"	,	""	,	"	     408-11", "414-06="]			# 3609-15
	],
	[self,
	  "period:[北涼]",
	  ["[神璽]1"	,			"@F"	,	"name=[段業];397-05^Chinese0239"],			# 3455-15
	  ["[天璽=ja:%%<天璽_(北涼)>]1"	,	""	,	"	     399-02"],					# 3488-10
	  ["[永安=ja:%%<永安_(北涼)>]1"	,	"@A"	,	"name=[沮渠蒙遜];401-05"],				# 武宣王 3523-9
	  ["[玄始, alias:元始=ja:%%<玄始>]1",	""	,	"	     412-11^Chinese0412"],			# 3656-19 本年から玄始暦を使用
	  ["[承玄]1"	,			""	,	"	     428"],					# ?
	  ["[義和=ja:%%<義和_(北涼)>]1"	,	""	,	"	     431"],					# 3835-26
	  ["[承和=ja:%%<承和_(北涼)>,alias:永和=ja:%%<承和_(北涼)>]1","@A","name=[沮渠牧犍];433-04", "439-09="]		# 哀王
	],
	[self,
	  "period:[西涼]",
	  ["[庚子=ja:%%<庚子_(西涼)>]1"	,	"@F"	,	"name=[李暠];400-01^Chinese0239"],			# 武昭王 3515-35
	  ["[建初=ja:%%<建初_(西涼)>]1"	,	""	,	"	     405-01"],					# 3580-4
	  ["[嘉興=ja:%%<嘉興_(西涼)>]1"	,	"@A"	,	"name=[李歆];417-02"],					# 3699-4
	  ["[永建=ja:%%<永建_(西涼)>]1"	,	"@A"	,	"name=[李恂];420-07", "420-07="]			# 3737-13
	],
	[self,
	  "period:[夏=ja:%%<夏_(五胡十六国)>]",
	  ["[龍昇,alias:龍升=ja:%%<龍昇>]1.06","@F"	,	"name=[赫連勃勃];407-06^Chinese0239"],			# 武烈帝 3598-10
	  ["[鳳翔=ja:%%<鳳翔_(夏)>]1"	,	""	,	"	     413-03"],					# 3658-7
	  ["[昌武]1"	,			""	,	"	     418-11"],					# 3723-16
	  ["[真興=ja:%%<真興_(夏)>]1"	,	""	,	"	     419-02"],					# 3726-9
	  ["[承光=ja:%%<承光_(夏)>]1"	,	"@A"	,	"name=[赫連昌];425-08"],				# 3776-11
	  ["[勝光]1"	,			"@A"	,	"name=[赫連定];428-02", "431-06="]			# 3800-5
	],
	[self,
	  "period:[代=ja:%%<代_(五胡十六国)>]",
	  ["[建国=ja:%%<建国_(代)>]1"	,	"@F","name=[什翼犍=ja:%%<拓跋什翼ケン>];338-11^Chinese0239", "376-12="]	# 昭成帝 3025-11
	]
      ],
      [self,
	"period:[北朝=ja:%%<南北朝時代_(中国)>#%.<北朝>]",
	[self,
	  "period:[北魏]",
	  ["[登国]1"	,			"@F"	,	"name=[道武帝];386-01-06^Chinese0239"],			# 戊申
	  ["[皇始=ja:%%<皇始_(北魏)>]1"	,	""	,	"	     396-07"],
	  ["[天興=ja:%%<天興_(北魏)>]1"	,	""	,	"	     398-12-02"],				# 己丑
	  ["[天賜=ja:%%<天賜_(北魏)>]1"	,	""	,	"	     404-10-28"],				# 辛巳
	  ["[永興=ja:%%<永興_(明元帝)>]1",	"@A"	,	"name=[明元帝];409-11-18"],				# 壬申
	  ["[神瑞]1"	,			""	,	"	     414-01-01"],				# 辛酉
	  ["[泰常]1"	,			""	,	"	     416-04-05"],				# 壬子
	  ["[始光]1"	,			"@A"	,	"name=[太武帝];424-01"],
	  ["[神䴥=ja:%%<神カ>]1",		""	,	"	     428-02"],					#
	  ["[延和=ja:%%<延和_(北魏)>]1"	,	""	,	"	     432-01-01"],				# 丙午
	  ["[太延]1"	,			""	,	"	     435-01-26"],				# 甲申
	  ["[太平真君]1"	,		""	,	"	     440-06-21"],				# 丁丑
	  ["[正平=ja:%%<正平_(北魏)>]1"	,	""	,	"	     451-06-09"],				# 壬戌 本年以前は景初暦を使用していた模様
	  ["[承平=ja:%%<承平_(北魏)>,alias:永平=ja:%%<承平_(北魏)>]1","@A"	,	"name=[南安王=ja:%%<拓跋余>];452-02^Chinese0412"],	# 3974-2
	  ["[興安=ja:%%<興安_(北魏)>]1"	,"@A","name=[文成帝=ja:%%<文成帝_(北魏)>];452-10-03"],				# 戊申 本年から玄始暦を使用
	  ["[興光]1"	,			""	,	"	     454-07-06"],				# 辛丑
	  ["[太安=ja:%%<太安_(北魏)>]1"	,	""	,	"	     455-06-02"],				# 壬戌
	  ["[和平=ja:%%<和平_(北魏)>]1"	,	""	,	"	     460-01-01"],				# 甲子
	  ["[天安=ja:%%<天安_(北魏)>]1"	,	"@A"	,	"name=[献文帝];466-01-01"],				# 己丑
	  ["[皇興]1"	,			""	,	"	     467-08-29"],				# 戊申
	  ["[延興=ja:%%<延興_(北魏)>]1"	,	"@A"	,	"name=[孝文帝];471-08-20"],				# 丙午
	  ["[承明]1"	,			""	,	"	     476-06-14"],				# 壬申
	  ["[太和=ja:%%<太和_(北魏)>]1"	,	""	,	"	     477-01-01"],				# 乙酉
	  ["[景明]1"	,			"@A"	,	"name=[宣武帝];500-01-05"],				# 乙巳
	  ["[正始=ja:%%<正始_(北魏)>]1"	,	""	,	"	     504-01-19"],				# 丙寅
	  ["[永平=ja:%%<永平_(北魏)>]1"	,	""	,	"	     508-08-16"],				# 丁卯
	  ["[延昌=ja:%%<延昌_(北魏)>]1"	,	""	,	"	     512-04-25"],				# 乙酉
	  ["[熙平]1"	,			"@A"	,	"name=[孝明帝];516-01-01"],				# 戊辰
	  ["[神亀=ja:%%<神亀_(北魏)>]1"	,	""	,	"	     518-02-23"],				# 己酉
	  ["[正光]1"	,			""	,	"	     520-07-19", "523-01^Chinese0523", ""],	# 辛卯 正光４年から正光暦を使用
	  ["[孝昌]1"	,			""	,	"	     525-06-10"],				# 癸未
	  ["[武泰=ja:%%<武泰_(北魏)>]1"	,	""	,	"	     528-01-08"],				# 丙寅
	  ["[建義=ja:%%<建義_(北魏)>]1"	,	"@A"	,	"name=[孝荘帝];528-04-14"],				# 辛丑
	  ["[永安=ja:%%<永安_(北魏)>]1"	,	""	,	"	     528-09-21"],				# 乙亥
	  ["[建明=ja:%%<建明_(北魏)>]1"	,	"@A"	,	"name=[東海王=ja:%%<元曄>];530-10-30"],			# 壬申
	  ["[普泰]1"	,	"@A"	,	"name=[節閔帝=ja:%%<元恭>];531-02-29"],					# 己巳
	  ["[中興=ja:%%<中興_(北魏)>]1"	,	"@A"	,	"name=[後廃帝=ja:%%<元朗_(北魏)>];531-10-06"],		# 壬寅
	  ["[太昌]1"	,	"@A"	,	"name=[孝武帝=ja:%%<孝武帝_(北魏)>];532-04-25"],			# 戊子
	  ["[永興=ja:%%<永興_(孝武帝)>]1",	""	,	"	     532-12-28"],				# 丁亥
	  ["[永熙=ja:%%<永熙_(北魏)>]1"	,	""	,	"	     532-12-28", "535-01-01"]
	],
	[self,
	  "period:[東魏]",
	  ["[天平=ja:%%<天平_(東魏)>]1"	,	"@A"	,	"name=[孝静帝];534-10-17^Chinese0523"],			# 丙寅 北魏に引続き正光暦を使用
	  ["[元象]1"	,			""	,	"	     538-01-07"],				# 丁卯
	  ["[興和=ja:%%<興和_(東魏)>]1"	,	""	,	"	     539-11", "540-01^Chinese0540",""],		# 興和２年から興和暦を使用
	  ["[武定=ja:%%<武定_(東魏)>]1"	,	""	,	"	     543-01-01", "550-05-10"]			# 壬戌
	],
	[self,
	  "period:[北斉]",
	  ["[天保=ja:%%<天保_(北斉)>]1"	,	"@F"	,	"name=[文宣帝];550-05-10^Chinese0540"],			# 戊午 天保２年から天保暦を使用
	  ["[乾明]1"	,			"@A"	,"name=[廃帝=ja:%%<廃帝_(北斉)>];560-01-01"],			# 癸丑
	  ["[皇建=ja:%%<皇建_(北斉)>]1"	,	"@A"	,	"name=[孝昭帝];560-08-03"],				# 壬午
	  ["[太寧=ja:%%<太寧_(北斉)>,alias:大寧=ja:%%<太寧_(北斉)>]1","@A","name=[武成帝];561-11-11"],			# 癸丑 5216-23
	  ["[河清]1"	,			""	,	"	     562-04-06"],				# 乙巳
	  ["[天統=ja:%%<天統_(北斉)>]1"	,	"@A"	,"name=[後主=ja:%%<後主_(北斉)>];565-04-24"],			# 丙子
	  ["[武平=ja:%%<武平_(北斉)>]1"	,	""	,	"	     570-01-01"],				# 乙酉
	  ["[隆化]1"	,			""	,	"	     576-12-13", "577-01-01"],			# 丁巳 5361
	  ["[徳昌=ja:%%<徳昌_(北斉)>]1"	,	""	,"name=[安徳王=ja:%%<高延宗>];576-12-14", "576-12-17="],	# 戊午, 辛酉 5362
	  ["[承光=ja:%%<承光_(北斉)>]1"	,"@A",	"name=[幼主=ja:%%<幼主_(北斉)>];577-01-01", "577-03="]			# 乙亥
	],
	[self,
	  "period:[西魏]",
	  ["[大統]1"	,	"@A"	,	"name=[文帝=ja:%%<文帝_(西魏)>];535-01-01^Chinese0523"],		# 戊申
	  ["[<廃帝>=ja:%%<廃帝_(西魏)>]1.01",	"@A"	,	"name=[廃帝=ja:%%<廃帝_(西魏)>];552-01"],
	  ["[<恭帝>=ja:%%<恭帝_(西魏)>]1.01",	"@A"	,	"name=[恭帝=ja:%%<恭帝_(西魏)>];554-01", "556-12-30"]
	],
	[self,
	  "period:[北周]",
	  ["[<孝閔帝>=ja:%%<孝閔帝>]1.12.30",	"@F"	,	"name=[孝閔帝];556-12-30^Chinese0523"],			# 庚子 北周が西魏に代わる
	  ["[<明帝>=ja:%%<明帝_(北周)>]1.09.28","@A"	,"name=[明帝=ja:%%<明帝_(北周)>];557-09-28"],			# 甲子
	  ["[武成=ja:%%<武成_(北周)>]1"	,	""	,	"	     559-08-14"],				# 乙亥 or 己亥 5190-30
	  ["[保定=ja:%%<保定_(北周)>]1"	,	"@A"	,"name=[武帝=ja:%%<武帝_(北周)>];561-01-01"],			# 戊申
	  ["[天和=ja:%%<天和_(北周)>]1"	,	""	,	"	     566-01-05"],				# 癸未 本年から天和暦を使用
	  ["[建徳=ja:%%<建徳_(北周)>]1"	,	""	,	"	     572-03-15"],				# 丁巳
	  ["[宣政]1"	,			""	,	"	     578-03-25"],				# 壬辰
	  ["[大成=ja:%%<大成_(北周)>]1"	,	"@A"	,"name=[宣帝=ja:%%<宣帝_(北周)>];579-01-01"],			# 癸巳
	  ["[大象=ja:%%<大象_(北周)>]1"	,	"@A"	,	"name=[静帝];579-02-19"],				# 辛巳 本年から大象暦を使用
	  ["[大定=ja:%%<大定_(北周)>]1"	,	""	,	"	     581-01-01", "581-02-13"]			# 壬午
	]
      ],
      [self,
	"period:[隋]",
	["[開皇]1"	,			"@F","name=[文帝=ja:%%<楊堅>];581-02-13^Chinese0523"],			# 甲子 開皇４年から開皇暦を使用
	["[仁寿=ja:%%<仁寿_(隋)>]1"	,	""	,	"	     601-01-01"],				# 乙酉 開皇１７年から大業暦を使用
	["[大業]1"	,			"@A"	,	"name=[煬帝];605-01-01", "618-03-10="],			# 壬辰
	["[皇泰]1"	,			"@A","name=[恭帝=ja:%%<恭帝トウ>];618-05-24^ChineseTwin?note=Chinese::大業暦", "619-04-08"],	# 戊辰 5791-26
	[self,
	  "period:[鄭<王世充>=ja:%%<王世充>]",
	  ["[開明=ja:%%<開明_(王世充)>]1"	,"@F"	,"name=[王世充];619-04-08^ChineseTwin?note=Chinese::大業暦", "621-05-{1丁卯}"]	# 丙午 5853-45, 621-05-09=は丁卯
	],
	[self,
	  "period:[梁<蕭銑>=ja:%%<蕭銑>]",
	  ["[鳳鳴, alias:鳴鳳=ja:%%<鳳鳴>]1",	"@F",	"name=[蕭銑];617-10^ChineseTwin?note=Chinese::大業暦", "621-10-21="]		# 5761-17
	],
	[self,
	  "period:[梁<梁師都>=ja:%%<梁師都>]",
	  ["[永隆=ja:%%<永隆_(梁師都)>]1"	,"@F"	,"name=[梁師都];617-03^ChineseTwin?note=Chinese::大業暦", "628-04-26="]	# 5725-9
	],
	[self,
	  "period:[梁<沈法興>=ja:%%<沈法興>]",
	  ["[延康=ja:%%<延康_(沈法興)>]1"	,"@F"	,"name=[沈法興];619-08^ChineseTwin?note=Chinese::大業暦", "620-12="]		# 5863-77
	],
	[self,
	  "period:[元興<操師乞>=ja:%%<操師乞>]",
	  ["[始興=ja:%%<始興_(操師乞))>,alias:天成=ja:%%<始興_(操師乞))>]1","@F","name=[操師乞];616-12-01^ChineseTwin?note=Chinese::大業暦", "616-12="]	# 『隋書』十二月癸未 Wikipedia
	],
	[self,
	  "period:[呉<李子通>=ja:%%<李子通>]",
	  ["[明政=ja:%%<明政_(李子通)>]1"	,"@F"	,"name=[李子通];619-08^ChineseTwin?note=Chinese::大業暦", "621-11="]		# 5863-77
	],
	[self,
	  "period:[楚<林士弘>=ja:%%<林士弘>]",
	  ["[太平=ja:%%<太平_(林士弘)>]1"	,"@F"	,"name=[林士弘];616-12-10^ChineseTwin?note=Chinese::大業暦", "622-10-21="]	# 壬辰 5712-16
	],
	[self,
	  "period:[楚<朱粲>=ja:%%<朱粲>]",
	  ["[昌達]1"	,			"@F"	,"name=[朱粲];618-10^ChineseTwin?note=Chinese::大業暦", "619-02?"]		# 5817-22
	],
	[self,
	  "period:[夏<竇建徳>=ja:%%<竇建徳>]",
	  ["[丁丑=ja:%%<丁丑_(夏)>]1",	"@F",	"name=[竇建徳];617-01-05^ChineseTwin?note=Chinese::大業暦"],				# 丙辰 5718-2
	  ["[五鳳=ja:%%<五鳳_(夏)>]1",	"",	"	     618-07", "621-05-{1丁卯}"]					# 5805-54, 621-05-09=は丁卯
	],
	[self,
	  "period:[漢東<劉黒闥>=ja:%%<劉黒闥>]",
	  ["[天造]1"	,			"@F"	,"name=[劉黒闥];622-01^ChineseTwin?note=Chinese::大業暦", "623-01-{1庚辰}"]	# 5942-1, 623-01-03=は庚辰
	],
	[self,
	  "period:[魏<李密>=ja:%%<李密_(隋)>]",
	  ["[永平=ja:%%<永平_(李密)>]1"	,	"@F"	,"name=[李密=ja:%%<李密_(隋)>];617-02-19^ChineseTwin?note=Chinese::大業暦", "618-09="] # 庚子 5720-7
	],
	[self,
	  "period:[許<宇文化及>=ja:%%<宇文化及>]",
	  ["[天寿=ja:%%<天寿_(宇文化及)>]1"	,"@F"	,"name=[宇文化及];618-09^ChineseTwin?note=Chinese::大業暦", "619-02?"]	# 5761-17
	],
	[self,
	  "period:[涼<李軌>=ja:%%<李軌>]",
	  ["[安楽=ja:%%<安楽_(年号)>]1"	,	"@F"	,"name=[李軌];618-11-04^ChineseTwin?note=Chinese::大業暦", "619-05-{1辛巳}"]	# 乙巳 5820-28, 619-05-13=は辛巳
	],
	[self,
	  "period:[燕<高開道>=ja:%%<高開道>]",
	  ["[始興=ja:%%<始興_(高開道)>]1"	,"@F"	,"name=[高開道];618-12^ChineseTwin?note=Chinese::大業暦", "620-10-{1己酉}"]	# 5833-47, 620-10-19=は己酉
	],
	[self,
	  "period:[乗<高曇晟>=ja:%%<高曇晟>]",
	  ["[法輪=ja:%%<法輪_(高曇晟)>]1"	,"@F"	,"name=[高曇晟];618-12^ChineseTwin?note=Chinese::大業暦", "619-02="]		# 5833-47
	],
	[self,
	  "period:[定楊<劉武周>=ja:%%<劉武周>]",
	  ["[天興=ja:%%<天興_(劉武周)>]1"	,"@F"	,"name=[劉武周];617-03-17^ChineseTwin?note=Chinese::大業暦", "620-04="]	# 丁卯 5725-8
	],
	[self,
	  "period:[西秦<薛挙>=zh:%%<薛舉>]",
	  ["[秦興]1"	,			"@F"	,"name=[薛挙=zh:%%<薛舉>];617-04-03^ChineseTwin?note=Chinese::大業暦", "618-11-08="]	# 癸未 5725-11
	],
	[self,
	  "period:[永楽<郭子和>=ja:%%<郭子和>]",
	  ["[正平=ja:%%<正平_(郭子和)>, alias:丑平=]1","@F","name=[郭子和];617-03^ChineseTwin?note=Chinese::大業暦", "618-12="]	# 5725-10
	],
	[self,
	  "period:[<曹武徹>]",
	  ["[通聖]1"	,			"@F"	,"name=[曹武徹];617-12-11^ChineseTwin?note=Chinese::大業暦", "617-12="]	# 隋書による
	],
	[self,
	  "period:[隋<恭帝>=ja:%%<恭帝侑>]",
	  ["[義寧]1"	,			"@A"	,"name=[恭帝ja:%%<恭帝侑>];617-11-15^Chinese0523", "618-05-20"]	# 壬戌
	]
      ],
      [self,
	"period:[唐]",
	["[武徳]1"	,			"@F"	,	'name=[高祖=ja:%%<李淵>];618-05-20^Chinese#{A:0523}'], # 甲子 武徳２年から戊寅元暦を使用
	["[貞観=ja:%%<貞観_(唐)>]1"	,	"@A"	,	"name=[太宗=ja:%%<太宗_(唐)>];627-01-01"],		# 乙酉
	["[永徽]1"	,			"@A"	,	"name=[高宗=ja:%%<高宗_(唐)>];650-01-01"],		# 辛丑
	["[顕慶]1"	,			""	,	"	     656-01-07"],				# 壬申
	["[龍朔]1"	,			""	,	"	     661-02-30"],				# 乙未
	["[麟徳]1"	,			""	,	"	     664-01-01"],				# 己酉 麟徳２年から麟徳暦を使用
	["[乾封]1"	,			""	,	"	     666-01-05"],				# 壬申
	["[総章]1"	,			""	,	"	     668-03-06"],				# 庚寅
	["[咸亨]1"	,			""	,	"	     670-03-01"],				# 甲戌
	["[高宗#上元=ja:%%<上元_(唐高宗)>]1",	""	,	"	     674-08-15"],				# 壬辰
	["[儀鳳]1"	,			""	,	"	     676-11-08"],				# 壬申
	["[調露]1"	,			""	,	"	     679-06-03"],				# 辛亥
	["[永隆=ja:%%<永隆_(唐)>]1"	,	""	,	"	     680-08-23"],				# 乙丑
	["[開耀]1"	,			""	,	"	     681-09-30"],				# 乙丑
	["[永淳]1"	,			""	,	"	     682-02-19"],				# 癸未
	["[弘道=ja:%%<弘道_(唐)>]1"	,	""	,	"	     683-12-04"],				# 丁巳 本月４日丁巳高宗崩ず、同甲子中宗即位
	["[嗣聖]1"	,			"@A"	,	"name=[中宗=ja:%%<中宗_(唐)>];684-01-01"],		# 甲申
	["[文明=ja:%%<文明_(唐)>]1"	,	"@A"	,	"name=[睿宗=ja:%%<睿宗_(唐)>];684-02-07"],		# 己未 戊午中宗を廃する、旧唐書己未改元
	["[光宅]1"	,			""	,	"	     684-09-06"],				# 甲寅
	["[垂拱]1"	,			""	,	"name=[武太后=ja:%%<武則天>];685-01-01"],		# 丁未
	["[永昌=ja:%%<永昌_(唐)>]1"	,	""	,	"	     689-01-01"],				# 乙卯
	["[載初]1*11.01",			""	,	'	     690*11-01^Chinese#{B:0689}', "690-09-09"], # 庚辰 本年から１１月を歳首とする
	[self,
	  "period:[周=zh:%%<武周>]",
	  ["[天授=ja:%%<天授_(武周)>]1*",	"@F"	,	'name=[武則天];690-09-09^Chinese#{B:0689}'],		# 壬午
	  ["[如意=ja:%%<如意_(元号)>]1*",	""	,	"	     692-04-01"],				# 丙申
	  ["[長寿=ja:%%<長寿_(武周)>]1*",	""	,	"	     692-09-09"],				# 庚子
	  ["[延載]1*"	,			""	,	"	     694-05-11"],				# 甲午
	  ["[証聖]1*11.01",			""	,	"	     695*11-01"],				# 辛巳
	  ["[天冊万歳]1.09.09"	,		""	,	"	     695-09-09"],				# 甲寅
	  ["[万歳登封]1*12.11"	,		""	,	"	     696*12-11"],				# 甲申
	  ["[万歳通天]1.03.16"	,		""	,	"	     696-03-16"],				# 丁巳
	  ["[神功]1*"	,			""	,	"	     697-08-29"],				# 壬辰 コンサイス世界年表では１年ずれる
	  ["[聖暦]1*11.01",			""	,	"	     698*11-01"],				# 甲子
	  ["[久視]1*"	,			""	,	"	     700-05-05",				# 癸丑
								'	     700-10-01^Chinese#{A:0523}',""],		# 本年で歳首を元にもどす
	  ["[大足]1"	,			""	,	"	     701-01-03"],				# 丁丑
	  ["[長安=ja:%%<長安_(元号)>]1"	,	""	,	"	     701-10-22"],				# 辛酉
	  ["[神龍=ja:%%<神龍_(唐)>]1"	,	""	,	"	     705-01-01", "705-02-04"]			# 壬午
	],
	["[神龍=ja:%%<神龍_(唐)>]1.02.04",	""	,	'name=[中宗=ja:%%<中宗_(唐)>];705-02-04^Chinese#{A:0523}'], # 甲寅
	["[景龍=ja:%%<景龍_(唐)>]1"	,	""	,	"	     707-09-05"],				# 庚子
	["[唐隆]1"	,			"@A"	,	"name=[殤帝=ja:%%<殤帝_(唐)>];710-06-04"],		# 甲申
	["[景雲=ja:%%<景雲_(唐)>]1"	,	""	,	"name=[睿宗=ja:%%<睿宗_(唐)>];710-07-20"],		# 己巳
	["[太極=ja:%%<太極_(唐)>]1"	,	""	,	"	     712-01-19"],				# 己丑
	["[延和=ja:%%<延和_(唐)>]1"	,	""	,	"	     712-05-13"],				# 辛巳
	["[先天=ja:%%<先天_(唐)>]1"	,	"@A"	,	"name=[玄宗=ja:%%<玄宗_(唐)>];712-08-07"],		# 甲辰
	["[開元]1"	,			""	,	"	     713-12-01"],				# 庚寅 開元１７年から大衍暦を使用
	["[天宝=ja:%%<天宝_(唐)>]1"	,	""	,	"	     742-01-01"],				# 丁未
	["[至徳=ja:%%<至徳_(唐)>]1"	,	"@A"	,	"name=[粛宗=ja:%%<粛宗_(唐)>];756-07-12"],		# 甲子
	["[乾元=ja:%%<乾元_(唐)>]1"	,	""	,	"	     758-02-05"],				# 丁未
	["[粛宗#上元=ja:%%<上元_(唐粛宗)>]1",	""	,	"	     760-04=19"],				# 己卯
	["[<粛宗>=https://zh.wikisource.org/wiki/%%<去上元年號大赦文>]1"	,"","name=[粛宗=ja:%%<粛宗_(唐)>];761-09-21", '762*11^Chinese#{C:0761}', ""],	# 壬寅
	["[宝応]1"	,	"@A"	,	'name=[代宗=ja:%%<代宗_(唐)>];762-04-15^Chinese#{A:0523}'],		# 甲子 本年から五紀暦を使用
	["[広徳=ja:%%<広徳_(唐)>]1"	,	""	,	"	     763-07-11"],				# 壬子
	["[永泰=ja:%%<永泰_(唐)>]1"	,	""	,	"	     765-01-01"],				# 癸巳
	["[大暦]1"	,			""	,	"	     766-11-12"],				# 甲子
	["[建中=ja:%%<建中_(唐)>]1"	,	"@A"	,	"name=[徳宗=ja:%%<徳宗_(唐)>];780-01-01"],		# 丁卯
	["[興元]1"	,			""	,	"	     784-01-01"],				# 癸酉 本年から正元暦を使用
	["[貞元=ja:%%<貞元_(唐)>]1"	,	""	,	"	     785-01-01"],				# 丁酉
	["[永貞=ja:%%<永貞_(唐)>]1"	,	"@A"	,	"name=[順宗=ja:%%<順宗_(唐)>];805-08-05"],		# 辛丑
	["[元和=ja:%%<元和_(唐)>]1"	,	"@A"	,	"name=[憲宗=ja:%%<憲宗_(唐)>];806-01-02"],		# 丁卯 元和２年から観象暦を使用
	["[長慶]1"	,			"@A"	,	"name=[穆宗=ja:%%<穆宗_(唐)>];821-01-{1辛丑}"],		# 辛丑 長慶２年から宣明暦を使用
	["[宝暦=ja:%%<宝暦_(唐)>]1"	,	"@A"	,	"name=[敬宗];825-01-07"],				# 辛亥
	["[大和=ja:%%<大和_(唐)>,alias:太和=ja:%%<大和_(唐)>]1"	,"@A","name=[文宗=ja:%%<文宗_(唐)>];827-02-13"],	# 乙巳
	["[開成=ja:%%<開成_(唐)>]1"	,	""	,	"	     836-01-01"],				# 辛丑
	["[会昌]1"	,			"@A"	,	"name=[武宗=ja:%%<武宗_(唐)>];841-01-09"],		# 辛巳
	["[大中]1"	,			"@A"	,	"name=[宣宗=ja:%%<宣宗_(唐)>];847-01-17"],		# 甲寅
	["[咸通]1"	,			"@A"	,	"name=[懿宗=ja:%%<懿宗_(唐)>];860-11-02"],		# 丁丑
	["[乾符]1"	,			"@A"	,	"name=[僖宗=ja:%%<僖宗_(唐)>];874-11-05"],		# 庚寅
	["[広明=ja:%%<広明_(唐)>]1"	,	""	,	"	     880-01-01"],				# 乙卯
	["[中和=ja:%%<中和_(唐)>]1"	,	""	,	"	     881-07-11"],				# 丁巳
	["[光啓]1"	,			""	,	"	     885-03-14"],				# 己巳
	["[文徳=ja:%%<文徳_(唐)>]1"	,	""	,	"	     888-02-22"],				# 庚寅
	["[龍紀=ja:%%<龍紀_(唐)>]1"	,	"@A"	,	"name=[昭宗=ja:%%<昭宗_(唐)>];889-01-01"],		# 癸巳
	["[大順=ja:%%<大順_(唐)>]1"	,	""	,	"	     890-01-01"],				# 戊子
	["[景福=ja:%%<景福_(唐)>]1"	,	""	,	"	     892-01-21"],				# 丙寅 景福２年から崇玄暦を使用
	["[乾寧]1"	,			""	,	"	     894-01-01"],				# 乙丑
	["[光化]1"	,			""	,	"	     898-08-27"],				# 甲子
	["[天復=ja:%%<天復_(唐)>]1"	,	""	,	"	     901-04-25"],				# 丁丑
	["[天祐=ja:%%<天祐_(唐)>,alias:天佑=ja:%%<天祐_(唐)>]1","@A","	     904-04=11",				# 乙巳
								"name=[哀帝];904-08-15", "907-04-21="],			# 丙午
	[self,
	  "period:[燕=zh:%%<燕_(安史之乱)>]",
	  ["[聖武=ja:%%<聖武_(安禄山)>]1",	"@F"	,	"name=[安禄山];756-01-01^Chinese0523"],			# 乙卯朔 6951-1
	  ["[載初=ja:%%<載初_(安慶緒)>]1",	"@A"	,	"name=[安慶緒];757-01-06"],				# 乙卯 7013-2
	  ["[天成=ja:%%<天成_(安慶緒)>]1",	""	,	"	     757-10"],					# 丁卯? 23日 7042-12
	  ["[順天=ja:%%<順天_(史思明)>]1",	"@A"	,	"name=[史思明];759-04"],				# 甲辰? 7075-13
	  ["[応天=ja:%%<応天_(史思明)>]1",	""	,	"	     761-01-17"],				# 癸卯 7103-1
	  ["[顕聖]1"	,			"@A"	,	"name=[史朝義];761-03-13", "763-01="]			# 戊戌? 7108-8
	],
	[self,
	  "period:[斉=zh:%%<齐_(黄巢)>]",
	  ["[王霸]1"	,			"@F"	,	"name=[黄巣];878-02^Chinese0523"],			# 8200-6
	  ["[金統]1"	,			""	,	"	     880-12-13", "884-06-17="]			# 壬辰 8240-10
	]
      ],
      [self,
	"period:[五代十国=ja:%%<五代十国時代>, zh:五代十國=zh:%%<五代十国>]",
	[self,
	  "period:[呉=ja:%%<呉_(十国)>]",
	  ["[天祐=ja:%%<天祐_(唐)>,alias:天佑=ja:%%<天祐_(唐)>]4","","name=[楊渥];907-04-22^Chinese0523",		# 戊辰 唐滅亡後も、五代を認めず旧年号を使用 4.04.22
								"name=[楊隆演];908-05-09", ""],				# 己卯 8697-24
	  ["[武義]1"	,			"@F"	,	"	     919-04-01"],				# 高祖 戊戌朔 8843-8 以前は唐の年号を使用
	  ["[順義]1"	,			"@A"	,	"name=[楊溥];921-02"],					# 睿帝 8864-5
	  ["[乾貞=ja:%%<乾貞_(十国呉)>]1",	""	,	"	     927-11-17"],				# 甲子 出典は?
	  ["[大和=ja:%%<大和_(十国呉)>]1",	""	,	"	     929-11-03"],				# 壬辰 9035-32
	  ["[天祚]1"	,			""	,	"	     935-09-04", "937-10-05"]			# 丙申 9133-27
	],
	[self,
	  "period:[南唐]",
	  ["[昇元]1"	,	"@F"	,	"name=[徐知誥=ja:%%<李ベン>];937-10-05^Chinese0523"],			# 烈祖 甲申 9182-49
	  ["[保大=ja:%%<保大_(南唐)>]1"	,"@A","name=[李璟=ja:%%<李ケイ_(南唐)>];943-03-01", "947-01^Chinese0956", ""],	# 元宗 己卯朔 9246-6 950-1の間楚が使用
	  ["[中興=ja:%%<中興_(南唐)>]1"	,	""	,	"	     958-01-03"],				# 乙酉 9577-2 新五代史の表にない年号
	  ["[交泰]1"	,			""	,	"	     958-03-06", "958-04="]			# 丁亥 9579-16
	],
	[self,
	  "period:[呉越]",
	  ["[天宝=ja:%%<天宝_(呉越)>]1"	,	"@F"	,	"name=[銭鏐];908^Chinese0523", "912="],			# 武粛王 漢和辞典に見える
	  ["[宝大]1"	,			""	,	"	     924"],					# 〃
	  ["[宝正]1"	,			""	,	"	     926", "932-03-28="]
	],
	[self,
	  "period:[閩=ja:%%<ビン_(十国)>]",
	  ["[龍啓]1"	,			"@F"	,	"name=[王延鈞];933-01^Chinese0523"],			# 太宗 以前は五代の年号を使用 9081-2
	  ["[永和=ja:%%<永和_(ビン)>]1"	,	""	,	"	     935-01-01"],				# 丙申朔 9127-1
	  ["[通文]1"	,			"@A"	,	"name=[王昶=ja:%%<王継鵬>];936-03"],			# 康宗 9140-7
	  ["[永隆=ja:%%<永隆_(ビン)>]1"	,	"@A"	,	"name=[王曦=ja:%%<王延羲>];939-07", "943-02"],		# 景宗 9206-27
	  [self,
	    "period:[殷=ja:%%<王延政>]",
	    ["[天徳=ja:%%<天徳_(ビン)>]1",	"@A"	,	"name=[王延政];943-02^Chinese0523", "945-01"]		# 9247-4
	  ],
	  ["[天徳=ja:%%<天徳_(ビン)>]3.01",	""	,	"	     945-01^Chinese0523", "945-08-24="]		# 9283-4
	],
	[self,
	  "period:[南漢]",
	  ["[乾亨=ja:%%<乾亨_(南漢)>]1"	,"@F","name=[劉巖=ja:%%<劉ゲン>,alias:劉龑=ja:%%<劉ゲン>];917-07^Chinese0523"],	# 高祖 8817-4
	  ["[白龍=ja:%%<白龍_(南漢)>]1"	,	""	,	"	     925"],
	  ["[大有]1"	,			""	,	"	     928"],
	  ["[光天=ja:%%<光天_(南漢)>]1"	,	"@A"	,	"name=[劉玢];942-04"],					# 殤帝 9236-11
	  ["[応乾]1"	,			"@A"	,	"name=[劉晟];943-03-09"],				# 中宗 丙戌明旦 9249-8
	  ["[乾和]1"	,			""	,	"	     943-11-13", "947-01^Chinese0956", ""],	# 丁亥 9255-28
	  ["[大宝=ja:%%<大宝_(南漢)>]1"	,	"@A"	,	"name=[劉鋹];958-08-03", "971-02-05="]			# 後主 辛巳 9586-35
	],
	[self,
	  "period:[桀燕]",
	  ["[応天=ja:%%<応天_(桀燕)>]1"	,	"@F"	,	"name=[劉守光];911-08-13^Chinese0523", "914-01-15="]	# 甲子 8745-15
	],
	[self,
	  "period:[岐]",
	  ["[天祐=ja:%%<天祐_(唐)>,alias:天佑=ja:%%<天祐_(唐)>]4","@A","name=[李茂貞];907-04-22^Chinese0523", "924-01-14="] # 戊辰 唐滅亡後も、五代を認めず旧年号を使用 4.04.22
	],
	[self,
	  "period:[前蜀]",
	  ["[天復=ja:%%<天復_(唐)>]4",		""	,"name=[王建=ja:%%<王建_(前蜀)>];904-04=11^Chinese0523"],	# 乙巳 高祖 4.04=11
	  ["[武成=ja:%%<武成_(前蜀)>]1"	,	"@F"	,	"	     908-01-09"],				# 辛巳 8688-3
	  ["[永平=ja:%%<永平_(前蜀)>]1"	,	""	,	"	     911-01-01"],				# 丙戌 8730-37
	  ["[通正]1"	,			""	,	"	     916-01-01"],				# 丙辰 8799-22
	  ["[天漢=ja:%%<天漢_(前蜀)>]1"	,	""	,	"	     917-01-01"],				# 辛亥 8808-31
	  ["[光天=ja:%%<光天_(前蜀)>]1"	,	""	,	"	     918-01-01"],				# 乙亥 8822-14
	  ["[乾徳=ja:%%<乾徳_(前蜀)>]1"	,	"@A"	,	"name=[王衍];919-01-01"],				# 庚午 後主 8838-34
	  ["[咸康=ja:%%<咸康_(前蜀)>]1"	,	""	,	"	     925-01-01", "925-11-27="]			# 甲午 8929-75
	],
	[self,
	  "period:[後蜀=ja:%%<後蜀_(十国)>]",
	  ["[応順]1",				"@F"		,"name=[孟知祥];934-01=^Chinese0523"],			# 高祖 9102-13 1.01=
	  ["[明徳=ja:%%<明徳_(後蜀)>]1"	,	""	,	"	     934-04-12",				# 辛巳 9117-16
								"name=[孟昶];934-07-28", ""],				# 後主 丙寅 9123-48
	  ["[広政]1"	,			""	,	"	     938-01-01", "947-01^Chinese0956", "965-01-07="] # 戊申 9184-53 明徳は多分間違い
	],
	[self,
	  "period:[後梁]",
	  ["[開平=ja:%%<開平_(後梁)>]1"	,	"@F"	,	"name=[太祖=ja:%%<朱全忠>];907-04-22^Chinese0523"],	# 前月27日甲辰唐帝退位,18日甲子即位,戊辰改元
	  ["[乾化]1"	,			""	,	"	     911-05-01"],				# 甲申
	  ["[鳳歴=ja:%%<鳳歴_(五代後梁)>,alias:鳳暦=ja:%%<鳳歴_(五代後梁)>]1","@A",	"name=[郢王=ja:%%<朱友珪>];913-01-21"],	# 甲子 8765-2
	  ["[乾化]3.02.17",			"@A"	,	"name=[末帝=ja:%%<朱友貞>];913-02-17"],			# 庚寅 8767-6
	  ["[貞明=ja:%%<貞明_(五代後梁)>]1",	""	,	"	     915-11-09"],				# 乙丑
	  ["[龍徳=ja:%%<龍徳_(五代後梁)>]1",	""	,	"	     921-05-01", "923-10-08="]			# 丙戌, 戊寅
	],
	[self,
	  "period:[晋=ja:%%<李克用>]",
	  ["[天祐=ja:%%<天祐_(唐)>,alias:天佑=ja:%%<天祐_(唐)>]4","","name=[李克用];907-04-22^Chinese0523",	# 戊辰 唐滅亡後も、五代を認めず旧年号を使用 4.04.22
								"name=[荘宗=ja:%%<李存勗>];908-01-19", "923-04-25"]	# 辛卯 8688-4
	],
	[self,
	  "period:[後唐]",
	  ["[同光]1"	,			"@F"	,	"name=[荘宗=ja:%%<李存勗>];923-04-25^Chinese0523"],	# 己巳 天祐２０年に戻した後、ただちに改元
	  ["[天成=ja:%%<天成_(後唐)>]1"	,	"@A"	,	"name=[明宗=ja:%%<李嗣源>];926-04-28"],			# 甲寅
	  ["[長興]1"	,			""	,	"	     930-02-21"],				# 乙卯
	  ["[応順]1"	,			"@A"	,	"name=[閔帝=ja:%%<李従厚>];934-01-07"],			# 戊寅
	  ["[清泰]1"	,			"@A"	,	"name=[廃帝=ja:%%<李従珂>];934-04-16", "936-11-14"]	# 乙酉
	],
	[self,
	  "period:[後晋]",
	  ["[天福=ja:%%<天福_(後晋)>]1"	,	"@F"	,	"name=[高祖=ja:%%<石敬トウ>];936-11-14^Chinese0523",	# 己亥 長興７年に戻した後、ただちに改元
								"name=[出帝=ja:%%<石重貴>];942-06-13", ""],		# 乙丑 天福４年から調元暦を使用
	  ["[開運=ja:%%<開運_(後晋)>]1"	,	""	,	"	     944-07-01", "946-12-16="]			# 辛未
	],
	[self,
	  "period:[後漢=ja:%%<後漢_(五代)>]",
	  ["[天福=ja:%%<天福_(後晋)>]12",	""	,	"name=[高祖=ja:%%<劉知遠>];947-02-15^Chinese0956"],	# 辛未 12.02.15
	  ["[乾祐=ja:%%<乾祐_(五代後漢)>]1",	"@F"	,	"	     948-01-05",				# 乙卯
								"name=[隠帝=ja:%%<劉承祐>];948-02-01", "951-01-05="]	# 辛巳, 丁卯
	],
	[self,
	  "period:[北漢]",
	  ["[乾祐=ja:%%<乾祐_(五代後漢)>]4",	""	,	"name=[劉旻=ja:%%<劉崇>];951-01-16^Chinese0956",	# 世祖 戊寅 9453-13 4.01.16
								"name=[劉鈞];954-11", ""],				# 睿宗 9520-15
	  ["[天会=天会_(北漢)]1"	,	""	,	"	     957-07",					# 9571-20 ?
								"name=[劉継恩];968-07-27",				# 廃帝 戊申 115-30
								"name=[劉継元];968-09", ""],				# 英武帝 116-38
	  ["[広運=ja:%%<広運_(北漢)>]1"	,	""	,	"	     974-01", "979-05-06="]			# 178-3
	],
	[self,
	  "period:[後周]",
	  ["[広順]1"	,			"@F"	,	"name=[太祖=ja:%%<郭威>];951-01-05^Chinese0956"],	# 丁卯
	  ["[顕徳]1"	,			"@A"	,	"	     954-01-01",				# 丙子
								"name=[世宗=ja:%%<柴栄>];954-01-21",			# 丙申 顕徳３年から欽天暦を使用
								"name=[恭帝=ja:%%<柴宗訓>];959-06-20", "960-01-04"]	# 甲午
	]
      ],
      [self,
	"period:[宋=ja:%%<宋_(王朝)>]",
	[self,
	  "period:[北宋]",
	  ["[建隆]1"	,			"@F"	,	"name=[太祖=ja:%%<趙匡胤>];960-01-04^Chinese0956"],	# 甲辰 前日陳橋の変
	  ["[乾徳=ja:%%<乾徳_(宋)>]1"	,	""	,	"	     963-11-16"],				# 甲子 乾徳２年から応天暦を使用
	  ["[開宝]1"	,			""	,	"	     968-11-24"],				# 癸卯
	  ["[太平興国]1",			"@A"	,	"name=[太宗=ja:%%<太宗_(宋)>];976-12-22"],		# 甲寅 太平興国８年から乾元暦を使用
	  ["[雍熙]1"	,			""	,	"	     984-11-21"],				# 丁卯
	  ["[端拱]1"	,			""	,	"	     988-01-17"],				# 乙亥
	  ["[淳化]1"	,			""	,	"	     990-01-01"],				# 戊寅
	  ["[至道]1"	,			""	,	"	     995-01-01"],				# 戊申
	  ["[咸平]1"	,			"@A"	,	"name=[真宗=ja:%%<真宗_(宋)>];998-01-01"],		# 辛酉 咸平４年から儀天暦を使用
	  ["[景徳]1"	,			""	,	"	     1004-01-01"],				# 丙戌
	  ["[大中祥符]1",			""	,	"	     1008-01-06"],				# 戊辰
	  ["[天禧=ja:%%<天禧_(宋)>]1"	,	""	,	"	     1017-01-01"],				# 辛丑
	  ["[乾興=ja:%%<乾興_(宋)>]1"	,	""	,	"	     1022-01-01"],				# 辛未
	  ["[天聖]1"	,			"@A"	,	"name=[仁宗=ja:%%<仁宗_(宋)>];1023-01-01"],		# 丙寅 天聖２年から崇天暦を使用
	  ["[明道=ja:%%<明道_(北宋)>]1"	,	""	,	"	     1032-11-06"],				# 甲戌
	  ["[景祐]1"	,			""	,	"	     1034-01-01"],				# 壬戌
	  ["[宝元]1"	,			""	,	"	     1038-11-18"],				# 庚戌
	  ["[康定=ja:%%<康定_(宋)>]1"	,	""	,	"	     1040-02-21"],				# 丙午
	  ["[慶暦]1"	,			""	,	"	     1041-11-20"],				# 丙寅
	  ["[皇祐]1"	,			""	,	"	     1049-01-01"],				# 甲午
	  ["[至和]1"	,			""	,	"	     1054-03-16"],				# 庚辰
	  ["[嘉祐]1"	,			""	,	"	     1056-09-12"],				# 辛卯
	  ["[治平=ja:%%<治平_(宋)>]1"	,	"@A"	,	"name=[英宗=ja:%%<英宗_(宋)>];1064-01-01"],		# 丁酉 治平２年から明天暦を使用
	  ["[熙寧]1"	,			"@A"	,	"name=[神宗=ja:%%<神宗_(宋)>];1068-01-01"],		# 甲戌 本年から再び崇天暦を使用
	  ["[元豊=ja:%%<元豊_(宋)>]1"	,	""	,	"	     1078-01-01"],				# 丁未 熙寧８年から奉元暦を使用
	  ["[元祐]1"	,			"@A"	,	"name=[哲宗=ja:%%<哲宗_(宋)>];1086-01-01"],		# 庚寅
	  ["[紹聖]1"	,			""	,	"	     1094-04-12"],				# 癸丑 本年から観天暦を使用
	  ["[元符]1"	,			""	,	"	     1098-06-01"],				# 戊寅
	  ["[建中靖国]1",			"@A"	,	"name=[徽宗];1101-01-01"],				# 壬戌
	  ["[崇寧]1"	,			""	,	"	     1102-01-01"],				# 丁巳 崇寧２年から占天暦を使用
	  ["[大観=ja:%%<大観_(宋)>]1"	,	""	,	"	     1107-01-01"],				# 戊子 崇寧５年から紀元暦を使用
	  ["[政和]1"	,			""	,	"	     1111-01-01"],				# 甲子
	  ["[重和]1"	,			""	,	"	     1118-11-01"],				# 己酉
	  ["[宣和]1"	,			""	,	"	     1119-01-01"],				# 戊申
	  ["[靖康]1"	,			"@A"	,	"name=[欽宗];1126-01-01", "1127-05-01"] 		# 丁卯
	],
	[self,
	  "period:[蜀=ja:%%<応運>]",
	  ["[応運]1"	,			"@F"	,"name=[李順=ja:%%<応運>];994-01-16^Chinese0956", "994-05-25="]	# 己巳  399-3
	],
	[self,
	  "period:[聖公=ja:%%<方臘の乱>]",
	  ["[永楽=ja:%%<永楽_(方臘)>]1*11.01",	"@F"	,"name=[方臘=ja:%%<方臘の乱>];1121*11-01^Chinese1119", "1121-08-24="] # 戊戌朔 2425-47
	],
	[self,
	  "period:[斉=zh:%%<刘齐>]",
	  ["[天会=ja:%%<天会_(金)>]8",		"@A"	,"name=[劉予,alias:劉豫=ja:%%<劉予>];1130-09-09^Chinese0939"],	# 戊申に即位 2853-26 8.09.09
	  ["[阜昌]1"	,			""	,	"	     1130-11-22", "1137-11-18="]		# 辛酉 2865-57
	],
	[self,
	  "period:[楚=zh:%%<岳飛剿滅楊幺>]",
	  ["[天載]1"	,			"@F"	,"name=[鐘相=zh:%%<钟相>];1130-02^Chinese0956", "1130-03="],	 # 2821-25
	  ["[大聖天王]1",			"@A"	,"name=[楊麼=zh:%%<岳飛剿滅楊幺>];1133-04^Chinese0956", "1136-06="] # 2970-43
	],
	[self,
	  "period:[南宋]",
	  ["[建炎]1"	,			"@A"	,	"name=[高宗=ja:%%<高宗_(宋)>];1127-05-01^Chinese0956"],	# 庚寅
	  ["[紹興=ja:%%<紹興_(宋)>]1"	,	""	,	"	     1131-01"],					# 紹興６年から統元暦を使用
	  ["[隆興]1"	,			"@A"	,	"name=[孝宗=ja:%%<孝宗_(宋)>];1163-01-01"],		# 壬辰
	  ["[乾道=ja:%%<乾道_(宋)>]1"	,	""	,	"	     1165-01-01"],				# 辛亥 乾道４年から乾道暦を使用
	  ["[淳熙]1"	,			""	,	"	     1174-01-01"],				# 己丑 淳熙４年から淳熙暦を使用
	  ["[紹熙]1"	,			"@A"	,	"name=[光宗=ja:%%<光宗_(宋)>];1190-01-01"],		# 丙辰 紹熙２年から会元暦を使用
	  ["[慶元]1"	,			"@A"	,	"name=[寧宗=ja:%%<寧宗_(宋)>];1195-01-01"],		# 丁亥 慶元５年から統天暦を使用
	  ["[嘉泰=ja:%%<嘉泰_(宋)>]1"	,	""	,	"	     1201-01-01"],				# 壬子
	  ["[開禧]1"	,			""	,	"	     1205-01-01"],				# 己未 開禧４年から開禧暦を使用
	  ["[嘉定=ja:%%<嘉定_(南宋)>]1"	,	""	,	"	     1208-01-01"],				# 辛未
	  ["[宝慶]1"	,			"@A"	,	"name=[理宗];1225-01-01"],				# 壬戌
	  ["[紹定]1"	,			""	,	"	     1228-01-01"],				# 丙子
	  ["[端平]1"	,			""	,	"	     1234-01-01"],				# 庚子
	  ["[嘉熙]1"	,			""	,	"	     1237-01-01"],				# 癸丑
	  ["[淳祐=ja:%%<淳祐_(南宋)>]1"	,	""	,	"	     1241-01-01"],				# 庚寅 淳祐１２年のみ淳祐暦を使用
	  ["[宝祐]1"	,			""	,	"	     1253-01-01"],				# 庚辰 本年から会天暦を使用
	  ["[開慶]1"	,			""	,	"	     1259-01-01"],				# 乙巳
	  ["[景定]1"	,			""	,	"	     1260-01-01"],				# 己巳
	  ["[咸淳]1"	,			"@A"	,	"name=[度宗];1265-01-01"],				# 辛未 咸淳７年から成天暦を使用
	  ["[徳祐]1"	,			"@A"	,	"name=[恭帝=ja:%%<恭帝_(宋)>];1275-01-01"],		# 癸酉
	  ["[景炎]1"	,			"@A"	,	"name=[端宗=ja:%%<端宗_(宋)>];1276-05-01"],		# 乙未 景炎２年から本天暦を使用
	  ["[祥興]1"	,			"@A"	,	"name=[衛王=ja:%%<祥興帝>];1278-05-01", "1279-02-06="]	# 癸未
	]
      ],
      [self,
	"period:[遼]",
	[self,
	  "period:[五代十国#契丹=ja:%%<契丹>]",
	  ["[<太祖>=ja:%%<耶律阿保機>]1",	"@F"	,	"name=[太祖=ja:%%<耶律阿保機>];907-^Chinese0523"],
	  ["[神冊]1"	,			""	,	"	     916-12"],
	  ["[天賛]1"	,			""	,	"	     922-02"],
	  ["[天顕]1"	,			""	,	"	     926-02",
								"name=[太宗=ja:%%<耶律堯骨>];927-11", ""],
	  ["[会同]1"	,			""	,	"	     938-11", "947-02"]
	],
	["[大同=ja:%%<大同_(遼)>]1"	,	""	,	"	     947-02^Chinese0939"],			# 本年から調元暦を使用
	["[天禄=ja:%%<天禄_(遼)>]1"	,	"@A"	,	"name=[世宗=ja:%%<世宗_(遼)>];947-09"],
	["[応暦]1"	,			"@A"	,	"name=[穆宗=ja:%%<穆宗_(遼)>];951-09-04"],		# 癸亥
	["[保寧]1"	,			"@A"	,	"name=[景宗=ja:%%<景宗_(遼)>];969-02-23"],		# 庚午
	["[乾亨=ja:%%<乾亨_(遼)>]1"	,	""	,	"	     979-11-25", "983-06-10"],			# 辛丑
	[self,
	  "period:[契丹]",
	  ["[統和]1"	,			"@A"	,	"name=[聖宗=ja:%%<聖宗_(遼)>];983-06-10^Chinese0939"],	# 甲午 統和１３年から大明暦を使用
	  ["[開泰=ja:%%<開泰_(遼)>]1"	,	""	,	"	     1012-11-01"],				# 甲午
	  ["[太平=ja:%%<太平_(遼)>]1"	,	""	,	"	     1021-11-12"],				# 癸未
	  ["[景福=ja:%%<景福_(遼)>]1"	,	"@A"	,	"name=[興宗=ja:%%<興宗_(遼)>];1031-06-03"],		# 己卯
	  ["[重熙]1"	,			""	,	"	     1032-11-11"],				# 己卯
	  ["[清寧]1"	,			"@A"	,	"name=[道宗=ja:%%<道宗_(遼)>];1055-08-16", "1065-01-01"] # 辛丑
	],
	["[咸雍]1"	,			""	,	"	     1065-01-01^Chinese0939"],			# 辛酉
	["[太康=ja:%%<太康_(遼)>,alias:大康=ja:%%<太康_(遼)>]1","",	"	     1075-01-01"],			# 癸巳
	["[太安=ja:%%<太安_(遼)>,alias:大安=ja:%%<太安_(遼)>]1","",	"	     1085-01-01"],			# 丙申
	["[寿昌,alias:寿隆=ja:%%<寿昌>]1",	""	,	"	     1095-01-01"],				# 戊戌
	["[乾統=ja:%%<乾統_(遼)>]1"	,	"@A"	,	"name=[天祚帝];1101-02-01"],				# 壬辰
	["[天慶=ja:%%<天慶_(遼)>]1"	,	""	,	"	     1111-01-01"],				# 甲子
	["[保大=ja:%%<保大_(遼)>]1"	,	""	,	"	     1121-01-01", "1125-02-20="]		# 丁酉
      ],
      [self,
	"period:[金=ja:%%<金_(王朝)>]",
	["[収国]1"	,			"@F"	,	"name=[太祖=ja:%%<阿骨打>];1115-01-01^Chinese0956"],	# 壬申
	["[天輔=ja:%%<天輔_(金)>]1"	,	""	,	"	     1117-01-01"],				# 庚寅
	["[天会=ja:%%<天会_(金)>]1"	,	"@A"	,	"name=[太宗=ja:%%<呉乞買>];1123-09-16^Chinese0939"],	# 丙寅 本年から大明暦を使用
	["[天眷]1"	,			"@A"	,	"name=[熙宗=ja:%%<熙宗_(金)>];1138-01-01"],		# 戊子
	["[皇統=ja:%%<皇統_(金)>]1"	,	""	,	"	     1141-01-13"],				# 癸丑
	["[天徳=ja:%%<天徳_(金)>]1"	,	"@A"	,	"name=[海陵王];1149-12-11"],				# 己未
	["[貞元=ja:%%<貞元_(金)>]1"	,	""	,	"	     1153-03-26"],				# 乙卯
	["[正隆]1"	,			""	,	"	     1156-02-01"],				# 癸酉
	["[大定=ja:%%<大定_(金)>]1"	,	"@A"	,	"name=[世宗=ja:%%<世宗_(金)>];1161-10-08"],		# 丁未 大定２２年から重修大明暦を使用
	["[明昌=ja:%%<明昌_(金)>]1"	,	"@A"	,	"name=[章宗=ja:%%<章宗_(金)>];1190-01-01"],		# 丙辰
	["[承安=ja:%%<承安_(金)>]1"	,	""	,	"	     1196-11-22"],				# 丁酉
	["[泰和=ja:%%<泰和_(金)>]1"	,	""	,	"	     1201-01-01"],				# 壬子
	["[大安=ja:%%<大安_(金)>]1"	,	"@A"	,	"name=[衛紹王];1209-01-27"],				# 壬戌
	["[崇慶]1"	,			""	,	"	     1212-01"],
	["[至寧]1"	,			""	,	"	     1213-05"],
	["[貞祐=ja:%%<貞祐_(金)>]1"	,	"@A"	,	"name=[宣宗=ja:%%<宣宗_(金)>];1213-09-15"],		# 壬子
	["[興定]1"	,			""	,	"	     1217-09-08"],				# 壬午
	["[元光=ja:%%<元光_(金)>]1"	,	""	,	"	     1222-08-09"],				# 甲申
	["[正大]1"	,			"@A"	,	"name=[哀宗=ja:%%<哀宗_(金)>];1224-01-01"],		# 戊戌
	["[開興]1"	,			""	,	"	     1232-01"],
	["[天興=ja:%%<天興_(金)>]1"	,	""	,	"	     1232-04-14", "1234-01="],			# 甲子
	[self,
	  "period:[大真=ja:%%<大真国>]",
	  ["[天泰]1"	,			"@F"	,	"name=[蒲鮮万奴, alias:富鮮万努=];1215-10^Chinese0939", "1233-09="] # 4352-46
	]
      ],
      [self,
	"period:[元=ja:%%<元_(王朝)>]",
	[self,
	  "period:[モンゴル]",
	  ["[<太祖>=ja:%%<チンギス・カン>]1",	"@F"	,	"name=[太祖=ja:%%<チンギス・カン>];1206-^Chinese0956", "1215-01^Chinese0939", ""], # 太祖１０年から重修大明暦を使用
	  ["[<太宗>=ja:%%<オゴデイ>]1"	,	"@A"	,	"name=[太宗=ja:%%<オゴデイ>];1229-08"],
	  ["[<定宗>=ja:%%<グユク>]1"	,	"@A"	,	"name=[定宗=ja:%%<グユク>];1246-07"],
	  ["[<憲宗>=ja:%%<モンケ>]1"	,	"@A"	,	"name=[憲宗=ja:%%<モンケ>];1251-06"],
	  ["[<世祖>=ja:%%<クビライ>]1"	,	"@A"	,	"name=[世祖=ja:%%<クビライ>];1260-03-24"],		# 辛卯
	  ["[中統]1"	,			""	,	"	     1260-05-19"],				# 丙戌
	  ["[至元=ja:%%<至元_(元世祖)>]1",	""	,	"	     1264-08-16", "1271-11-15"]			# 丁巳
	],
	["[世祖#至元=ja:%%<至元_(元世祖)>]8.11.15","","name=[世祖=ja:%%<クビライ>];1271-11-15^Chinese0939"],		# 乙亥 至元１８年から授時暦を使用
	["[元貞]1"	,			"@A"	,	"name=[成宗=ja:%%<テムル>];1295-01-01"],		# 丙午
	["[大徳=ja:%%<大徳_(元)>]1"	,	""	,	"	     1297-02-27"],				# 庚申
	["[至大]1"	,			"@A"	,	"name=[武宗=ja:%%<カイシャン>];1308-01-01"],		# 辛酉
	["[皇慶=ja:%%<皇慶_(元)>]1"	,	"@A"	,	"name=[仁宗=ja:%%<アユルバルワダ>];1312-01-01"],	# 丁酉
	["[延祐]1"	,			""	,	"	     1314-01-22"],				# 丁未
	["[至治=ja:%%<至治_(元)>]1"	,	"@A"	,	"name=[英宗=ja:%%<シデバラ>];1321-01-01"],		# 乙亥
	["[泰定=ja:%%<泰定_(元)>]1"	,	"@A"	,	"name=[泰定帝=ja:%%<イェスン・テムル>];1324-01-01"],	# 戊子
	["[致和]1"	,			""	,	"	     1328-02-27", "1328-09-13"],		# 庚申, 壬申
	["[天順=ja:%%<天順_(元)>]1"	,	"@A"	,	"name=[天順帝=ja:%%<アリギバ>];1328-09-01", "1328-10-13="], # 庚申, 辛丑
	["[天暦=ja:%%<天暦_(元)>]1"	,	"@A"	,	"name=[文宗=ja:%%<トク・テムル>];1328-09-13",		# 壬申
								"name=[明宗=ja:%%<コシラ>];1329-01-28",			# 丙戌
								"name=[文宗=ja:%%<トク・テムル>];1329-08-06", ""],	# 庚寅
	["[至順]1"	,			""	,	"	     1330-05-08",				# 戊午
								"name=[寧宗=ja:%%<イリンジバル>];1332-10-04",		# 庚子
								"name=[順帝=ja:%%<トゴン・テムル>];1333-06-08", ""],	# 己巳
	["[元統=ja:%%<元統_(元)>]1"	,	""	,	"	     1333-10-08"],				# 戊辰
	["[順帝#至元=ja:%%<至元_(元順帝)>]1",	""	,	"	     1335-11-23"],				# 辛丑
	["[至正]1"	,			""	,	"	     1341-01-01", "1371-"],			# 己酉 至正２８年閏７月丙寅元北帰
	[self,
	  "period:[北元]",
	  ["[宣光]1"	,			"@A"	,	"name=[昭宗=ja:%%<アユルシリダラ>];1371-^Chinese0939"],
	  ["[天元=ja:%%<天元_(北元)>]1"	,	"@A"	,	"name=[天元帝=ja:%%<トグス・テムル>];1379-", "1388="]
	],
	[self,
	  "period:[<陳吊眼>=zh:%%<陳吊眼>]",
	  ["[昌泰=ja:%%<昌泰_(陳吊眼)>]1",	"@F",	"name=[陳吊眼=zh:%%<陳吊眼>];1281-08=01^Chinese0939", "1281-11-02="] # 鄭所南『元韃攻日本敗北歌並序』（『心史・中興集乙』1933年支那内学院刊本所収）Wikipedia
	],
	[self,
	  "period:[天完=zh:%%<天完>]",
	  ["[治平=ja:%%<治平_(徐寿輝)>]1",	"@F"	,	"name=[徐寿輝=ja:%%<徐寿輝>];1351-10^Chinese0939", "1360-05=03"] # 5723-36
	],
	[self,
	  "period:[漢=ja:%%<陳友諒>]",
	  ["[大義=ja:%%<大義_(年号)>]1"	,	"@F"	,	"name=[陳友諒];1360-05=03^Chinese0939"],		# 戊午 5869-25
	  ["[大定=ja:%%<大定_(陳友諒)>]1",	""	,		  "1361"],					# Wikipedia
	  ["[徳寿]1"	,			"@A"	,	"name=[陳理];1363-08-27", "1364-02-19="]		# 壬戌の翌日 5914-41
	],
	[self,
	  "period:[周=ja:%%<張士誠>]",
	  ["[天祐=ja:%%<天祐_(張士誠)>]0",	"@F"	,	"name=[張士誠];1353-05-29^Chinese0939", "1357="],	# 乙未 5756-28 Wikipedia 踰年
	# ["[天祐?=ja:%%<天祐_(張士誠)>]5",	"@F"	,	"1358", "1367-09-08="] # Wikipedia
	],
	[self,
	  "period:[夏=ja:%%<明玉珍>]",
	  ["[天統=ja:%%<天統_(明玉珍)>]1",	"@F"	,	"name=[明玉珍];1362-03-03^Chinese0939"],		# 己酉 5888-16
	  ["[開熙]0"	,			"@A"	,	"name=[明昇];1366-02", "1371-06="]			# 5962-18 Wikipedia 踰年
	],
	[self,
	  "period:[宋=ja:%%<韓林児>]",
	  ["[龍鳳=ja:%%<龍鳳_(韓林児)>]1",	"@F"	,	"name=[韓林児];1355-02-02^Chinese0939", "1367-01-01"]	# 乙未は２月にない 5778-6 Wikipedia 己未
	]
      ],
      [self,
	"period:[明]",
	[self,
	  "period:[呉=ja:%%<朱元璋>]",
	  ["[呉=ja:%%<朱元璋>]1"	,	""	,	"name=[朱元璋];1367-01-01^Chinese0939", "1368-01-03="]	# 5979-1
	],
	["[洪武]1"	,			"@F"	,	"name=[太祖=ja:%%<朱元璋>];1368-01-04^Chinese0939"],	# 乙亥 6012-3
	["[建文]1"	,			"@A"	,	"name=[恵帝=ja:%%<建文帝>];1399-01-01"],
	["[洪武]35.07.01",			""	,	"name=[成祖=ja:%%<永楽帝>];1402-07-01"],
	["[永楽=ja:%%<永楽_(明)>]1"	,	"@A"	,	"name=[成祖=ja:%%<永楽帝>];1403-01-01"],
	["[洪熙]1"	,			"@A"	,	"name=[仁宗=ja:%%<洪熙帝>];1425-01-01"],
	["[宣徳]1"	,			"@A"	,	"name=[宣宗=ja:%%<宣徳帝>];1426-01-01"],
	["[正統=ja:%%<正統_(明)>]1"	,	"@A"	,	"name=[英宗=ja:%%<英宗_(明)>];1436-01-01"],
	["[景泰=ja:%%<天順_(明)>]1"	,	"@A"	,	"name=[代宗=ja:%%<景泰帝>];1450-01-01"],
	["[天順]1"	,			""	,	"name=[英宗=ja:%%<英宗_(明)>];1457-01-17"],		# 壬午 陳舜臣 116 奪門の変
	["[成化]1"	,			"@A"	,	"name=[憲宗=ja:%%<成化帝>];1465-01-01"],
	["[弘治=ja:%%<弘治_(明)>]1"	,	"@A"	,	"name=[孝宗=ja:%%<弘治帝>];1488-01-01"],
	["[正徳=ja:%%<正徳_(明)>]1"	,	"@A"	,	"name=[武宗=ja:%%<正徳帝>];1506-01-01"],
	["[嘉靖]1"	,			"@A"	,	"name=[世宗=ja:%%<嘉靖帝>];1522-01-01"],
	["[隆慶=ja:%%<隆慶_(明)>]1"	,	"@A"	,	"name=[穆宗=ja:%%<隆慶帝>];1567-01-01"],
	["[万暦]1"	,			"@A"	,	"name=[神宗=ja:%%<万暦帝>];1573-01-01"],
	["[泰昌]1.08.01",			"@A"	,	"name=[光宗=ja:%%<泰昌帝>];1620-08-01"],
	["[天啓=ja:%%<天啓_(明)>]1"	,	"@A"	,	"name=[熹宗=ja:%%<天啓帝>];1621-01-01"],
	["[崇禎]1"	,			"@A"	,	"name=[思宗=ja:%%<崇禎帝>];1628-01-01", "1644-03-19="],
	[self,
	  "period:[南明]",
	  ["[崇禎]17.05.15",			""	,	"name=[福王=ja:%%<弘光帝>];1644-05-15^Chinese0939"],	# 陳舜臣 444 コンサイスでは７月
	  ["[弘光]1"	,			"@A"	,	"	     1645-01-01"],				# 前年３月崇禎帝が李自成に殺される
	  ["[隆武]1"	,			"@A"	,	"name=[唐王=ja:%%<隆武帝>];1645-06="],
	  ["[紹武]0"	,			""	,	"name=[紹武帝=ja:%%<紹武帝>];1646-11-05", "1646-12-15="], # Wikipedia 踰年
	  ["[永暦=ja:%%<永暦_(南明)>]1"	,	"@A"	,	"name=[永明王=ja:%%<永暦帝>];1647-01-01",
								"name=[鄭成功];1662-04-15=^Chinese1645",		# 陳舜臣 471 Wikipedia
								"name=[鄭経];1662-05-08=",				# 陳舜臣 471 Wikipedia
								"name=[鄭克塽=ja:%%<鄭克ソウ>];1681-01-28=", "1683-08="] # 陳舜臣 482 Wikipedia
	],
	[self,
	  "period:[順=zh:%%<大順_(政權)>]",
	  ["[永昌=ja:%%<永昌_(李自成)>]1",	"@F"	,	"name=[李自成];1644-01-01^Chinese0939", "1645-05="]	# 陳舜臣 375
	],
	[self,
	  "period:[西=zh:%%<大西>]",
	  ["[大順=ja:%%<大順_(張献忠)>]1",	"@F"	,	"name=[張献忠];1644-11^Chinese0939", "1646-11-27="]	# コンサイス Wikipedia
	]
      ],
      [self,
	"period:[清]",
	[self,
	  "period:[後金]",
	  ["[天命=ja:%%<天命_(後金)>]1"	,	"@F"	,	"name=[太祖=ja:%%<ヌルハチ>];1616-01-01^Chinese0939"],	# 授時暦を使用していたものと推定
	  ["[天聡]1"	,			"@A"	,	"name=[太宗=ja:%%<ホンタイジ>];1627-01-01", "1636-04-11"]
	],
	["[崇徳]1"	,			"@F"	,	"	     1636-04-11^Chinese0939"],			# 乙酉 陳舜臣 340
	["[順治]1"	,			"@A"	,	"name=[世祖=ja:%%<順治帝>];1644-01-01", "1645-01^Chinese1645", ""], # 順治２年から時憲暦を使用
	["[康熙]1"	,			"@A"	,	"name=[聖祖=ja:%%<康熙帝>];1662-01-01", "1723-01-01"],
	[self,
	  "period:[順=ja:%%<耿精忠>]",
	  ["[裕民]1"	,			"@F"	,	"name=[耿精忠];1674-10^Chinese1645", "1676-10="]
	],
	[self,
	  "period:[周=ja:%%<呉三桂>]",
	  ["[<呉三桂>]0"	,		"@F"	,	"name=[呉三桂];1673-11-21^Chinese1645"],		# 陳舜臣 476 Wikipedia  踰年
	  ["[昭武]1"	,			"@A"	,	"	       1678-03"],				# 陳舜臣 481
	  ["[洪化]0.08.15",			"@A"	,	"name=[呉世璠];1678-08-15", "1681-10-29="]		# 陳舜臣 481 Wikipedia 踰年, 1681-10 戊申
	],
	["[雍正]1"	,			"@A"	,	"name=[世宗=ja:%%<雍正帝>];1723-01-01^Chinese1645"],
	["[乾隆]1"	,			"@A"	,	"name=[高宗=ja:%%<乾隆帝>];1736-01-01", "1796-01-01"],
	[self,
	  "period:[明=ja:%%<林爽文>]",
	  ["[天運=zh:%%<天运_(林爽文)>]1",	"@F"	,	"name=[林爽文];1786-11^Chinese1645"],			# Wikipedia により天運と順天を分割
	  ["[順天=ja:%%<順天_(林爽文)>]1",	""	,	"1787", "1788-01="]
	],
	["[嘉慶=ja:%%<嘉慶_(清)>]1"	,	"@A"	,	"name=[仁宗=ja:%%<嘉慶帝>];1796-01-01^Chinese1645"],
	["[道光]1"	,			"@A"	,	"name=[宣宗=ja:%%<道光帝>];1821-01-01"],
	["[咸豊]1"	,			"@A"	,	"name=[文宗=ja:%%<咸豊帝>];1851-01-01", "1862-01-01"],
	[self,
	  "period:[太平天国]",
	  ["[天暦=ja:%%<天暦_(暦法)>]2"	,	"@F"	,	"name=[洪秀全];1852-1-1^TenrekiA", "1853-3-1^TenrekiB", "1868-7-1="]
	],
	[self,
	  "period:[大成国]",
	  ["[洪徳=ja:%%<洪徳_(陳開)>]1"	,	"@F"	,	"name=[陳開];1855-08^Chinese1645", "name=[黄鼎鳳];1861-07=", "1864="]
	],
	["[同治]1"	,			"@A"	,	"name=[穆宗=ja:%%<同治帝>];1862-01-01^Chinese1645"],
	["[光緒]1"	,			"@A"	,	"name=[徳宗=ja:%%<光緒帝>];1875-01-01"],
	["[宣統]1"	,			"@A"	,	"name=[末帝=ja:%%<愛新覚羅溥儀>];1909-01-01", "1911-11-12="],
      ],
      [self,
	"period:[中華民国=ja:%%<民国紀元>,ROC=en:Chinese_era_name#The_Republic_of_China_Era,zh:中華民國=zh:%%<民國紀年>]",
	["[民国=ja:%%<民国紀元>,ROC=en:Chinese_era_name#The_Republic_of_China_Era,zh:民國=zh:%%<民國紀年>]1",
				"@F",		"name=;1912-01-01^Gregorian?note=Chinese"]
      ]
    ]]
  end
end
