# -*- coding: utf-8 -*-
=begin
  Copyright (C) 2013-2015 Takashi SUGA

  You may use and/or modify this file according to the license described in the LICENSE.txt file included in this archive.
=end

require 'when_exe/region/chinese'
require 'when_exe/region/chinese/calendars'

#
#  王位年号一覧表
#
# （参考文献）
# 東洋史辞典（東京創元社）
# コンサイス世界年表（三省堂）
# 朝鮮史（山川出版社）
# 資治通鑑（宏業書局）
# 続資治通鑑（宏業書局）
# 歴代紀元編（台湾中華書局）
#
module When
  class TM::CalendarEra

    #
    # 満洲
    #
    Manchurian = [self, [
      "locale:[=ja:, en=en:, alias]",
      "area:[満洲,Manchurian=en:Manchuria]",
      [self,
	"period:[震=ja:%%<渤海_(国)>]",
	["[<高王>=ja:%%<大祚栄>]1",	"@F",	"name=[高王=ja:%%<大祚栄>];698^Chinese0523", "713"]
      ],
      [self,
	"period:[渤海=ja:%%<渤海_(国)>,Balhae]",
	["[<高王>=ja:%%<大祚栄>]16",	"@A",	"name=[高王=ja:%%<大祚栄>];713^Chinese0523"],
	["[仁安=ja:%%<仁安_(渤海)>]0",	"@A",	"name=[武王=ja:%%<大武芸>];719"],		# コンサイスでは翌年が元年、以下翌年のものは(*)
	["[大興=ja:%%<大興_(渤海)>]1",	"@A",	"name=[文王=ja:%%<大欽茂>];738", "794"],
	["[宝暦=ja:%%<宝暦_(渤海)>]1",	"",				  "774", "775"],	# Wikipedia
	["[<廃王>=ja:%%<大元義>]1",	"@A",	"name=[廃王=ja:%%<大元義>];794"],		# 朝鮮史では793?とする
	["[中興=ja:%%<中興_(渤海)>]1",	"@A",	"name=[成王=ja:%%<大華ヨ>];794"],		# 新唐書では786が元年？
	["[正暦=ja:%%<正暦_(渤海)>]1",	"@A",	"name=[康王=ja:%%<大嵩リン>];795"],
	["[永徳=ja:%%<永徳_(渤海)>]0",	"@A",	"name=[定王=ja:%%<大元瑜>];809"],		# コンサイスでは翌年が元年
	["[朱雀=ja:%%<朱雀_(渤海)>]1",	"@A",	"name=[僖王=ja:%%<大言義>];813"],		# 朝鮮史では817が末年		(*)
	["[太始=ja:%%<太始_(渤海)>]1",	"@A",	"name=[簡王=ja:%%<大明忠>];818"],		# 新唐書では前年が元年
	["[建興=ja:%%<建興_(渤海)>]0",	"@A",	"name=[宣王=ja:%%<大仁秀>];818"],		# コンサイスでは翌年が元年	(*)
	["[咸和=ja:%%<咸和_(渤海)>]0",	"@A", "name=[彝震王=ja:%%<大彝震>];830"],		# （以後同様）		(*)
	["[<虔晃王>=ja:%%<大虔晃>]0",	"@A", "name=[虔晃王=ja:%%<大虔晃>];857"],		# 新唐書には以後記載なし (*)
	["[<景王>=ja:%%<大玄錫>]0",	"@A",	"name=[景王=ja:%%<大玄錫>];871"],		# 朝鮮史は871?	(*)
	["[<瑋瑎>=ja:%%<大イカイ>]0",	"@A",	"name=[瑋瑎=ja:%%<大イカイ>];894"],		# 朝鮮史では在位不明 (*)
	["[<哀王>=ja:%%<大インセン>]0",	"@A",	"name=[哀王=ja:%%<大インセン>];906",		#		(*)
	    "926="]
      ],
      [self,
	"period:[満洲国=ja:%%<満州国>]",
	["[大同=ja:%%<大同_(満州)>]1",	"@F",	"name=[溥儀=ja:%%<愛新覚羅溥儀>];1932-03-01^Gregorian?note=Chinese"],
	["[康徳]1.03.01","",							"1934-03-01", "1945-08-18="]
      ],
    ]]

    #
    # 柔然
    #
    Rouran = [{
        'V'=>{'464'  => {'A'=>'464', 'B'=>'485', 'C'=>'492', 'D'=>'506', 'E'=>'508', 'F'=>'520'},
              '466'  => {'A'=>'466', 'B'=>'487', 'C'=>'494', 'D'=>'508', 'E'=>'510', 'F'=>'522'}}
      }, self, [
      "locale:[=ja:, en=en:, alias]",
      'area:[柔然#{?V=V}=ja:%%<柔然>,Rouran#{?V=V}=en:Rouran]',
      ["[永康=ja:%%<永康_(柔然)>]1",	"@A",	'name=[予成=ja:%%<郁久閭予成>];#{A:464}^Chinese0445'],
      ["[太平=ja:%%<太平_(柔然)>]1",	"@A",	'name=[豆崙=ja:%%<郁久閭豆崙>];#{B:485}'],
      ["[太安=ja:%%<太安_(柔然)>]1",	"@A",	'name=[那蓋=ja:%%<郁久閭那蓋>];#{C:492}'],
      ["[始平]1",			"@A",	'name=[伏図=ja:%%<郁久閭伏図>];#{D:506}'],
      ["[建昌=ja:%%<建昌_(柔然)>]1",	"@A",	'name=[醜奴=ja:%%<郁久閭醜奴>];#{E:508}', '#{F:520}=']	# Wikipedia 509 -> 508
    ]]

    #
    # 高昌
    #
    Gaochang = [self, [
      "locale:[=ja:, en=en:, alias]",
      "area:[高昌,Gaochang]",
      ["[章和=ja:%%<章和_(高昌)>]1",	"@A",	  "name=[麹堅];531^Chinese0523"],
      ["[永平=ja:%%<永平_(高昌)>]1",	"@A",	"name=[麹玄喜];549"],
      ["[和平=ja:%%<和平_(高昌)>]1",	"@A",		"name=;551"],
      ["[建昌=ja:%%<建昌_(高昌)>]1",	"@A",	"name=[麹宝茂=zh:%%<麴寶茂>];555"],
      ["[延昌=ja:%%<延昌_(高昌)>]1",	"@A",	"name=[麹乾固=zh:%%<麴乾固>];561"],
      ["[延和=ja:%%<延和_(高昌)>]1",	"@A",	"name=[麹伯雅];602"],
      ["[義和=ja:%%<義和_(高昌)>]1",	"@A",		"name=;614"],
      ["[重光=ja:%%<重光_(高昌)>]1",	"",	"name=[麹伯雅];620"],
      ["[延寿=ja:%%<延寿_(高昌)>]1",	"@A",	"name=[麹文泰];624", "name=[麹智盛];640", "640="]
    ]]

    #
    # 雲南
    #
    Yunnan = [self, [
      "locale:[=ja:, en=en:, alias]",
      "area:[雲南,Yunnan]",
      [self,
	"period:[南詔,Nanzhao]",
	["[<閣羅鳳>=zh:%%<閣羅鳳>]1",	"@A",   "name=[閣羅鳳=zh:%%<閣羅鳳>];748^Chinese0523"],
	["[賛普鍾=zh:%%<賛普鍾>]1",	"@A",				    "752"],
	["[長寿=ja:%%<長寿_(南詔)>]1",	"",				    "769"],
	["[見龍]1",			"@A",	"name=[異牟尋=zh:%%<异牟寻>];780"],
	["[上元=ja:%%<上元_(南詔)>]1",	"",				    "784"],
	["[上元?=ja:%%<上元_(南詔)>]2",	"",				    "785"],		# 元封?
	["[応道]1",			"@A",	"name=[尋閣勧=zh:%%<寻阁劝>];809"],
	["[龍興=ja:%%<龍興_(南詔)>]1",	"@A",	"name=[勧龍晟=zh:%%<劝龙晟>];810"],
	["[全義]1",			"@A",	"name=[勧利晟=zh:%%<劝利晟>];817"],
	["[大豊=ja:%%<大豊_(南詔)>]1",	"",				    "820"],
	["[保和]1",			"@A",	"name=[晟豊祐=zh:%%<劝丰祐>];824"],
	["[保和?=ja:%%<保和>, alias:天啓=ja:%%<天啓_(南詔)>]17", "",	    "840"],		# 天啓?
	["[建極]1",			"@A",	    "name=[世隆=zh:%%<世隆>];860"],
	["[建極?=ja:%%<建極>, alias:法尭=ja:%%<法尭>]14","",		    "873", "877="]	# 法尭?
      ],
      [self,
	"period:[大封民=zh:%%<南诏>#%.<改國號>]",
	["[貞明=ja:%%<貞明_(南詔)>]1",	"@F",	"name=[隆舜=zh:%%<隆舜>];878^Chinese0523"],
	["[貞明?=ja:%%<貞明_(南詔)>]2",	"",				"879"],			# 承智, 大同?
	["[嵯耶]1",			"",				"889"],
	["[中興=ja:%%<中興 (南詔)>]1", "@A","name=[舜化貞=zh:%%<舜化貞>];897", "903"]
      ],
      [self,
	"period:[大長和=zh:%%<大長和>]",
	["[安国=ja:%%<安国_(南詔)>]1",	"@F",	"name=[鄭買嗣=zh:%%<鄭買嗣>];903^Chinese0523"],
	["[始元=ja:%%<始元_(南詔)>]1",	"@A",	"name=[鄭仁旻=zh:%%<鄭仁旻>];910"],
	["[始元?=ja:%%<始元_(南詔)>]2",	"",				    "911"],		# 天瑞景星, 安和, 貞祐, 初暦, 孝治?
	["[天応=ja:%%<天応_(南詔)>]1",	"@A",	"name=[鄭隆亶=zh:%%<鄭隆亶>];927", "928"]
      ],
      [self,
	"period:[大天興=zh:%%<大天興>]",
	["[尊聖]1",	"@F",	"name=[趙善政=zh:%%<趙善政>];928^Chinese0523", "930"]
      ],
      [self,
	"period:[大義寧=zh:%%<大義寧>]",
	["[興聖=ja:%%<興聖_(南詔)>]1",	"@F",	"name=[楊干貞=zh:%%<楊干貞>];930^Chinese0523"],
	["[大明=zh:%%<大明_(大义宁)>]1",	"",			    "931", "938"]
      ],
      [self,
	"period:[大理,Dali]",
	["[文徳=ja:%%<文徳_(大理)>]1",	"@F",	"name=[段思平];0938^Chinese0523"],
	["[文徳?=ja:%%<文徳_(大理)>]2","",		      "0939"],	# 神武?
	["[文経]1",			"@A",	"name=[段思英];0945"],
	["[至治=ja:%%<至治_(大理)>]1",	"@A",	"name=[段思良];0946", "0947^Chinese0956", ""],
	["[明徳=ja:%%<明徳_(大理)>]1",	"@A",	"name=[段思聡];0952"],
	["[明徳?=ja:%%<明徳_(大理)>]2","",		      "0953"],	# 広徳?
	["[順徳=ja:%%<順徳_(大理)>]1",	"",		      "0968"],
	["[明政=ja:%%<明政_(大理)>]1",	"@A",	"name=[段素順];0969"],
	["[広明=ja:%%<広明_(大理)>]1",	"@A",	"name=[段素英];0986"],
	["[明応=ja:%%<明応_(大理)>]1",	"",		      "1005"],
	["[明応?=ja:%%<明応_(大理)>]2","",		      "1006"],	# 明聖, 明徳, 明治?
	["[明啓]1",			"@A",	"name=[段素廉];1010"],
	["[乾興=ja:%%<乾興_(大理)>]1",	"",		      "1022"],	# 李家瑞「用文物補正南詔及大理国的紀年」（『歴史研究』1958年第7期）from Wikipedia
	["[明通]1",			"@A",	"name=[段素隆];1023"],
	["[正治=ja:%%<正治_(大理)>]1",	"@A",	"name=[段素真];1027"],
	["[聖明=ja:%%<聖明_(大理)>]1",	"@A",	"name=[段素興];1042"],
	["[聖明?=ja:%%<聖明_(大理)>]2","",		      "1043"],	# 天明?
	["[保安=ja:%%<保安_(大理)>]1",	"@A",	"name=[段思廉];1045"],
	["[正安=ja:%%<正安_(大理)>]1",	"",		      "1053"],
	["[正安?=ja:%%<正安_(大理)>]4","",		      "1056"],	# 正徳(政徳), 保徳, 太安, 明侯?
	["[上徳]1","@A","name=[段連義, alias:段連義=ja:%%<段廉義>];1076"],
	["[広安=ja:%%<広安_(大理)>]1",	"",		      "1077"],
	["[上明]1",			"@A",	"name=[段寿輝];1081"],
	["[保立]1",			"@A",	"name=[段正明];1082"],
	["[保立?=ja:%%<保立>]2",	"",		      "1083"],	# 建安, 天祐?
	["[上治]1",			"@A",	"name=[高昇泰];1095"],
	["[天授=ja:%%<天授_(大理)>]1",	"@A",	"name=[段正淳];1096"],
	["[開明=ja:%%<開明_(大理)>]1",	"",		      "1097"],
	["[天政, alias:天正=ja:%%<天政>]1",	"",	      "1103"],
	["[文安=ja:%%<文安_(大理)>]1",	"",		      "1105"],
	["[日新=ja:%%<日新_(大理)>]1",	"@A",	"name=[段正厳];1109"],
	["[文治=ja:%%<文治_(大理)>]1",	"",		      "1110"],
	["[文治?=ja:%%<文治_(大理)>]10",	"",	      "1119"],	# 永嘉?
	["[保天]1",	"",				      "1129"],
	["[保天?=ja:%%<保天>]9",	"",		      "1137"],	# 広運?
	["[永貞=ja:%%<永貞_(大理)>]1",	"@A",	"name=[段正興];1148"],
	["[大宝=ja:%%<大宝_(大理)>]1",	"",		      "1149"],
	["[大宝?=ja:%%<大宝_(大理)>]8","",		      "1156"],	# 龍興, 盛明, 建徳?
	["[利貞]1",			"@A",	"name=[段智興];1172"],
	["[盛徳=ja:%%<盛徳_(大理)>]1",	"",		      "1176"],
	["[嘉会]1",			"",		      "1181"],
	["[元亨=ja:%%<元亨_(大理)>]1",	"",		      "1185"],
	["[元亨?=ja:%%<元亨_(大理)>]12",	"",	      "1196"],	# 安定?
	["[鳳暦=ja:%%<鳳歴_(大理)>]1",	"@A",	"name=[段智廉];1201"],
	["[鳳暦?=ja:%%<鳳歴_(大理)>]2",	"",	      "1202"],		# 元寿?
	["[天開=ja:%%<天開_(大理)>]1",	"@A",	"name=[段智祥];1205"],
	["[天輔=ja:%%<天輔_(大理)>]1",	"",		      "1226"],
	["[天輔?=ja:%%<天輔_(大理)>]2","",		      "1227"],	# 仁寿?
	["[道隆]1",			"@A",	"name=[段祥興];1239"],
	["[天定=ja:%%<天定_(大理)>]1",	"@A",	"name=[段興智];1252", "1254="]
      ]
    ]]

    #
    # 西夏
    #
    WesternXia = [{
        'V'=>{'103407'=> {'E'=>'1034-07', 'F'=>'1034-08', 'G'=>'1067','H'=>'1069',
                          'I'=>'1090', 'J'=>'1098', 'K'=>'1101', 'L'=>'1114', 'M'=>'1119', 'N'=>'1170'}}
      }, self, [
      "locale:[=ja:, en=en:, alias]",
      'area:[西夏#{?V=V}=ja:%%<西夏>, Western_Xia#{?V=V}=en:Western_Xia]',
      ["[顕道]1"	,	"@F"	,	"name=[景宗=ja:%%<李元昊>];1032-11-25^Chinese0956"],	# 癸巳 886-35
      ["[開運=ja:%%<開運_(西夏)>]1"	,	""	,	'	     #{E:1034-09}'],		# 920-76
      ["[広運=ja:%%<広運_(西夏)>]1"	,	""	,	'	     #{F:1034-10}'],		# 〃
      ["[大慶=ja:%%<大慶_(夏景宗)>]1"	,	""	,	"	     1036-12"],			# 950-49
      ["[天授礼法延祚]1",			""	,	"	     1038-10-11"],		# 甲戌 973-63
      ["[延嗣寧国]1"	,	"@A"	,    "name=[毅宗=ja:%%<毅宗_(西夏)>];1049"],			# 1228-84
      ["[天祐垂聖]1"	,			""	,	"	     1050"],			# 1246-92
      ["[福聖承道]1"	,			""	,	"	     1053"],			# 1307-35
      ["[奲都=ja:%%<シャ都>]1"	,		""	,	"	     1057"],			# 1385-28
      ["[拱化]1"	,			""	,	"	     1063"],			# 1504-98
      ["[乾道=ja:%%<乾道_(西夏)>]1","@A",'name=[恵宗=ja:%%<恵宗_(西夏)>];#{G:1068}'],			# 1632-108
      ["[天賜礼盛国慶]1",		""	,	'		 #{H:1070}'],			# 1696-60
      ["[大安=ja:%%<大安_(西夏)>]1"	,	""	,	"	     1075"],			# 1787-98
      ["[天安礼定]1"	,			""	,	"	     1086"],			# 2008-6
      ["[天儀治平]1"	,	"@A"	,    "name=[崇宗=ja:%%<崇宗_(西夏)>];1087"],			# 2031-112
      ["[天祐民安]1"	,		""	,	'		 #{I:1091}'],			# 2084-73
      ["[永安=ja:%%<永安_(西夏)>]1"	,	""	,	'	 #{J:1099}'],			# 2192-91
      ["[貞観=ja:%%<貞観_(西夏)>]1"	,	""	,	'	 #{K:1102}'],			# 2248-52
      ["[雍寧]1"	,	""	,	'			 #{L:1115}'],			# 2375-64
      ["[元徳=ja:%%<元徳_(西夏)>,alias:天徳=ja:%%<元徳_(西夏)>]1","",'	 #{M:1120}'],			# 2426-57
      ["[正徳=ja:%%<正徳_(西夏)>]1"	,	""	,	"	     1127-04"],			# 2646-103
      ["[大徳=ja:%%<大徳_(西夏)>]1"	,	""	,	"	     1135"],			# 3084-66
      ["[大慶=ja:%%<大慶_(夏仁宗)>]0","@A",  "name=[仁宗=ja:%%<仁宗_(西夏)>];1139-06"],			# 3219-15
      ["[人慶]1"	,			""	,	"	     1144"],
      ["[天盛]1"	,			""	,	"	     1149"],
      ["[乾祐=ja:%%<乾祐_(西夏)>]1"	,	""	,	'	 #{N:1171}'],
      ["[天慶=ja:%%<天慶_(西夏)>]1","@A",    "name=[桓宗=ja:%%<桓宗_(西夏)>];1194"],
      ["[応天=ja:%%<応天_(西夏)>]1","@A",    "name=[襄宗=ja:%%<襄宗_(西夏)>];1206-01"],			# 4235-12
      ["[皇建=ja:%%<皇建_(西夏)>]1"	,	""	,	"	     1210"],
      ["[光定=ja:%%<光定_(西夏)>]1","@A",    "name=[神宗=ja:%%<神宗_(西夏)>];1211-08"],			# 4302-26
      ["[乾定]1"	,	"@A"	,    "name=[献宗=ja:%%<献宗_(西夏)>];1223-12", "1226-7="]	# 4450-35(1226-07)
    ]]

    #
    # 西遼
    #
    KaraKhitan = [{
        'V'=>{'1124'  => {'P'=>'1124-02-05'},
              '1131'  => {'P'=>'1131', 'Q'=>'1134', 'R'=>'1144', 'S'=>'1151', 'T'=>'1164', 'U'=>'1178'},
              '1132'  => {'P'=>'1132', 'Q'=>'1134', 'R'=>'1144', 'S'=>'1151', 'T'=>'1164', 'U'=>'1178'}}
      }, self, [
      "locale:[=ja:, en=en:, alias]",
      'area:[西遼#{?V=V}=ja:%%<西遼>, Kara-Khitan_Khanate#{?V=V}=en:Kara-Khitan_Khanate]',
      ["[延慶=ja:%%<延慶_(西遼)>]1","@A",  'name=[徳宗=ja:%%<耶律大石>];#{P:1125-02-05}^Chinese0939'],	# 1125-02-05^Chinese0939"], # 2483-12 Wikipeda
      ["[康国]1"	,	"",				       '#{Q:1127}'],			# 1127 -> 1134 Wikipedia
      ["[咸清]1"	,	"@A",    'name=[感天后=zh:%%<蕭塔不煙>];#{R:1136-11}'],			# 1136 -> 1144 Wikipedia
      ["[紹興=ja:%%<紹興_(西遼)>]1","@A",  'name=[仁宗=ja:%%<耶律夷列>];#{S:1142-11}'],			# 1142 -> 1151 Wikipedia
      ["[崇福]1"	,	"@A",    'name=[承天后=ja:%%<プスワン>];#{T:1154-12}'],			# 1154 -> 1164 Wikipedia
      ["[天禧=ja:%%<天禧_(西遼)>]1","@A",'name=[末主=ja:%%<耶律直魯古>];#{U:1168}', "1211="]		# 1168 -> 1178 Wikipedia
    ]]
  end
end
