# -*- coding: utf-8 -*-
=begin
  Copyright (C) 2011-2020 Takashi SUGA

  You may use and/or modify this file according to the license described in the LICENSE.txt file included in this archive.
=end

class When::TM::OrdinalReferenceSystem

  #
  # Geological Ages - http://www.stratigraphy.org/ICSchart/ChronostratChart2020-01.pdf
  #
  GeologicalAge = [self, [
    "locale:[=ja:, *en=en:]",
    "*area:[地球,Earth]",

    [When::TM::OrdinalEra,
      "*eon:[冥王代,Hadean]",
      "begin:-4_600_000_000-"
    ],
    [When::TM::OrdinalEra,
      "namespace:[root=https://ja.wikipedia.org/wiki/%%<始生代>#]",
      "*eon:[始生代,Archean]",
      [When::TM::OrdinalEra,
        "*era:[原始生代=root:%.<原始生代_(Eoarchean)>,Eoarchean]",
        "begin:-4_000_000_000-"
      ],
      [When::TM::OrdinalEra,
        "*era:[古始生代=root:%.<古始生代_(Paleoarchean)>,Paleoarchean]",
        "begin:-3_600_000_000-"
      ],
      [When::TM::OrdinalEra,
        "*era:[中始生代=root:%.<中始生代_(Mesoarchean)>,Mesoarchean]",
        "begin:-3_200_000_000-"
      ],
      [When::TM::OrdinalEra,
        "*era:[新始生代=root:%.<新始生代_(Neoarchean)>,Neoarchean]",
        "begin:-2_800_000_000-"
      ]
    ],
    [When::TM::OrdinalEra,
      "*eon:[原生代,Proterozoic]",
      [When::TM::OrdinalEra,
        "*era:[前期=,Paleoproterozoic]",
        [When::TM::OrdinalEra,
          "*period:[シデリアン,Siderian]",
          "begin:-2_500_000_000-"
        ],
        [When::TM::OrdinalEra,
          "*period:[リアキアン,Rhyacian]",
          "begin:-2_300_000_000-"
        ],
        [When::TM::OrdinalEra,
          "*period:[オロシリアン,Orosirian]",
          "begin:-2_050_000_000-"
        ],
        [When::TM::OrdinalEra,
          "*period:[スタテリアン,Statherian]",
          "begin:-1_800_000_000-"
        ],
      ],
      [When::TM::OrdinalEra,
        "*era:[中期=,Mesoproterozoic]",
        [When::TM::OrdinalEra,
          "*period:[カリミアン,Calymmian]",
          "begin:-1_600_000_000-"
        ],
        [When::TM::OrdinalEra,
          "*period:[エクタシアン,Ectasian]",
          "begin:-1_400_000_000-"
        ],
        [When::TM::OrdinalEra,
          "*period:[ステニアン,Stenian]",
          "begin:-1_200_000_000-"
        ],
      ],
      [When::TM::OrdinalEra,
        "*era:[後期=,Neoproterozoic]",
        [When::TM::OrdinalEra,
          "*period:[トニアン,Tonian]",
          "begin:-1_000_000_000-"
        ],
        [When::TM::OrdinalEra,
          "*period:[クリオジェニアン,Cryogenian]",
          "begin:-720_000_000-"
        ],
        [When::TM::OrdinalEra,
          "*period:[エディアカラン,Ediacaran]",
          "begin:-635_000_000-"
        ]
      ]
    ],
    [When::TM::OrdinalEra,
      "*eon:[顕生代,Phanerozoic]",
      [When::TM::OrdinalEra,
        "*era:[古生代,Paleozoic]",
        [When::TM::OrdinalEra,
          "*period:[カンブリア紀,Cambrian]",
          [When::TM::OrdinalEra,
            "*epoch:[テレニュービアン=,Terreneuvian]",
            [When::TM::OrdinalEra,
              "*age:[フォルツニアン=,Fortunian]",
              "begin:-541_000_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[第2期=,Age_2=en_wiki:Cambrian_Stage_2]",
              "begin:-529_000_000-"
            ]
          ],
          [When::TM::OrdinalEra,
            "*epoch:[第2世=,Epoch_2=en_wiki:Cambrian_Series_2]",
            [When::TM::OrdinalEra,
              "*age:[第3期=,Age_3=en_wiki:Cambrian_Stage_3]",
              "begin:-521_000_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[第4期=,Age_4=en_wiki:Cambrian_Stage_4]",
              "begin:-514_000_000-"
            ],
          ],
          [When::TM::OrdinalEra,
            "*epoch:[ミャオリンギアン=,Miaolingian]",
            [When::TM::OrdinalEra,
              "*age:[ウリウアン=,Wuliuan]",
              "begin:-509_000_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[ドルミアン=,Drumian]",
              "begin:-504_500_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[グザンギアン=,Guzhangian]",
              "begin:-500_500_000-"
            ]
          ],
          [When::TM::OrdinalEra,
            "*epoch:[フロンギアン=,Furongian]",
            [When::TM::OrdinalEra,
              "*age:[パイビアン=,Paibian]",
              "begin:-497_000_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[ジャングシャニアン=,Jiangshanian]",
              "begin:-494_000_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[第10期=,Age_10=en_wiki:Cambrian_Stage_10]",
              "begin:-489_500_000-"
            ]
          ]
        ],
        [When::TM::OrdinalEra,
          "*period:[オルドビス紀,Ordovician]",
          [When::TM::OrdinalEra,
            "*epoch:[前期=,Early_Ordovician]",
            [When::TM::OrdinalEra,
              "*age:[トレマドック=,Tremadocian]",
              "begin:-485_400_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[フロイアン=,Floian]",
              "begin:-477_700_000-"
            ]
          ],
          [When::TM::OrdinalEra,
            "*epoch:[中期=,Middle_Ordovician]",
            [When::TM::OrdinalEra,
              "*age:[ダピンギアン=,Dapingian]",
              "begin:-470_000_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[ダーリウィリアン=,Darriwilian]",
              "begin:-467_300_000-"
            ]
          ],
          [When::TM::OrdinalEra,
            "*epoch:[後期=,Late_Ordovician]",
            [When::TM::OrdinalEra,
              "*age:[サンドビアン=,Sandbian]",
              "begin:-458_400_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[カティアン=,Katian]",
              "begin:-453_000_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[ヒルナンティアン=,Hirnantian]",
              "begin:-445_200_000-"
            ]
          ]
        ],
        [When::TM::OrdinalEra,
          "*period:[シルル紀,Silurian]",
          [When::TM::OrdinalEra,
            "*epoch:[ランドベリ=,Llandovery]",
            [When::TM::OrdinalEra,
              "*age:[ルダニアン=,Rhuddanian]",
              "begin:-443_800_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[アエロニアン=,Aeronian]",
              "begin:-440_800_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[テリチアン=,Telychian]",
              "begin:-438_500_000-"
            ]
          ],
          [When::TM::OrdinalEra,
            "*epoch:[ウェンロック=,Wenlock]",
            [When::TM::OrdinalEra,
              "*age:[シェインウッディアン=,Sheinwoodian]",
              "begin:-433_400_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[ホメリアン=,Homerian]",
              "begin:-430_500_000-"
            ]
          ],
          [When::TM::OrdinalEra,
            "*epoch:[ラドロウ=,Ludlow]",
            [When::TM::OrdinalEra,
              "*age:[ゴースティアン=,Gorstian]",
              "begin:-427_400_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[ルドフォーディアン=,Ludfordian]",
              "begin:-425_600_000-"
            ]
          ],
          [When::TM::OrdinalEra,
            "*epoch:[プリドリ=,Pridoli=en_wiki:Pridoli_epoch#Pridoli]",
            "begin:-423_000_000-"
          ]
        ],
        [When::TM::OrdinalEra,
          "*period:[デボン紀,Devonian]",
          [When::TM::OrdinalEra,
            "*epoch:[前期=,Early_Devonian]",
            [When::TM::OrdinalEra,
              "*age:[ロホコビアン=,Lochkovian]",
              "begin:-419_200_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[プラーギアン=,Pragian]",
              "begin:-410_800_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[エムシアン=,Emsian]",
              "begin:-407_600_000-"
            ]
          ],
          [When::TM::OrdinalEra,
            "*epoch:[中期=,Middle_Devonian]",
            [When::TM::OrdinalEra,
              "*age:[アイフェリアン=,Eifelian]",
              "begin:-393_300_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[ジベーティアン=,Givetian]",
              "begin:-387_700_000-"
            ]
          ],
          [When::TM::OrdinalEra,
            "*epoch:[後期=,Late_Devonian]",
            [When::TM::OrdinalEra,
              "*age:[フラスニアン=,Frasnian]",
              "begin:-382_700_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[ファメニアン=,Famennian]",
              "begin:-372_200_000-"
            ]
          ]
        ],
        [When::TM::OrdinalEra,
          "*period:[石炭紀,Carboniferous]",
          [When::TM::OrdinalEra,
            "*subperiod:[ミシシッピアン=,Mississippian]",
            [When::TM::OrdinalEra,
              "*age:[トゥルネージアン=,Tournaisian]",
              "begin:-358_900_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[ヴィゼアン=,Visean]",
              "begin:-346_700_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[サープコビアン=,Serpukhovian]",
              "begin:-330_900_000-"
            ]
          ],
          [When::TM::OrdinalEra,
            "*subperiod:[ペンシルバニアン=,Pennsylvanian]",
            [When::TM::OrdinalEra,
              "*age:[バシュキーリアン=,Bashkirian]",
              "begin:-323_200_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[モスコビアン=,Moscovian]",
              "begin:-315_200_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[カシモービアン=,Kasimovian]",
              "begin:-307_000_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[グゼーリアン=,Gzhelian]",
              "begin:-303_700_000-"
            ]
          ]
        ],
        [When::TM::OrdinalEra,
          "*period:[ペルム紀,Permian]",
          [When::TM::OrdinalEra,
            "*epoch:[キスラリアン=,Cisuralian]",
            [When::TM::OrdinalEra,
              "*age:[アッセリアン=,Asselian]",
              "begin:-298_900_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[サクマーリアン=,Sakmarian]",
              "begin:-293_520_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[アルチンスキアン=,Artinskian]",
              "begin:-290_100_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[クングーリアン=,Kungurian]",
              "begin:-283_500_000-"
            ]
          ],
          [When::TM::OrdinalEra,
            "*epoch:[ガダリューピアン=,Guadalupian]",
            [When::TM::OrdinalEra,
              "*age:[ローディアン=,Roadian]",
              "begin:-272_950_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[ワーディアン=,Wordian]",
              "begin:-268_800_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[カピタニアン=,Capitanian]",
              "begin:-265_100_000-"
            ]
          ],
          [When::TM::OrdinalEra,
            "*epoch:[ロピンギアン=,Lopingian]",
            [When::TM::OrdinalEra,
              "*age:[ウキアピンギアン=,Wuchiapingian]",
              "begin:-259_100_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[チャンシンギアン=,Changhsingian]",
              "begin:-254_140_000-"
            ]
          ]
        ]
      ],
      [When::TM::OrdinalEra,
        "*era:[中生代,Mesozoic]",
        [When::TM::OrdinalEra,
          "*period:[三畳紀,Triassic]",
          [When::TM::OrdinalEra,
            "*epoch:[前期=,Early_Triassic]",
            [When::TM::OrdinalEra,
              "*age:[インドゥアン=,Induan]",
              "begin:-251_902_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[オレネキアン=,Olenekian]",
              "begin:-251_200_000-"
            ]
          ],
          [When::TM::OrdinalEra,
            "*epoch:[中期=,Middle_Triassic]",
            [When::TM::OrdinalEra,
              "*age:[アニシアン=,Anisian]",
              "begin:-247_200_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[ラディニアン=,Ladinian]",
              "begin:-242_000_000-"
            ]
          ],
          [When::TM::OrdinalEra,
            "*epoch:[後期=,Late_Triassic]",
            [When::TM::OrdinalEra,
              "*age:[カーニアン=,Carnian]",
              "begin:-237_000_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[ノーリアン=,Norian]",
              "begin:-227_000_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[レーティアン=,Rhaetian]",
              "begin:-208_500_000-"
            ]
          ]
        ],
        [When::TM::OrdinalEra,
          "*period:[ジュラ紀,Jurassic]",
          [When::TM::OrdinalEra,
            "*epoch:[前期=,Early_Jurassic]",
            [When::TM::OrdinalEra,
              "*age:[ヘッタンジアン=,Hettangian]",
              "begin:-201_300_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[シネムーリアン=,Sinemurian]",
              "begin:-199_300_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[プリンスバッキアン=,Pliensbachian]",
              "begin:-190_800_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[トアルシアン=,Toarcian]",
              "begin:-182_700_000-"
            ]
          ],
          [When::TM::OrdinalEra,
            "*epoch:[中期=,Middle_Jurassic]",
            [When::TM::OrdinalEra,
              "*age:[アーレニアン=,Aalenian]",
              "begin:-174_100_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[バジョシアン=,Bajocian]",
              "begin:-170_300_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[バトニアン=,Bathonian]",
              "begin:-168_300_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[カロビアン=,Callovian]",
              "begin:-166_100_000-"
            ]
          ],
          [When::TM::OrdinalEra,
            "*epoch:[後期=,Late_Jurassic]",
            [When::TM::OrdinalEra,
              "*age:[オクスフォーディアン=,Oxfordian]",
              "begin:-163_500_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[キンメリッジアン=,Kimmeridgian]",
              "begin:-157_300_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[チトニアン=,Tithonian]",
              "begin:-152_100_000-"
            ]
          ]
        ],
        [When::TM::OrdinalEra,
          "*period:[白亜紀,Cretaceous]",
          [When::TM::OrdinalEra,
            "*epoch:[前期= ,Early_Cretaceous]",
            [When::TM::OrdinalEra,
              "*age:[ベリアシアン,Berriasian]",
              "begin:-145_000_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[バランギニアン,Valanginian]",
              "begin:-139_800_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[オーテリビアン,Hauterivian]",
              "begin:-132_600_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[バレミアン,Barremian]",
              "begin:-129_400_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[アプチアン,Aptian]",
              "begin:-125_000_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[アルビアン,Albian]",
              "begin:-113_000_000-"
            ]
          ],
          [When::TM::OrdinalEra,
            "*epoch:[後期=,Late_Cretaceous]",
            [When::TM::OrdinalEra,
              "*age:[セノマニアン=Cenomanian]",
              "begin:-100_500_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[チューロニアン,Turonian]",
              "begin:-93_900_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[コニアシアン,Coniacian]",
              "begin:-89_800_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[サントニアン,Santonian]",
              "begin:-86_300_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[カンパニアン,Campanian]",
              "begin:-83_600_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[マーストリヒチアン,Maastrichtian]",
              "begin:-72_100_000-"
            ]
          ]
        ]
      ],
      [When::TM::OrdinalEra,
        "*era:[新生代,Cenozoic]",
        [When::TM::OrdinalEra,
          "*period:[古第三紀,Paleogene]",
          [When::TM::OrdinalEra,
            "*epoch:[暁新世,Paleocene]",
            [When::TM::OrdinalEra,
              "*age:[ダニアン,Danian]",
              "begin:-66_000_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[セランディアン,Selandian]",
              "begin:-61_600_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[サネティアン,Thanetian]",
              "begin:-59_200_000-"
            ]
          ],
          [When::TM::OrdinalEra,
            "*epoch:[始新世,Eocene]",
            [When::TM::OrdinalEra,
              "*age:[ヤプレシアン,Ypresian]",
              "begin:-56_000_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[ルテシアン,Lutetian]",
              "begin:-47_800_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[バートニアン,Bartonian]",
              "begin:-41_200_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[プリアボニアン,Priabonian]",
              "begin:-37_710_000-"
            ]
          ],
          [When::TM::OrdinalEra,
            "*epoch:[漸新世,Oligocene]",
            [When::TM::OrdinalEra,
              "*age:[ルペリアン,Rupelian]",
              "begin:-33_900_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[チャッティアン,Chattian]",
              "begin:-27_820_000-"
            ]
          ]
        ],
        [When::TM::OrdinalEra,
          "*period:[新第三紀,Neogene]",
          [When::TM::OrdinalEra,
            "*epoch:[中新世,Miocene]",
            [When::TM::OrdinalEra,
              "*age:[アキタニアン,Aquitanian]",
              "begin:-23_030_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[バーディガリアン,Burdigalian]",
              "begin:-20_440_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[ランギアン,Langhian]",
              "begin:-15_970_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[サーラバリアン,Serravallian]",
              "begin:-13_820_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[トートニアン,Tortonian]",
              "begin:-11_630_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[メッシニアン,Messinian]",
              "begin:-7_244_000-"
            ]
          ],
          [When::TM::OrdinalEra,
            "*epoch:[鮮新世,Pliocene]",
            [When::TM::OrdinalEra,
              "*age:[ザンクリアン,Zanclean]",
              "begin:-5_331_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[ピアセンジアン,Piacenzian]",
              "begin:-3_598_000-"
            ]
          ]
        ],
        [When::TM::OrdinalEra,
          "*period:[第四紀,Quaternary]",
          [When::TM::OrdinalEra,
            "*epoch:[更新世,Pleistocene]",
            [When::TM::OrdinalEra,
              "*age:[ジェラシアン,Gelasian]",
              "begin:-2_578_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[カラブリアン,Calabrian]",
              "begin:-1_798_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[チバニアン,Chibanian]",
              "begin:-772_000-"
            ],
            [When::TM::OrdinalEra,
              "*age:[後期更新世,Late_Pleistocene]",
              "begin:-127_000-"
            ]
          ],
          [When::TM::OrdinalEra,
            "*epoch:[完新世,Holocene]",
           [When::TM::OrdinalEra,
              "*age:[グリーンランディアン,Greenlandian]",
              "begin:-9_700-",
            ],
            [When::TM::OrdinalEra,
              "*age:[ノースグリッピアン,Northgrippian]",
              "begin:-6_200-",
            ],
            [When::TM::OrdinalEra,
              "*age:[メーガーラヤン,Meghalayan]",
              "begin:-2_200-",
              "end:+Infinity"
            ]
          ]
        ]
      ]
    ]
  ]]
end
