# -*- coding: utf-8 -*-
=begin
  Copyright (C) 2014-2016 Takashi SUGA

  You may use and/or modify this file according to the license
  described in the LICENSE.txt file included in this archive.
=end

module When::CalendarNote::Japanese::Eclipse

  #
  # 日月食情報
  #

  Eclipses = {
    # 実際の仮名暦のデータ(読み下し文は大崎正次編『近世日本天文史料』による)
    "1730.06.01"=>["日そく八分",     "むまの七刻西の方よりかけはしめひつしの四刻甚<br/>申の二刻東の方に終"], # 京都で金環
    "1742.05.01"=>["日そく八分",     "たつの八刻西の方よりかけはしめミの初刻甚<br/>ミの五刻東の方におはる"], # 京都で皆既
    "1763.09.01"=>nil, # 予報が外れたと問題視された日食
    "1786.01.01"=>["日そく皆既",     "むまの一刻西の方よりかけはしめむまの六刻<br/>甚しくひつしの二刻東の方ニおはる"], # 紀伊半島で皆既、京都は深い部分食
    "1803.12.14"=>["月帯そく四分",   "翌とらの八刻左の下よりかけはしめうの五刻左と下の<br/>間ニ甚しくうの八刻はかりかけなから入"],
    "1843.11.01"=>["日そく七分",     "未の五刻下の右よりかけはしめ申の一刻<br/>左の下に甚しく申の六刻上と左の間におはる"],
    "1844.10.16"=>["月帯そく",       "京都にてハ見へす西国にてハ明六時ニ分上と左の間<br/>よりかけはしめ六時三分ニ分はかりかけなから入"],
    "1845.04.16"=>["月そく皆既",     "四時六分左と下の間よりかけはしめ九時七分<br/>甚しく八時八分右と下の間におはる"],
    "1852.11.01"=>["日そく九分半余", "朝四時七分右の上よりかけはしめ昼九時五分<br/>甚しく八時三分左の上におはる"], # 京都で皆既

    # 宣明暦の日食データ - http://www.wagoyomi.info/sennmyou_nissyokuhyou_v3.html に基づく(主に南半球で見える日食は“()”で囲んでいる)
    "0862.02.01"=>["夜日蝕七分"],		"0862.08.01"=>["(日蝕一分)"],		"0863.02.01"=>["(夜日蝕ニ分)"],
    "0863.07.01"=>["日蝕九分半余"],		"0864.01.01"=>["(日蝕三分)"],		"0864.07.01"=>["日蝕三分"],
    "0864.12.01"=>["夜日蝕一分"],		"0865.06.01"=>["(日帯蝕七分)"],		"0865.12.01"=>["夜日蝕六分"],
    "0866.05.01"=>["日蝕ニ分"],			"0866.11.01"=>["(日帯蝕九分)"],		"0867.05.01"=>["日蝕ニ分"],
    "0867.11.01"=>["(日蝕三分)"],		"0868.04.01"=>["(日帯蝕七分)"],		"0868.10.01"=>["夜日蝕八分"],
    "0869.03.01"=>["(夜日蝕四分)"],		"0869.09.01"=>["日蝕九分"],		"0870.03.01"=>["夜日蝕ニ分"],
    "0870.09.01"=>["(夜日蝕四分)"],		"0871.03.01"=>["日帯蝕六分"],		"0872.07.01"=>["日蝕一分"],
    "0873.12.30"=>["(日蝕ニ分)"],		"0873.07.01"=>["日蝕九分半余"],		"0874.12.30"=>["日帯蝕七分"],
    "0874.06.01"=>["(日蝕七分)"],		"0875.06.01"=>["(夜日蝕ニ分)"],		"0875.11.01"=>["(夜日蝕四分)"],
    "0876.05.01"=>["日蝕六分"],			"0876.11.01"=>["(夜日蝕ニ分)"],		"0877.04.01"=>["(夜日蝕七分)"],
    "0877.10.01"=>["日帯蝕九分"],		"0878.04.01"=>["(日蝕五分)"],		"0878.10.01"=>["夜日蝕九分"],
    "0879.09.01"=>["(夜日蝕九分)"],		"0880.02.01"=>["夜日蝕八分"],		"0880.08.01"=>["(日蝕九分)"],
    "0881.02.01"=>["(日蝕八分)"],		"0881.08.01"=>["夜日蝕四分"],		"0882.07=01"=>["夜日蝕九分"],
    "0882.12.01"=>["夜日蝕六分"],		"0883.06.01"=>["(日蝕七分)"],		"0883.12.01"=>["日蝕五分"],
    "0884.06.01"=>["(夜日蝕ニ分)"],		"0884.12.01"=>["(日帯蝕九分半余)"],	"0885.05.01"=>["日帯蝕五分"],
    "0885.11.01"=>["(夜日蝕ニ分)"],		"0887.04.01"=>["(日蝕九分)"],		"0887.10.01"=>["夜日蝕一分にミたす"],
    "0888.03.01"=>["日蝕九分半余"],		"0888.09.01"=>["(日蝕九分半余)"],	"0889.03.01"=>["日蝕一分"],
    "0889.09.01"=>["(夜日蝕九分半余)"],		"0890.02.01"=>["(日蝕一分にミたす)"],	"0890.08.01"=>["日帯蝕七分"],
    "0891.01.02"=>["(夜日蝕三分)"],		"0891.07.02"=>["日帯蝕三分"],		"0892.12.30"=>["日帯蝕六分"],
    "0892.07.01"=>["(夜日蝕ニ分)"],		"0893.01.01"=>["夜日蝕六分"],		"0893.06.01"=>["(日蝕三分)"],
    "0894.05.02"=>["日帯蝕六分"],		"0894.11.01"=>["(日蝕七分)"],		"0895.05.01"=>["(日蝕一分)"],
    "0895.11.01"=>["夜日蝕ニ分"],		"0896.04.01"=>["(日帯蝕七分)"],		"0897.03.01"=>["日帯蝕六分"],
    "0897.09.01"=>["(日蝕ニ分)"],		"0898.03.01"=>["日蝕六分"],		"0898.09.01"=>["(夜日蝕ニ分)"],
    "0899.02.01"=>["(夜日蝕一分にミたす)"],	"0899.08.01"=>["夜日蝕五分"],		"0901.01.01"=>["日蝕三分"],
    "0901.12.01"=>["夜日蝕七分"],		"0902.06.01"=>["(日蝕ニ分)"],		"0902.12.01"=>["(夜日蝕七分)"],
    "0903.06.01"=>["夜日蝕五分"],		"0903.12.01"=>["(夜日蝕一分)"],		"0904.04.01"=>["(日蝕一分にミたす)"],
    "0904.10.01"=>["日蝕三分"],			"0905.04.01"=>["(日蝕四分)"],		"0905.10.01"=>["夜日蝕三分"],
    "0906.04.01"=>["日帯蝕七分"],		"0906.10.01"=>["(夜日蝕五分)"],		"0907.09.01"=>["(日蝕四分)"],
    "0908.02.01"=>["(日帯蝕六分)"],		"0909.02.01"=>["(日蝕八分)"],		"0909.08.01"=>["夜日蝕三分"],
    "0910.01.01"=>["夜日蝕六分"],		"0910.07.01"=>["(日帯蝕一分)"],		"0911.01.01"=>["日蝕三分"],
    "0911.12.01"=>["(夜日蝕三分)"],		"0912.05=01"=>["夜日蝕六分"],		"0912.11.01"=>["(夜日蝕九分)"],
    "0913.05.01"=>["(日蝕一分)"],		"0913.11.01"=>["夜日蝕八分"],		"0914.05.01"=>["(夜日蝕一分)"],
    "0914.11.01"=>["日蝕五分"],			"0916.03.01"=>["日帯蝕六分"],		"0916.09.01"=>["(日帯蝕一分)"],
    "0917.03.01"=>["(夜日蝕四分)"],		"0917.09.01"=>["日蝕九分"],		"0918.02.01"=>["(日帯蝕五分)"],
    "0918.08.01"=>["日蝕三分"],			"0919.07.01"=>["(夜日蝕八分)"],		"0920.01.01"=>["夜日蝕一分にミたす"],
    "0920.06=01"=>["(日蝕七分)"],		"0920.12.01"=>["(日蝕七分)"],		"0921.06.01"=>["日蝕五分"],
    "0921.12.01"=>["(日蝕一分にミたす)"],	"0922.05.01"=>["(日蝕ニ分)"],		"0922.11.01"=>["夜日蝕六分"],
    "0923.04=01"=>["(夜日蝕ニ分)"],		"0923.10.01"=>["日蝕七分"],		"0924.04.01"=>["夜日蝕九分半余"],
    "0924.10.01"=>["(夜日蝕四分)"],		"0925.04.01"=>["夜日蝕九分半余"],	"0927.02.01"=>["(日蝕一分にミたす)"],
    "0927.08.01"=>["日蝕七分"],			"0928.02.01"=>["日蝕六分"],		"0928.08.01"=>["(日蝕八分)"],
    "0929.12.01"=>["(夜日蝕一分にミたす)"],	"0930.06.01"=>["日蝕三分"],		"0930.12.01"=>["(夜日蝕九分)"],
    "0931.05=01"=>["(夜日蝕一分にミたす)"],	"0931.11.01"=>["日蝕三分"],		"0932.05.01"=>["(日蝕七分)"],
    "0932.11.01"=>["夜日蝕六分"],		"0933.10.01"=>["(夜日蝕七分)"],		"0934.03.01"=>["夜日蝕六分"],
    "0934.09.01"=>["(日蝕七分)"],		"0935.03.01"=>["(日蝕九分)"],		"0935.09.01"=>["夜日蝕五分"],
    "0936.09.01"=>["夜日蝕九分"],		"0937.01.02"=>["日帯蝕六分"],		"0937.07.01"=>["(日蝕四分)"],
    "0938.01.02"=>["日帯蝕九分"],		"0938.07.01"=>["(夜日蝕四分)"],		"0939.01.01"=>["(夜日蝕ニ分)"],
    "0939.07.01"=>["日蝕九分半余"],		"0939.12.01"=>["(夜日蝕四分)"],		"0940.05.01"=>["(夜日蝕一分にミたす)"],
    "0941.05.01"=>["(日帯蝕六分)"],		"0941.11.01"=>["夜日蝕三分"],		"0942.04.01"=>["日蝕一分にミたす"],
    "0942.10.01"=>["(日蝕九分)"],		"0943.04.01"=>["日蝕四分"],		"0943.10.01"=>["(夜日蝕九分)"],
    "0944.09.01"=>["日蝕三分"],			"0945.02.01"=>["(夜日蝕三分)"],		"0945.08.01"=>["日蝕六分"],
    "0946.02.01"=>["日帯蝕六分"],		"0946.08.01"=>["(夜日蝕五分)"],		"0947.02.01"=>["夜日蝕八分"],
    "0947.07=01"=>["(日蝕六分)"],		"0948.06.01"=>["日蝕五分"],		"0948.12.01"=>["(日蝕八分)"],
    "0949.06.01"=>["日蝕六分"],			"0949.12.01"=>["夜日蝕九分"],		"0950.05=01"=>["(日蝕八分)"],
    "0950.11.01"=>["日帯蝕ニ分"],		"0951.04.01"=>["夜日蝕五分"],		"0952.04.01"=>["日蝕八分"],
    "0952.10.01"=>["(夜日蝕一分)"],		"0953.03.01"=>["(日帯蝕三分)"],		"0953.09.01"=>["夜日蝕五分"],
    "0954.03.01"=>["(日蝕一分)"],		"0955.02.01"=>["日蝕一分にミたす"],	"0956.01.01"=>["夜日蝕ニ分"],
    "0956.07.01"=>["(日蝕一分にミたす)"],	"0957.01.01"=>["(夜日蝕ニ分)"],		"0957.07.01"=>["夜日蝕ニ分"],
    "0958.07.01"=>["日蝕一分"],			"0958.11.01"=>["日帯蝕一分"],		"0959.05.01"=>["(日蝕九分半余)"],
    "0959.11.01"=>["夜日蝕八分"],		"0960.05.01"=>["日蝕八分"],		"0960.11.01"=>["(夜日蝕七分)"],
    "0961.04.01"=>["日蝕ニ分"],			"0961.10.01"=>["(日蝕三分)"],		"0962.03.01"=>["(日蝕五分)"],
    "0962.09.01"=>["夜日蝕八分"],		"0963.03.01"=>["(日蝕八分)"],		"0963.09.01"=>["夜日蝕五分"],
    "0964.02.01"=>["夜日蝕七分"],		"0964.08.01"=>["(夜日蝕七分)"],		"0965.02.01"=>["日蝕三分"],
    "0965.08.01"=>["(日蝕一分)"],		"0966.01.01"=>["(夜日蝕七分)"],		"0966.12.01"=>["(夜日蝕一分にミたす)"],
    "0967.06.01"=>["日蝕七分"],			"0967.12.01"=>["夜日蝕三分"],		"0968.06.01"=>["(夜日蝕六分)"],
    "0968.12.01"=>["日帯蝕ニ分"],		"0970.04.01"=>["日蝕五分"],		"0970.10.01"=>["(日帯蝕三分)"],
    "0971.04.01"=>["(夜日蝕五分)"],		"0971.10.01"=>["日蝕五分"],		"0972.03.01"=>["(日帯蝕七分)"],
    "0972.09.01"=>["日蝕四分"],			"0973.08.01"=>["(夜日蝕八分)"],		"0974.02.01"=>["日蝕一分にミたす"],
    "0974.08.01"=>["(日蝕九分)"],		"0975.01.01"=>["(日蝕八分)"],		"0975.07.01"=>["日蝕七分"],
    "0976.07.01"=>["夜日蝕八分"],		"0976.12.01"=>["夜日蝕ニ分"],		"0977.06.01"=>["(日帯蝕五分)"],
    "0977.11.01"=>["日帯蝕九分"],		"0978.05.01"=>["(夜日蝕一分)"],		"0978.11.01"=>["(日帯蝕四分)"],
    "0979.05.01"=>["夜日蝕三分"],		"0979.11.01"=>["(夜日蝕一分にミたす)"],	"0981.03.01"=>["(日帯蝕ニ分)"],
    "0981.09.01"=>["日帯蝕四分"],		"0982.03.01"=>["日蝕四分"],		"0982.09.01"=>["(日蝕四分)"],
    "0985.01.01"=>["(夜日蝕六分)"],		"0985.07.01"=>["夜日蝕四分"],		"0985.12.01"=>["日蝕三分"],
    "0986.06.01"=>["(日帯蝕八分)"],		"0986.12.01"=>["夜日蝕六分"],		"0987.06.01"=>["(日蝕一分)"],
    "0987.11.01"=>["(夜日蝕三分)"],		"0988.05.01"=>["夜日蝕ニ分"],		"0988.10.01"=>["(日蝕六分)"],
    "0989.04.01"=>["(日蝕五分)"],		"0989.10.01"=>["夜日蝕六分"],		"0990.10.01"=>["夜日蝕九分"],
    "0991.02=01"=>["日帯蝕五分"],		"0991.08.01"=>["(日蝕三分)"],		"0992.02.01"=>["日蝕八分"],
    "0992.08.01"=>["(夜日蝕ニ分)"],		"0993.02.01"=>["(日蝕一分)"],		"0993.08.01"=>["日蝕三分"],
    "0994.01.01"=>["(日帯蝕七分)"],		"0994.12.01"=>["日蝕三分"],		"0995.06.01"=>["(夜日蝕七分)"],
    "0995.12.01"=>["夜日蝕九分"],		"0996.06.01"=>["(夜日蝕ニ分)"],		"0996.11.01"=>["(日帯蝕八分)"],
    "0997.05.01"=>["日蝕七分"],			"0997.11.01"=>["(夜日蝕五分)"],		"0998.10.01"=>["日蝕ニ分"],
    "0999.03=01"=>["(夜日蝕一分)"],		"0999.09.01"=>["日蝕九分半余"],		"1000.03.01"=>["日帯蝕八分"],
    "1000.09.01"=>["(夜日蝕七分)"],		"1001.03.01"=>["夜日蝕九分半余"],	"1001.09.01"=>["(日帯蝕八分)"],
    "1002.01.01"=>["(日蝕一分にミたす)"],	"1002.07.01"=>["日蝕四分"],		"1003.01.01"=>["(日蝕九分半余)"],
    "1003.07.01"=>["日帯蝕六分"],		"1004.01.01"=>["夜日蝕五分"],		"1004.07.01"=>["(日蝕六分)"],
    "1004.12.01"=>["日蝕五分"],			"1005.06.01"=>["(日蝕三分)"],		"1006.05.01"=>["日帯蝕八分"],
    "1006.11.01"=>["(夜日蝕四分)"],		"1007.05.01"=>["(日蝕五分)"],		"1007.10.01"=>["夜日蝕七分"],
    "1008.04.01"=>["(日蝕四分)"],		"1010.02=01"=>["夜日蝕ニ分"],		"1010.08.01"=>["(日帯蝕四分)"],
    "1011.02.01"=>["(夜日蝕ニ分)"],		"1011.08.01"=>["夜日蝕九分"],		"1012.08.01"=>["日蝕三分"],
    "1013.06.01"=>["(日蝕六分)"],		"1013.12.01"=>["日蝕六分"],		"1014.06.01"=>["(日蝕一分にミたす)"],
    "1014.12.01"=>["(夜日蝕九分半余)"],		"1015.06.01"=>["日蝕五分"],		"1016.04.01"=>["(日蝕ニ分)"],
    "1016.10.01"=>["夜日蝕七分"],		"1017.04.01"=>["(日蝕七分)"],		"1017.10.01"=>["夜日蝕一分"],
    "1018.04.01"=>["夜日蝕九分"],		"1018.09.01"=>["(夜日蝕九分半余)"],	"1019.03.01"=>["日蝕五分"],
    "1019.09.01"=>["(日蝕ニ分)"],		"1020.02.01"=>["(夜日蝕七分)"],		"1020.08.01"=>["夜日蝕九分"],
    "1021.01.01"=>["(夜日蝕四分)"],		"1021.07.01"=>["日蝕八分"],		"1022.01.01"=>["夜日蝕三分"],
    "1022.07.01"=>["(日帯蝕六分)"],		"1023.11.01"=>["(夜日蝕一分にミたす)"],	"1024.05.01"=>["日蝕ニ分"],
    "1024.11.01"=>["(日帯蝕九分半余)"],		"1025.05.01"=>["夜日蝕八分"],		"1025.11.01"=>["日蝕一分"],
    "1026.05.01"=>["(日蝕八分)"],		"1026.10.01"=>["日蝕五分"],		"1027.09.01"=>["(夜日蝕七分)"],
    "1028.03.01"=>["夜日蝕一分"],		"1028.09.01"=>["(日蝕六分)"],		"1029.02=01"=>["(日帯蝕八分)"],
    "1029.08.01"=>["日帯蝕九分"],		"1030.02.01"=>["(日蝕一分)"],		"1030.08.01"=>["夜日蝕九分半余"],
    "1031.01.01"=>["夜日蝕七分"],		"1031.07.01"=>["(日蝕三分)"],		"1031.12.01"=>["夜日蝕六分"],
    "1032.06.01"=>["(日蝕七分)"],		"1032.12.01"=>["(日蝕九分)"],		"1033.06.01"=>["夜日蝕八分"],
    "1033.12.01"=>["(夜日蝕四分)"],		"1035.04.01"=>["(日帯蝕九分)"],		"1035.10.01"=>["夜日蝕三分"],
    "1036.04.01"=>["日蝕一分"],			"1036.10.01"=>["(日蝕四分)"],		"1037.03.30"=>["日蝕一分にミたす"],
    "1039.02.01"=>["(夜日蝕四分)"],		"1039.08.01"=>["夜日蝕一分にミたす"],	"1040.01.01"=>["日蝕八分"],
    "1040.07.01"=>["(夜日蝕九分)"],		"1041.01.01"=>["夜日蝕九分"],		"1041.07.01"=>["(夜日蝕九分)"],
    "1042.06.01"=>["日帯蝕四分"],		"1042.11.01"=>["(日帯蝕九分)"],		"1043.05.01"=>["日蝕三分"],
    "1043.11.01"=>["夜日蝕一分"],		"1044.05.01"=>["(夜日蝕九分半余)"],	"1044.11.01"=>["夜日蝕七分"],
    "1045.09.01"=>["(日蝕一分)"],		"1046.03.01"=>["日蝕七分"],		"1047.03.01"=>["(日蝕一分)"],
    "1047.09.01"=>["日蝕一分"],			"1048.02.01"=>["(日帯蝕七分)"],		"1049.01.01"=>["日蝕ニ分"],
    "1050.01.01"=>["夜日蝕一分"],		"1050.07.01"=>["(夜日蝕五分)"],		"1050.12.02"=>["(夜日蝕一分にミたす)"],
    "1051.06.01"=>["夜日蝕七分"],		"1051.12.01"=>["(夜日蝕一分)"],		"1052.11.01"=>["日蝕ニ分"],
    "1053.05.01"=>["(夜日蝕ニ分)"],		"1053.10.01"=>["日蝕九分半余"],		"1054.04.01"=>["日蝕九分"],
    "1054.10.01"=>["(夜日蝕八分)"],		"1055.10.01"=>["(日帯蝕八分)"],		"1056.08.01"=>["日蝕ニ分"],
    "1057.02.01"=>["(日蝕九分)"],		"1057.08.01"=>["夜日蝕ニ分"],		"1058.02.01"=>["夜日蝕四分"],
    "1058.08.01"=>["(日蝕一分にミたす)"],	"1059.01.01"=>["日蝕四分"],		"1059.07.01"=>["(日蝕六分)"],
    "1059.12.01"=>["(夜日蝕三分)"],		"1060.06.01"=>["夜日蝕一分にミたす"],	"1060.12.01"=>["(夜日蝕七分)"],
    "1061.06.01"=>["日蝕九分"],			"1061.11.01"=>["夜日蝕六分"],		"1062.05.01"=>["(日蝕八分)"],
    "1062.11.01"=>["夜日蝕三分"],		"1064.04.01"=>["夜日蝕ニ分"],		"1064.09.01"=>["(日帯蝕八分)"],
    "1065.03.01"=>["(夜日蝕九分)"],		"1065.09.01"=>["夜日蝕七分"],		"1066.03.01"=>["(夜日蝕三分)"],
    "1066.09.01"=>["日蝕五分"],			"1067.07.01"=>["(夜日蝕六分)"],		"1068.01.01"=>["日蝕九分"],
    "1068.07.01"=>["(夜日蝕一分)"],		"1069.01.01"=>["(夜日蝕六分)"],		"1069.07.01"=>["日蝕八分"],
    "1069.12.01"=>["(日蝕ニ分)"],		"1070.06.01"=>["夜日蝕五分"],		"1070.11.01"=>["夜日蝕一分"],
    "1071.05.01"=>["(日蝕六分)"],		"1071.11.01"=>["夜日蝕七分"],		"1072.05.01"=>["夜日蝕九分半余"],
    "1072.10.01"=>["(夜日蝕八分)"],		"1073.04.01"=>["日帯蝕八分"],		"1073.10.01"=>["(日帯蝕ニ分)"],
    "1074.03.01"=>["(日蝕四分)"],		"1074.09.01"=>["夜日蝕八分"],		"1075.03.01"=>["(夜日蝕一分にミたす)"],
    "1075.08.01"=>["日蝕七分"],			"1076.02.01"=>["日帯蝕四分"],		"1076.08.01"=>["(日蝕九分半余)"],
    "1077.08.01"=>["(日蝕一分)"],		"1078.12.01"=>["(日蝕七分)"],		"1079.06.01"=>["夜日蝕三分"],
    "1079.12.01"=>["日蝕ニ分"],			"1080.06.01"=>["(日蝕九分)"],		"1080.11.01"=>["日蝕七分"],
    "1081.10.01"=>["(夜日蝕六分)"],		"1082.04.01"=>["夜日蝕ニ分"],		"1082.10.01"=>["(日蝕四分)"],
    "1083.04.01"=>["(日帯蝕八分)"],		"1083.09.01"=>["日帯蝕八分"],		"1084.03.01"=>["(日蝕ニ分)"],
    "1084.09.01"=>["夜日蝕八分"],		"1085.02.01"=>["夜日蝕五分"],		"1085.08.01"=>["(日蝕ニ分)"],
    "1086.02.01"=>["夜日蝕ニ分"],		"1086.07.01"=>["(日蝕九分)"],		"1087.01.01"=>["(日蝕九分半余)"],
    "1087.07.01"=>["日蝕一分"],			"1088.01.01"=>["(夜日蝕九分半余)"],	"1089.05.01"=>["(日蝕七分)"],
    "1089.11.01"=>["夜日蝕六分"],		"1090.05.01"=>["(日帯蝕五分)"],		"1090.11.01"=>["(日蝕八分)"],
    "1091.05.01"=>["日蝕四分"],			"1091.10.01"=>["(夜日蝕七分)"],		"1092.03.01"=>["(夜日蝕八分)"],
    "1093.03.01"=>["(夜日蝕ニ分)"],		"1093.09.01"=>["日帯蝕五分"],		"1094.03.01"=>["日蝕八分"],
    "1094.08.01"=>["(夜日蝕五分)"],		"1095.02.01"=>["日帯蝕八分"],		"1096.07.01"=>["日蝕一分"],
    "1097.01.01"=>["(夜日蝕五分)"],		"1097.06.01"=>["日帯蝕七分"],		"1097.12.01"=>["夜日蝕九分半余"],
    "1098.06.01"=>["(夜日蝕七分)"],		"1098.12.01"=>["夜日蝕三分"],		"1099.06.01"=>["(日蝕三分)"],
    "1100.04.01"=>["日蝕六分"],			"1100.10.01"=>["(夜日蝕一分)"],		"1101.04.01"=>["(日蝕一分)"],
    "1101.10.01"=>["日帯蝕六分"],		"1102.04.01"=>["(日蝕八分)"],		"1104.02.01"=>["夜日蝕三分"],
    "1104.08.01"=>["(夜日蝕三分)"],		"1105.02.01"=>["(夜日蝕八分)"],		"1105.07.01"=>["夜日蝕四分"],
    "1106.07.01"=>["日蝕ニ分"],			"1106.12.01"=>["日蝕ニ分"],		"1107.06.01"=>["(夜日蝕三分)"],
    "1107.11.01"=>["日帯蝕九分"],		"1108.05.01"=>["(日蝕四分)"],		"1108.11.01"=>["(夜日蝕六分)"],
    "1109.05.01"=>["夜日蝕五分"],		"1109.11.01"=>["(日蝕五分)"],		"1110.09.01"=>["日蝕ニ分"],
    "1111.03.01"=>["(日蝕八分)"],		"1111.09.01"=>["夜日蝕一分にミたす"],	"1112.03.01"=>["夜日蝕六分"],
    "1112.09.01"=>["(日帯蝕七分)"],		"1113.03.01"=>["日蝕三分"],		"1113.08.01"=>["(日帯蝕九分)"],
    "1114.01.01"=>["(夜日蝕六分)"],		"1114.07.01"=>["夜日蝕八分"],		"1115.01.01"=>["(夜日蝕ニ分)"],
    "1115.07.01"=>["日蝕七分"],			"1116.01.01"=>["夜日蝕五分"],		"1116.06.01"=>["(日帯蝕三分)"],
    "1116.12.01"=>["夜日蝕七分"],		"1118.05.01"=>["日蝕三分"],		"1118.10.01"=>["(日蝕七分)"],
    "1119.04.01"=>["日帯蝕九分"],		"1119.10.01"=>["夜日蝕六分"],		"1120.04.01"=>["(夜日蝕三分)"],
    "1120.10.01"=>["日蝕七分"],			"1121.08.01"=>["(夜日蝕七分)"],		"1122.02.01"=>["日蝕七分"],
    "1122.08.01"=>["(夜日蝕一分にミたす)"],	"1123.02.01"=>["(夜日蝕七分)"],		"1123.08.01"=>["日帯蝕九分半余"],
    "1124.02.01"=>["(日蝕四分)"],		"1124.07.01"=>["日帯蝕九分半余"],	"1125.06.01"=>["(日帯蝕四分)"],
    "1125.12.01"=>["夜日蝕五分"],		"1126.06.01"=>["(日帯蝕七分)"],		"1126.11.01"=>["(夜日蝕ニ分)"],
    "1127.05.01"=>["夜日蝕九分"],		"1127.11.01"=>["(夜日蝕四分)"],		"1128.05.01"=>["夜日蝕一分"],
    "1128.10.01"=>["夜日蝕八分"],		"1129.04.01"=>["(日帯蝕七分)"],		"1129.09.01"=>["日蝕六分"],
    "1130.03.01"=>["日帯蝕八分"],		"1130.09.01"=>["(日蝕五分)"],		"1131.09.01"=>["(日蝕四分)"],
    "1133.01.01"=>["(日蝕六分)"],		"1133.07.01"=>["夜日蝕ニ分"],		"1134.01.01"=>["日蝕一分にミたす"],
    "1134.07.01"=>["(日蝕三分)"],		"1134.12=01"=>["日蝕七分"],		"1135.06.01"=>["(夜日蝕五分)"],
    "1136.05.01"=>["夜日蝕ニ分"],		"1136.11.01"=>["(夜日蝕五分)"],		"1137.05.01"=>["夜日蝕六分"],
    "1137.10.01"=>["日蝕一分"],			"1138.04.01"=>["(夜日蝕三分)"],		"1138.10.01"=>["夜日蝕六分"],
    "1139.04.01"=>["(日蝕一分)"],		"1139.09.01"=>["(日蝕一分)"],		"1140.03.01"=>["夜日蝕一分にミたす"],
    "1140.08.01"=>["(日蝕九分)"],		"1141.02.01"=>["(日蝕四分)"],		"1141.08.01"=>["日蝕三分"],
    "1142.02.01"=>["(夜日蝕七分)"],		"1143.01.01"=>["夜日蝕三分"],		"1143.06.01"=>["(日蝕三分)"],
    "1143.12.01"=>["日帯蝕九分"],		"1144.06.01"=>["(夜日蝕三分)"],		"1144.12.01"=>["(日蝕三分)"],
    "1145.06.01"=>["日蝕七分"],			"1145.11.01"=>["(夜日蝕五分)"],		"1147.10.01"=>["日帯蝕八分"],
    "1148.04.01"=>["(日蝕五分)"],		"1148.09.01"=>["(夜日蝕七分)"],		"1149.03.01"=>["日帯蝕九分半余"],
    "1151.02.01"=>["(夜日蝕三分)"],		"1151.07.01"=>["夜日蝕一分"],		"1152.01.01"=>["夜日蝕七分"],
    "1152.07.01"=>["(夜日蝕六分)"],		"1153.07.01"=>["(日蝕六分)"],		"1154.05.01"=>["日帯蝕四分"],
    "1154.11.01"=>["(夜日蝕八分)"],		"1155.05.01"=>["日蝕五分"],		"1155.11.01"=>["夜日蝕四分"],
    "1156.05.01"=>["(日蝕九分)"],		"1156.10.01"=>["夜日蝕ニ分"],		"1157.04.01"=>["(夜日蝕四分)"],
    "1158.03.01"=>["日帯蝕七分"],		"1158.09.01"=>["(日帯蝕三分)"],		"1159.03.01"=>["(夜日蝕九分半余)"],
    "1159.08.01"=>["夜日蝕六分"],		"1160.08.01"=>["日蝕四分"],		"1162.01.01"=>["日帯蝕五分"],
    "1162.06.01"=>["(日蝕四分)"],		"1162.12.01"=>["(夜日蝕七分)"],		"1163.06.01"=>["日蝕九分半余"],
    "1163.12.01"=>["(日蝕三分)"],		"1164.06.01"=>["日蝕一分"],		"1164.10=01"=>["夜日蝕一分"],
    "1165.04.01"=>["(日蝕五分)"],		"1166.04.01"=>["(夜日蝕九分)"],		"1166.10.01"=>["(夜日蝕九分)"],
    "1167.04.01"=>["日蝕四分"],			"1167.09.01"=>["(夜日蝕六分)"],		"1168.02.01"=>["(夜日蝕五分)"],
    "1169.02.01"=>["(夜日蝕一分)"],		"1169.08.01"=>["日蝕五分"],		"1170.02.01"=>["夜日蝕九分"],
    "1170.07.01"=>["(日帯蝕九分)"],		"1171.01.01"=>["夜日蝕九分"],		"1171.12.01"=>["(夜日蝕一分)"],
    "1172.12.01"=>["(日蝕三分)"],		"1173.05.01"=>["日蝕八分"],		"1173.11.01"=>["夜日蝕四分"],
    "1174.05.01"=>["(日帯蝕九分)"],		"1174.11.01"=>["日蝕八分"],		"1176.03.01"=>["日蝕四分"],
    "1176.09.01"=>["(夜日蝕一分)"],		"1177.03.01"=>["(夜日蝕八分)"],		"1177.09.01"=>["(日帯蝕七分)"],
    "1178.03.01"=>["(日蝕六分)"],		"1178.08.01"=>["夜日蝕七分"],		"1179.07.01"=>["(夜日蝕六分)"],
    "1180.01.01"=>["夜日蝕三分"],		"1180.07.01"=>["(日蝕七分)"],		"1181.01.01"=>["(日帯蝕八分)"],
    "1181.06.01"=>["夜日蝕三分"],		"1181.12.01"=>["(夜日蝕九分)"],		"1182.06.01"=>["夜日蝕六分"],
    "1183.05.01"=>["(日蝕六分)"],		"1183.10=01"=>["日蝕七分"],		"1184.04.01"=>["(日帯蝕九分)"],
    "1184.10.01"=>["日蝕五分"],			"1185.04.01"=>["夜日蝕一分にミたす"],	"1185.10.01"=>["(日蝕五分)"],
    "1187.02.01"=>["(日蝕六分)"],		"1187.08.02"=>["日帯蝕七分"],		"1188.02.01"=>["日蝕一分"],
    "1188.08.01"=>["(日蝕一分にミたす)"],	"1189.02.01"=>["日蝕六分"],		"1189.07.01"=>["(夜日蝕五分)"],
    "1190.06.01"=>["日帯蝕三分"],		"1190.12.01"=>["(夜日蝕九分)"],		"1191.06.01"=>["日帯蝕八分"],
    "1191.12.01"=>["日蝕五分"],			"1192.05.01"=>["(夜日蝕九分)"],		"1192.11.01"=>["夜日蝕七分"],
    "1193.05.01"=>["(日蝕四分)"],		"1194.09.01"=>["(日帯蝕八分)"],		"1195.03.01"=>["(日蝕一分)"],
    "1195.09.01"=>["(日蝕七分)"],		"1196.03.01"=>["(夜日蝕七分)"],		"1196.09.01"=>["夜日蝕一分にミたす"],
    "1197.07.01"=>["(日蝕一分)"],		"1198.01.01"=>["日帯蝕九分半余"],	"1199.01.01"=>["(日帯蝕六分)"],
    "1199.07.01"=>["日帯蝕八分"],		"1200.01.01"=>["(日帯蝕三分)"],		"1200.06.01"=>["日蝕一分"],
    "1200.11.01"=>["日蝕一分"],			"1201.05.01"=>["(夜日蝕五分)"],		"1201.11.01"=>["夜日蝕九分"],
    "1202.05.01"=>["(日蝕一分にミたす)"],	"1202.10=01"=>["夜日蝕九分"],		"1203.04.01"=>["日帯蝕七分"],
    "1203.10.01"=>["(夜日蝕一分)"],		"1204.10.01"=>["(夜日蝕六分)"],		"1205.03.01"=>["(日帯蝕九分)"],
    "1206.02.01"=>["日帯蝕九分"],		"1206.08.01"=>["(夜日蝕八分)"],		"1207.08.01"=>["(日蝕八分)"],
    "1208.12.01"=>["(日帯蝕五分)"],		"1209.06.01"=>["夜日蝕七分"],		"1209.12.01"=>["夜日蝕六分"],
    "1210.06.01"=>["(日蝕四分)"],		"1210.12.01"=>["日帯蝕六分"],		"1211.05.01"=>["(夜日蝕ニ分)"],
    "1212.04.01"=>["日帯蝕九分"],		"1212.10.01"=>["(日蝕ニ分)"],		"1213.04.01"=>["日帯蝕九分"],
    "1213.09=01"=>["(夜日蝕五分)"],		"1214.03.01"=>["(夜日蝕一分にミたす)"],	"1214.09.01"=>["日蝕七分"],
    "1216.02.01"=>["日帯蝕四分"],		"1216.07.01"=>["(日帯蝕七分)"],		"1217.01.01"=>["(夜日蝕六分)"],
    "1217.07.01"=>["日蝕八分"],			"1218.01.01"=>["(日帯蝕八分)"],		"1218.07.01"=>["日蝕三分"],
    "1219.05.01"=>["(日蝕ニ分)"],		"1219.11.01"=>["夜日蝕五分"],		"1220.05.01"=>["(日帯蝕四分)"],
    "1220.11.01"=>["夜日蝕六分"],		"1221.05.01"=>["日蝕六分"],		"1221.10=01"=>["(夜日蝕九分)"],
    "1222.04.01"=>["夜日蝕三分"],		"1223.03.01"=>["(夜日蝕一分)"],		"1223.09.01"=>["日蝕五分"],
    "1224.03.01"=>["夜日蝕七分"],		"1224.08.01"=>["夜日蝕九分"],		"1225.02.01"=>["夜日蝕五分"],
    "1225.08.01"=>["(日蝕三分)"],		"1226.01.01"=>["(夜日蝕七分)"],		"1227.01.01"=>["(日帯蝕九分)"],
    "1227.06.01"=>["日蝕五分"],			"1227.12.01"=>["夜日蝕ニ分"],		"1228.06.01"=>["(日蝕三分)"],
    "1228.12.01"=>["日帯蝕五分"],		"1229.06.01"=>["(夜日蝕四分)"],		"1230.04.01"=>["日蝕ニ分"],
    "1231.04.01"=>["夜日蝕九分"],		"1231.10.01"=>["(日帯蝕五分)"],		"1232.04.01"=>["(日蝕七分)"],
    "1232.09=01"=>["夜日蝕三分"],		"1234.08.01"=>["(日蝕六分)"],		"1235.02.01"=>["(日蝕六分)"],
    "1235.07.01"=>["日帯蝕九分半余"],		"1236.07.01"=>["夜日蝕八分"],		"1237.06.01"=>["(日蝕四分)"],
    "1237.12.01"=>["日蝕九分半余"],		"1238.05.01"=>["(夜日蝕九分半余)"],	"1238.11.01"=>["日蝕六分"],
    "1239.05.01"=>["夜日蝕八分"],		"1239.11.01"=>["(日蝕四分)"],		"1241.03.01"=>["(日蝕三分)"],
    "1241.09.01"=>["夜日蝕九分"],		"1242.03.01"=>["(日蝕七分)"],		"1242.09.01"=>["日蝕一分にミたす"],
    "1243.03.01"=>["日蝕七分"],			"1245.07.01"=>["日蝕九分"],		"1246.01.01"=>["日蝕五分"],
    "1246.06.01"=>["(夜日蝕八分)"],		"1246.12.01"=>["夜日蝕一分"],		"1247.06.01"=>["(日蝕六分)"],
    "1248.05.01"=>["夜日蝕一分"],		"1248.11.01"=>["(日帯蝕六分)"],		"1249.04.01"=>["日蝕六分"],
    "1249.10.01"=>["(日帯蝕九分半余)"],		"1250.04.01"=>["(日帯蝕六分)"],		"1250.10.01"=>["夜日蝕ニ分"],
    "1251.04.01"=>["(夜日蝕五分)"],		"1251.09=01"=>["夜日蝕六分"],		"1252.02.01"=>["日帯蝕九分"],
    "1253.02.01"=>["(日帯蝕九分半余)"],		"1253.08.01"=>["夜日蝕七分"],		"1254.02.01"=>["(日蝕三分)"],
    "1255.12.01"=>["夜日蝕五分"],		"1256.06.01"=>["(日蝕六分)"],		"1256.12.01"=>["夜日蝕一分"],
    "1257.05.01"=>["夜日蝕九分"],		"1257.11.01"=>["(夜日蝕六分)"],		"1258.11.01"=>["(夜日蝕ニ分)"],
    "1259.04.01"=>["(日蝕八分)"],		"1260.03.01"=>["(日蝕四分)"],		"1260.09.01"=>["夜日蝕九分"],
    "1261.03.01"=>["日帯蝕ニ分"],		"1261.09.01"=>["(日帯蝕九分半余)"],	"1262.03.01"=>["夜日蝕五分"],
    "1263.01.01"=>["(日蝕四分)"],		"1263.07.01"=>["夜日蝕一分にミたす"],	"1264.01.01"=>["夜日蝕五分"],
    "1264.07.01"=>["(日帯蝕八分)"],		"1265.01.01"=>["日帯蝕八分"],		"1265.06.01"=>["(日蝕三分)"],
    "1266.05.01"=>["夜日蝕四分"],		"1266.11.01"=>["(日帯蝕三分)"],		"1267.05.01"=>["日帯蝕九分"],
    "1267.11.01"=>["(夜日蝕七分)"],		"1268.04.01"=>["(夜日蝕三分)"],		"1268.10.01"=>["日蝕九分半余"],
    "1270.03.01"=>["日蝕三分"],			"1270.09.01"=>["(日帯蝕九分)"],		"1271.02.01"=>["夜日蝕八分"],
    "1271.08.01"=>["(日蝕四分)"],		"1272.02.01"=>["(日帯蝕九分)"],		"1272.08.01"=>["日蝕五分"],
    "1273.06.01"=>["(夜日蝕三分)"],		"1273.12.01"=>["夜日蝕九分半余"],	"1274.06.01"=>["(日蝕八分)"],
    "1274.12.01"=>["夜日蝕三分"],		"1275.06.01"=>["日蝕八分"],		"1275.12.01"=>["(夜日蝕七分)"],
    "1276.05.01"=>["夜日蝕ニ分"],		"1277.04.01"=>["(夜日蝕九分半余)"],	"1277.10.01"=>["日蝕六分"],
    "1278.04.01"=>["(夜日蝕九分)"],		"1278.10.01"=>["夜日蝕八分"],		"1279.03.01"=>["夜日蝕一分"],
    "1279.09.01"=>["(日蝕四分)"],		"1281.02.01"=>["(日帯蝕八分)"],		"1281.07=01"=>["日帯蝕一分"],
    "1282.01.01"=>["日蝕五分"],			"1282.07.01"=>["日蝕四分"],		"1283.01.01"=>["日帯蝕ニ分"],
    "1283.07.01"=>["(夜日蝕四分)"],		"1284.11.01"=>["(夜日蝕六分)"],		"1285.05.01"=>["夜日蝕三分"],
    "1285.11.01"=>["(日蝕三分)"],		"1286.05.01"=>["(日蝕九分)"],		"1286.11.01"=>["夜日蝕四分"],
    "1287.04.01"=>["(日蝕一分)"],		"1287.10.01"=>["日蝕ニ分"],		"1288.09.01"=>["(日蝕四分)"],
    "1289.03.01"=>["日蝕一分"],			"1289.09.01"=>["(日帯蝕九分半余)"],	"1290.08.01"=>["日帯蝕八分"],
    "1291.08.01"=>["夜日蝕六分"],		"1292.01.01"=>["日蝕八分"],		"1292.06=01"=>["(夜日蝕四分)"],
    "1292.12.01"=>["日蝕三分"],			"1293.06.01"=>["日蝕一分"],		"1293.12.01"=>["(日蝕六分)"],
    "1294.06.01"=>["日蝕一分"],			"1295.10.01"=>["夜日蝕九分"],		"1296.04.01"=>["(日帯蝕七分)"],
    "1296.10.01"=>["日蝕一分"],			"1297.04.01"=>["日蝕九分半余"],		"1298.09.01"=>["(夜日蝕五分)"],
    "1299.08.01"=>["日蝕五分"],			"1300.02.01"=>["日帯蝕八分"],		"1300.07=01"=>["日帯蝕九分半余"],
    "1301.01.01"=>["夜日蝕一分"],		"1301.07.01"=>["(日蝕九分半余)"],	"1302.06.01"=>["日帯蝕ニ分"],
    "1302.12.01"=>["(日蝕三分)"],		"1303.05.01"=>["日蝕九分"],		"1303.11.01"=>["(夜日蝕五分)"],
    "1304.05.01"=>["(日蝕三分)"],		"1304.11.01"=>["日帯蝕六分"],		"1305.05.01"=>["(日蝕四分)"],
    "1305.11.01"=>["夜日蝕五分"],		"1306.03.01"=>["日帯蝕八分"],		"1307.03.01"=>["日帯蝕九分"],
    "1307.09.01"=>["(夜日蝕四分)"],		"1308.03.01"=>["(日蝕六分)"],		"1310.07.01"=>["(日蝕九分半余)"],
    "1311.01.01"=>["夜日蝕八分"],		"1311.06=01"=>["夜日蝕八分"],		"1311.12.01"=>["(夜日蝕七分)"],
    "1312.06.01"=>["日蝕四分"],			"1313.05.01"=>["(日蝕四分)"],		"1313.11.01"=>["夜日蝕ニ分"],
    "1314.04.01"=>["(日蝕三分)"],		"1314.10.01"=>["夜日蝕九分半余"],	"1315.04.01"=>["日蝕六分"],
    "1315.10.01"=>["(日帯蝕九分)"],		"1316.04.01"=>["夜日蝕七分"],		"1317.02.01"=>["(日蝕五分)"],
    "1317.08.01"=>["夜日蝕九分"],		"1318.02.01"=>["(夜日蝕六分)"],		"1318.08.01"=>["夜日蝕六分"],
    "1319.02.01"=>["日帯蝕九分"],		"1319.07=01"=>["(日蝕五分)"],		"1320.12.01"=>["(夜日蝕七分)"],
    "1321.06.01"=>["日蝕七分"],			"1321.12.01"=>["(夜日蝕五分)"],		"1322.05=01"=>["(夜日蝕七分)"],
    "1322.11.01"=>["日帯蝕八分"],		"1323.05.01"=>["(夜日蝕四分)"],		"1324.04.01"=>["日蝕一分"],
    "1324.10.01"=>["(夜日蝕九分半余)"],		"1325.03.01"=>["夜日蝕九分"],		"1325.09.01"=>["(日帯蝕三分)"],
    "1326.03.01"=>["(夜日蝕五分)"],		"1326.09.01"=>["日帯蝕五分"],		"1327.09.01"=>["日蝕一分"],
    "1328.01.01"=>["夜日蝕六分"],		"1328.07.01"=>["(日蝕四分)"],		"1329.01.01"=>["夜日蝕九分半余"],
    "1329.07.01"=>["日蝕八分"],			"1330.01.01"=>["(夜日蝕四分)"],		"1330.06=01"=>["夜日蝕九分半余"],
    "1331.05.01"=>["(日蝕三分)"],		"1331.11.01"=>["日帯蝕六分"],		"1332.05.01"=>["(日帯蝕五分)"],
    "1332.11.01"=>["日帯蝕九分"],		"1333.04.01"=>["夜日蝕五分"],		"1333.10.01"=>["(日帯蝕五分)"],
    "1334.04.01"=>["日蝕一分"],			"1335.03.01"=>["(日帯蝕九分半余)"],	"1336.02.01"=>["(日蝕七分)"],
    "1336.08.01"=>["日蝕八分"],			"1337.02.01"=>["日蝕四分"],		"1338.12.01"=>["(夜日蝕一分)"],
    "1339.06.01"=>["夜日蝕八分"],		"1339.12.01"=>["(日蝕九分)"],		"1340.06.01"=>["(夜日蝕ニ分)"],
    "1340.12.01"=>["夜日蝕八分"],		"1341.05.01"=>["(夜日蝕四分)"],		"1342.10.01"=>["(日蝕ニ分)"],
    "1343.04.01"=>["日蝕四分"],			"1343.10.01"=>["(日帯蝕九分)"],		"1344.03.01"=>["(夜日蝕一分)"],
    "1344.09.01"=>["日蝕七分"],			"1345.03.01"=>["(日帯蝕三分)"],		"1345.09.01"=>["夜日蝕七分"],
    "1346.02.01"=>["日蝕七分"],			"1347.01.01"=>["日蝕一分にミたす"],	"1347.07.01"=>["(日蝕八分)"],
    "1348.01.01"=>["(日蝕八分)"],		"1348.07.01"=>["日帯蝕ニ分"],		"1349.01.01"=>["(夜日蝕五分)"],
    "1349.06.01"=>["(夜日蝕一分にミたす)"],	"1349.11.01"=>["夜日蝕四分"],		"1350.05.01"=>["(夜日蝕八分)"],
    "1350.11.01"=>["日蝕一分にミたす"],		"1351.05.01"=>["日帯蝕四分"],		"1351.11.01"=>["(夜日蝕三分)"],
    "1352.10.01"=>["(夜日蝕五分)"],		"1353.09.01"=>["日蝕三分"],		"1354.03.01"=>["(日蝕五分)"],
    "1354.09.01"=>["日帯蝕四分"],		"1355.08.01"=>["(日帯蝕六分)"],		"1356.02.01"=>["夜日蝕六分"],
    "1356.08.01"=>["(夜日蝕七分)"],		"1357.01.01"=>["(日蝕ニ分)"],		"1357.07.01"=>["夜日蝕三分"],
    "1357.12.01"=>["(夜日蝕五分)"],		"1358.06.01"=>["(日蝕三分)"],		"1358.12.01"=>["日蝕九分半余"],
    "1359.06.01"=>["(日蝕八分)"],		"1360.04=01"=>["夜日蝕五分"],		"1361.04.01"=>["日蝕七分"],
    "1361.10.01"=>["(夜日蝕四分)"],		"1362.04.01"=>["(日蝕九分)"],		"1362.10.01"=>["夜日蝕七分"],
    "1364.08.01"=>["(日帯蝕八分)"],		"1365.02.01"=>["夜日蝕七分"],		"1365.08.01"=>["(夜日蝕七分)"],
    "1366.01.01"=>["(夜日蝕ニ分)"],		"1366.07.01"=>["日蝕六分"],		"1367.06.01"=>["(日蝕一分)"],
    "1367.12.01"=>["日蝕七分"],			"1368.06.01"=>["(日帯蝕九分)"],		"1368.11.01"=>["夜日蝕六分"],
    "1369.05.01"=>["日蝕九分"],			"1369.11.01"=>["(日帯蝕八分)"],		"1370.05.01"=>["夜日蝕六分"],
    "1371.03=01"=>["(日蝕四分)"],		"1371.09.01"=>["夜日蝕八分"],		"1372.03.01"=>["(夜日蝕七分)"],
    "1372.09.01"=>["夜日蝕四分"],		"1373.03.01"=>["日帯蝕八分"],		"1373.09.01"=>["(日蝕六分)"],
    "1374.02.01"=>["日蝕ニ分"],			"1374.08.01"=>["(日蝕三分)"],		"1375.01.01"=>["(夜日蝕九分半余)"],
    "1375.07.01"=>["日蝕四分"],			"1376.01.01"=>["(夜日蝕八分)"],		"1376.07.01"=>["(日蝕三分)"],
    "1376.12.01"=>["夜日蝕四分"],		"1377.06.01"=>["(日蝕五分)"],		"1377.12.01"=>["日蝕ニ分"],
    "1378.11.01"=>["(夜日蝕七分)"],		"1379.04=01"=>["夜日蝕一分"],		"1379.10.01"=>["(夜日蝕一分)"],
    "1380.04.01"=>["(日帯蝕ニ分)"],		"1380.10.01"=>["日帯蝕六分"],		"1381.10.01"=>["日蝕三分"],
    "1382.02.01"=>["夜日蝕七分"],		"1382.08.01"=>["(日蝕ニ分)"],		"1383.02.01"=>["夜日蝕九分半余"],
    "1383.08.01"=>["(日蝕三分)"],		"1384.02.01"=>["(日帯蝕四分)"],		"1384.08.01"=>["夜日蝕ニ分"],
    "1385.01.01"=>["(日蝕一分)"],		"1385.06.01"=>["(日蝕ニ分)"],		"1385.12.01"=>["夜日蝕三分"],
    "1386.06.01"=>["(日蝕九分半余)"],		"1386.12.01"=>["日蝕一分にミたす"],	"1387.05=01"=>["夜日蝕三分"],
    "1387.11.01"=>["(日帯蝕八分)"],		"1388.05.01"=>["日蝕三分"],		"1389.04.01"=>["(日帯蝕八分)"],
    "1390.03=01"=>["(日帯蝕九分)"],		"1390.09.01"=>["日蝕九分"],		"1391.03.01"=>["日蝕六分"],
    "1391.09.01"=>["(夜日蝕一分)"],		"1393.07.01"=>["日帯蝕九分半余"],	"1394.01.01"=>["(日蝕五分)"],
    "1394.07.01"=>["夜日蝕八分"],		"1395.01.01"=>["夜日蝕五分"],		"1395.07.01"=>["(夜日蝕一分)"],
    "1396.11.01"=>["(日蝕一分)"],		"1397.05.01"=>["日蝕九分"],		"1397.11.01"=>["(日帯蝕九分)"],
    "1398.04=01"=>["(夜日蝕六分)"],		"1398.10.01"=>["日蝕五分"],		"1399.04.01"=>["(日蝕五分)"],
    "1399.10.01"=>["夜日蝕八分"],		"1400.03.01"=>["日蝕五分"],		"1401.02.01"=>["日蝕三分"],
    "1401.08.01"=>["(日帯蝕九分)"],		"1402.02.01"=>["(日蝕九分半余)"],	"1402.08.01"=>["夜日蝕ニ分"],
    "1403.02.01"=>["(夜日蝕八分)"],		"1403.12.01"=>["夜日蝕八分"],		"1404.06.01"=>["(夜日蝕三分)"],
    "1404.12.01"=>["夜日蝕七分"],		"1405.06.01"=>["夜日蝕五分"],		"1405.12.01"=>["(夜日蝕七分)"],
    "1406.06.01"=>["日蝕三分"],			"1406.11.01"=>["(夜日蝕一分)"],		"1407.10.01"=>["日蝕三分"],
    "1408.04.01"=>["(日蝕一分にミたす)"],	"1408.10.01"=>["夜日蝕ニ分"],		"1409.03=01"=>["夜日蝕ニ分"],
    "1409.09.01"=>["(日蝕四分)"],		"1410.09.01"=>["(夜日蝕九分)"],		"1411.02.01"=>["(日蝕三分)"],
    "1412.01.01"=>["(夜日蝕九分)"],		"1412.07.01"=>["日帯蝕九分"],		"1413.01.01"=>["日蝕八分"],
    "1413.07.01"=>["(日帯蝕七分)"],		"1414.01.01"=>["夜日蝕四分"],		"1414.07.01"=>["(日蝕一分)"],
    "1414.11.01"=>["(夜日蝕四分)"],		"1415.05.01"=>["日蝕六分"],		"1415.11.01"=>["(夜日蝕九分)"],
    "1416.05.01"=>["(日蝕五分)"],		"1416.11.01"=>["夜日蝕一分にミたす"],	"1417.05.01"=>["(夜日蝕七分)"],
    "1418.09.01"=>["(日帯蝕六分)"],		"1419.03.01"=>["日帯蝕七分"],		"1419.09.01"=>["(夜日蝕九分)"],
    "1420.02.01"=>["(夜日蝕三分)"],		"1420.08.01"=>["日蝕八分"],		"1421.08.01"=>["日蝕三分"],
    "1422.01.01"=>["日蝕六分"],			"1422.07.01"=>["(夜日蝕九分半余)"],	"1422.12.01"=>["夜日蝕六分"],
    "1423.06.01"=>["(日蝕五分)"],		"1423.12.01"=>["(日蝕五分)"],		"1424.06.01"=>["夜日蝕八分"],
    "1425.05.01"=>["(日蝕ニ分)"],		"1425.10.01"=>["夜日蝕六分"],		"1426.04.01"=>["(夜日蝕五分)"],
    "1426.10.01"=>["夜日蝕四分"],		"1427.04.01"=>["夜日蝕一分"],		"1427.10.01"=>["(日帯蝕六分)"],
    "1428.03=01"=>["日帯蝕五分"],		"1428.09.01"=>["(日蝕三分)"],		"1429.02.01"=>["(夜日蝕九分)"],
    "1429.08.01"=>["日蝕一分"],			"1430.02.01"=>["(日帯蝕八分)"],		"1430.08.01"=>["日蝕六分"],
    "1431.01.01"=>["夜日蝕一分にミたす"],	"1431.07.01"=>["(日蝕九分)"],		"1432.12.01"=>["(夜日蝕三分)"],
    "1433.06.01"=>["夜日蝕八分"],		"1433.11.01"=>["(日帯蝕六分)"],		"1434.05.01"=>["(日蝕一分)"],
    "1434.11.01"=>["日帯蝕九分"],		"1435.05.01"=>["(日蝕ニ分)"],		"1435.11.01"=>["日蝕四分"],
    "1436.04.01"=>["夜日蝕八分"],		"1437.03.01"=>["夜日蝕九分半余"],	"1437.09.01"=>["(日蝕一分)"],
    "1438.03.01"=>["(日帯蝕四分)"],		"1438.09.01"=>["夜日蝕一分にミたす"],	"1439.02.01"=>["(日蝕三分)"],
    "1440.07.01"=>["(日蝕七分)"],		"1441.01.01"=>["日蝕一分"],		"1441.07.01"=>["(日蝕六分)"],
    "1441.12.02"=>["(夜日蝕七分)"],		"1442.06.01"=>["日帯蝕四分"],		"1442.12.01"=>["(夜日蝕三分)"],
    "1443.05.01"=>["(日蝕三分)"],		"1444.05.01"=>["(日帯蝕七分)"],		"1444.10.01"=>["日蝕六分"],
    "1445.04.01"=>["日蝕九分"],			"1445.10.01"=>["(夜日蝕八分)"],		"1447.08.01"=>["日蝕六分"],
    "1448.02.01"=>["(日蝕三分)"],		"1448.08.01"=>["夜日蝕六分"],		"1449.02.01"=>["日帯蝕五分"],
    "1449.08.01"=>["(夜日蝕三分)"],		"1450.07.01"=>["(夜日蝕八分)"],		"1450.12.01"=>["(日蝕四分)"],
    "1451.06.01"=>["日帯蝕五分"],		"1451.12.01"=>["(日帯蝕八分)"],		"1452.06.01"=>["(夜日蝕四分)"],
    "1452.11.01"=>["日帯蝕九分半余"],		"1453.05.01"=>["(日蝕七分)"],		"1453.11.01"=>["夜日蝕四分"],
    "1454.04.01"=>["日蝕五分"],			"1454.10.01"=>["(夜日蝕九分)"],		"1455.04.01"=>["日蝕四分"],
    "1455.09.01"=>["(日帯蝕七分)"],		"1456.03.01"=>["(日蝕七分)"],		"1456.09.01"=>["夜日蝕三分"],
    "1457.03.01"=>["(夜日蝕九分)"],		"1458.01=01"=>["日帯蝕八分"],		"1458.07.01"=>["(日蝕一分)"],
    "1459.01.01"=>["夜日蝕七分"],		"1459.07.01"=>["(夜日蝕四分)"],		"1460.01.01"=>["(夜日蝕ニ分)"],
    "1460.07.01"=>["日蝕六分"],			"1461.05.01"=>["(夜日蝕一分)"],		"1461.11.01"=>["日蝕五分"],
    "1462.05.01"=>["(日蝕五分)"],		"1462.11.01"=>["夜日蝕ニ分"],		"1463.05.01"=>["日帯蝕八分"],
    "1463.10.01"=>["(日蝕九分半余)"],		"1464.10.01"=>["(夜日蝕八分)"],		"1465.03.01"=>["(日蝕ニ分)"],
    "1465.09.01"=>["夜日蝕七分"],		"1466.02=01"=>["(夜日蝕九分半余)"],	"1466.08.01"=>["夜日蝕四分"],
    "1467.02.01"=>["日蝕七分"],			"1467.08.01"=>["(夜日蝕ニ分)"],		"1468.02.01"=>["夜日蝕四分"],
    "1468.08.01"=>["(日蝕四分)"],		"1468.12.02"=>["(夜日蝕九分半余)"],	"1469.06.01"=>["日蝕三分"],
    "1469.12.01"=>["(夜日蝕ニ分)"],		"1470.06.01"=>["(日蝕四分)"],		"1470.12.01"=>["夜日蝕八分"],
    "1471.06.01"=>["(夜日蝕五分)"],		"1471.11.01"=>["夜日蝕一分"],		"1472.10.01"=>["(日蝕三分)"],
    "1473.04.01"=>["日蝕七分"],			"1473.10.01"=>["(夜日蝕九分)"],		"1474.04.01"=>["(夜日蝕四分)"],
    "1474.09.01"=>["日蝕九分半余"],		"1475.09.01"=>["日蝕四分"],		"1476.02.01"=>["日蝕ニ分"],
    "1477.01=01"=>["夜日蝕八分"],		"1477.07.01"=>["(日蝕ニ分)"],		"1478.01.01"=>["(日蝕六分)"],
    "1478.07.01"=>["夜日蝕一分"],		"1479.11.02"=>["夜日蝕三分"],		"1480.05.01"=>["(日帯蝕九分)"],
    "1480.11.01"=>["夜日蝕九分"],		"1481.05.01"=>["夜日蝕一分"],		"1481.11.01"=>["(夜日蝕七分)"],
    "1482.05.01"=>["夜日蝕八分"],		"1482.10.01"=>["(日蝕一分)"],		"1483.03.01"=>["(日帯蝕六分)"],
    "1484.03.01"=>["(日帯蝕七分)"],		"1484.09.01"=>["日蝕九分"],		"1485.03.01"=>["夜日蝕一分"],
    "1485.08.01"=>["(日蝕七分)"],		"1487.01.01"=>["(夜日蝕一分)"],		"1487.07.01"=>["夜日蝕九分"],
    "1487.12.01"=>["(日帯蝕九分)"],		"1488.06.01"=>["日蝕四分"],		"1488.12.01"=>["日帯蝕七分"],
    "1489.06.01"=>["(夜日蝕八分)"],		"1489.12.01"=>["日帯蝕一分"],		"1491.04.01"=>["夜日蝕三分"],
    "1491.10.01"=>["(日蝕七分)"],		"1492.04.01"=>["(夜日蝕五分)"],		"1492.10.01"=>["夜日蝕ニ分"],
    "1493.04.01"=>["(日蝕五分)"],		"1493.09.01"=>["夜日蝕九分半余"],	"1494.08.01"=>["(日帯蝕六分)"],
    "1495.02.01"=>["日蝕三分"],			"1495.08.01"=>["(日蝕九分半余)"],	"1496.02.01"=>["(夜日蝕一分)"],
    "1496.07.01"=>["夜日蝕三分"],		"1497.01.01"=>["(夜日蝕七分)"],		"1497.07.01"=>["夜日蝕七分"],
    "1497.12.01"=>["夜日蝕四分"],		"1498.06.01"=>["(夜日蝕七分)"],		"1498.11.01"=>["日蝕三分"],
    "1499.05.01"=>["日蝕四分"],			"1499.11.01"=>["(夜日蝕六分)"],		"1500.05.01"=>["日蝕四分"],
    "1501.09.01"=>["日蝕五分"],			"1502.03.01"=>["(日蝕五分)"],		"1502.09.01"=>["夜日蝕一分"],
    "1503.03.01"=>["日蝕三分"],			"1503.09.01"=>["(日帯蝕六分)"],		"1505.01.01"=>["(日蝕五分)"],
    "1506.01.01"=>["(日蝕三分)"],		"1506.07.01"=>["夜日蝕八分"],		"1506.12.01"=>["日帯蝕七分"],
    "1507.06.01"=>["(日蝕九分半余)"],		"1507.12.01"=>["夜日蝕五分"],		"1508.05.01"=>["夜日蝕ニ分"],
    "1508.11.01"=>["(夜日蝕五分)"],		"1509.05.01"=>["日帯蝕八分"],		"1509.10.01"=>["(夜日蝕七分)"],
    "1510.04.01"=>["(日蝕三分)"],		"1510.10.01"=>["夜日蝕四分"],		"1511.04.01"=>["(夜日蝕九分)"],
    "1512.03.01"=>["日帯蝕七分"],		"1512.08.01"=>["(日蝕一分にミたす)"],	"1513.02.01"=>["夜日蝕六分"],
    "1513.08.01"=>["(夜日蝕三分)"],		"1514.02.01"=>["(夜日蝕ニ分)"],		"1514.08.01"=>["日蝕八分"],
    "1515.12.01"=>["日蝕四分"],			"1516.06.01"=>["(日蝕八分)"],		"1516.12.01"=>["夜日蝕九分"],
    "1517.06.01"=>["日蝕六分"],			"1517.11.01"=>["(日蝕七分)"],		"1518.05.01"=>["日蝕三分"],
    "1518.11.01"=>["(夜日蝕三分)"],		"1519.10.01"=>["夜日蝕九分"],		"1520.04.01"=>["(夜日蝕五分)"],
    "1520.09.01"=>["夜日蝕ニ分"],		"1521.03.01"=>["日蝕七分"],		"1521.09.01"=>["(夜日蝕九分)"],
    "1522.03.01"=>["日帯蝕六分"],		"1522.09.01"=>["(日蝕六分)"],		"1523.02.01"=>["(夜日蝕七分)"],
    "1523.07.01"=>["日蝕一分"],			"1524.01.01"=>["(夜日蝕九分)"],		"1524.07.01"=>["日帯蝕七分"],
    "1525.01.01"=>["夜日蝕五分"],		"1525.07.01"=>["(日帯蝕五分)"],		"1525.12.01"=>["日蝕四分"],
    "1527.05.01"=>["日蝕四分"],			"1527.11.01"=>["(夜日蝕ニ分)"],		"1528.05.01"=>["(日蝕ニ分)"],
    "1528.10.01"=>["日蝕五分"],			"1529.04.01"=>["(日帯蝕一分にミたす)"],	"1529.10.01"=>["日蝕五分"],
    "1530.03.01"=>["日蝕一分にミたす"],		"1531.03.01"=>["夜日蝕六分"],		"1531.08.01"=>["(夜日蝕一分にミたす)"],
    "1532.02.01"=>["(日蝕八分)"],		"1532.08.01"=>["夜日蝕三分"],		"1533.02.01"=>["(日蝕一分にミたす)"],
    "1533.08.01"=>["日蝕一分"],			"1534.06.01"=>["(日蝕五分)"],		"1534.12.01"=>["日帯蝕五分"],
    "1535.06.01"=>["(夜日蝕六分)"],		"1535.12.01"=>["(夜日蝕六分)"],		"1536.06.01"=>["夜日蝕七分"],
    "1537.04.01"=>["(日蝕三分)"],		"1538.04.01"=>["(日帯蝕四分)"],		"1538.10.01"=>["日帯蝕七分"],
    "1539.04.01"=>["夜日蝕三分"],		"1539.09.01"=>["(日蝕三分)"],		"1540.03.01"=>["日蝕一分にミたす"],
    "1540.09.01"=>["(日蝕一分にミたす)"],	"1541.08.01"=>["夜日蝕八分"],		"1542.02.01"=>["(日帯蝕七分)"],
    "1542.07.01"=>["日蝕九分"],			"1543.01.01"=>["日蝕四分"],		"1543.07.01"=>["(夜日蝕五分)"],
    "1544.07.01"=>["(日蝕三分)"],		"1544.11=01"=>["(夜日蝕ニ分)"],		"1545.05.01"=>["日帯蝕五分"],
    "1545.11.01"=>["(日蝕八分)"],		"1546.05.01"=>["(夜日蝕七分)"],		"1546.11.01"=>["夜日蝕ニ分"],
    "1547.05.01"=>["(日蝕七分)"],		"1547.10.01"=>["夜日蝕八分"],		"1548.09.01"=>["(夜日蝕五分)"],
    "1549.03.01"=>["日蝕九分"],			"1549.09.01"=>["(日蝕八分)"],		"1550.03.01"=>["(夜日蝕一分)"],
    "1550.08.01"=>["夜日蝕五分"],		"1551.02.01"=>["(夜日蝕七分)"],		"1551.08.01"=>["夜日蝕九分"],
    "1552.01.01"=>["夜日蝕七分"],		"1553.01.01"=>["日帯蝕九分"],		"1553.06.01"=>["(夜日蝕四分)"],
    "1553.12.01"=>["(日帯蝕八分)"],		"1554.06.01"=>["夜日蝕六分"],		"1554.12.01"=>["(夜日蝕三分)"],
    "1555.05.01"=>["(夜日蝕三分)"],		"1555.10=01"=>["日蝕四分"],		"1556.04.01"=>["(日蝕八分)"],
    "1556.10.01"=>["日帯蝕五分"],		"1557.04.01"=>["日蝕一分"],		"1557.10.01"=>["(日帯蝕八分)"],
    "1559.02.01"=>["(日蝕四分)"],		"1560.02.01"=>["(日蝕六分)"],		"1560.08.01"=>["夜日蝕五分"],
    "1561.02.01"=>["日帯蝕六分"],		"1561.07.01"=>["(日蝕四分)"],		"1562.01.01"=>["夜日蝕九分"],
    "1562.07.01"=>["(日蝕一分)"],		"1562.12.01"=>["(夜日蝕一分)"],		"1563.06.01"=>["夜日蝕四分"],
    "1563.12.01"=>["(夜日蝕九分)"],		"1564.05.01"=>["日蝕ニ分"],		"1564.11.01"=>["夜日蝕九分"],
    "1565.05.01"=>["(夜日蝕八分)"],		"1565.11.01"=>["夜日蝕一分"],		"1566.04.01"=>["日帯蝕五分"],
    "1567.03.01"=>["日帯蝕五分"],		"1567.09.01"=>["(夜日蝕ニ分)"],		"1568.03.01"=>["(夜日蝕三分)"],
    "1568.09.01"=>["日蝕七分"],			"1570.01.01"=>["日蝕一分"],		"1570.07.01"=>["(日帯蝕八分)"],
    "1571.01.01"=>["夜日蝕九分"],		"1571.07.01"=>["(日蝕一分にミたす)"],	"1572.01.01"=>["(日帯蝕四分)"],
    "1572.06.01"=>["日蝕六分"],			"1572.12.01"=>["(夜日蝕一分にミたす)"],	"1573.11.01"=>["夜日蝕五分"],
    "1574.05.01"=>["(夜日蝕九分)"],		"1574.11.01"=>["夜日蝕三分"],		"1575.04.01"=>["日蝕九分半余"],
    "1575.10.01"=>["(夜日蝕九分半余)"],		"1576.04.01"=>["日帯蝕九分"],		"1576.10.01"=>["(日帯蝕七分)"],
    "1577.03.01"=>["(夜日蝕九分)"],		"1577.08.01"=>["日蝕一分"],		"1578.02.01"=>["(夜日蝕六分)"],
    "1578.08.01"=>["夜日蝕五分"],		"1579.02.01"=>["夜日蝕八分"],		"1579.08.01"=>["(日蝕八分)"],
    "1580.02.01"=>["日蝕四分"],			"1580.12.01"=>["(日帯蝕四分)"],		"1581.06.01"=>["日蝕ニ分"],
    "1581.12.01"=>["(夜日蝕九分)"],		"1582.06.01"=>["日蝕八分"],		"1582.12.01"=>["日蝕一分"],
    "1583.05.01"=>["(夜日蝕六分)"],		"1583.11.01"=>["日蝕五分"],		"1585.04.01"=>["夜日蝕四分"],
    "1585.09.01"=>["(夜日蝕三分)"],		"1586.03.01"=>["(日蝕九分)"],		"1586.09.01"=>["夜日蝕七分"],
    "1587.03.01"=>["(日蝕一分)"],		"1587.09.01"=>["日蝕四分"],		"1588.07.01"=>["(日蝕三分)"],
    "1589.01.01"=>["日蝕三分"],			"1589.07.01"=>["(日蝕八分)"],		"1590.01.01"=>["(夜日蝕三分)"],
    "1590.07.01"=>["日帯蝕五分"],		"1591.01.01"=>["(日帯蝕四分)"],		"1591.06.01"=>["夜日蝕五分"],
    "1591.11.01"=>["日蝕一分"],			"1592.05.01"=>["(夜日蝕五分)"],		"1592.11.01"=>["日帯蝕八分"],
    "1593.05.01"=>["夜日蝕九分半余"],		"1593.10.01"=>["(日蝕五分)"],		"1594.04.01"=>["日蝕ニ分"],
    "1594.10.01"=>["(日蝕一分)"],		"1595.09.01"=>["夜日蝕七分"],		"1596.03.01"=>["(夜日蝕五分)"],
    "1596.08.01"=>["日蝕九分"],			"1597.02.01"=>["日蝕三分"],		"1597.08.01"=>["(夜日蝕三分)"],
    "1598.08.01"=>["(夜日蝕八分)"],		"1599.06.01"=>["日蝕ニ分"],		"1599.12.01"=>["(日蝕九分半余)"],
    "1600.06.01"=>["夜日蝕一分にミたす"],	"1600.12.01"=>["夜日蝕八分"],		"1601.06.01"=>["(日蝕九分)"],
    "1601.11=01"=>["夜日蝕三分"],		"1602.10.01"=>["(夜日蝕四分)"],		"1603.04.01"=>["日蝕九分"],
    "1603.10.01"=>["(日蝕六分)"],		"1604.04.01"=>["(日蝕一分)"],		"1604.09.01"=>["夜日蝕七分"],
    "1605.03.01"=>["(日蝕七分)"],		"1605.09.01"=>["夜日蝕九分"],		"1606.02.01"=>["日帯蝕六分"],
    "1607.02.01"=>["日帯蝕六分"],		"1607.07.01"=>["(夜日蝕ニ分)"],		"1608.01.01"=>["(日蝕八分)"],
    "1608.07.01"=>["夜日蝕ニ分"],		"1609.12.01"=>["日帯蝕一分"],		"1610.05.01"=>["(日蝕八分)"],
    "1610.11.01"=>["日帯蝕九分半余"],		"1611.05.01"=>["日帯蝕ニ分"],		"1611.11.01"=>["(日帯蝕九分)"],
    "1612.05.01"=>["日帯蝕三分"],		"1612.10=01"=>["(夜日蝕九分半余)"],	"1613.03.01"=>["(日蝕一分)"],
    "1614.03.01"=>["(日蝕九分半余)"],		"1614.09.01"=>["夜日蝕一分"],		"1615.03.01"=>["日帯蝕八分"],
    "1615.08.01"=>["(日帯蝕ニ分)"],		"1616.02.01"=>["夜日蝕九分"],		"1616.08.01"=>["(日蝕四分)"],
    "1617.07.01"=>["夜日蝕七分"],		"1618.01.01"=>["(夜日蝕五分)"],		"1618.06.01"=>["日帯蝕八分"],
    "1618.12.01"=>["夜日蝕四分"],		"1619.06.01"=>["(日帯蝕七分)"],		"1619.12.01"=>["夜日蝕四分"],
    "1620.06.01"=>["(日蝕三分)"],		"1621.04.01"=>["日蝕五分"],		"1621.10.01"=>["(夜日蝕一分)"],
    "1622.04.01"=>["(夜日蝕九分)"],		"1622.10.01"=>["日蝕一分にミたす"],	"1623.04.01"=>["(夜日蝕五分)"],
    "1624.08.01"=>["(夜日蝕七分)"],		"1625.02.01"=>["夜日蝕五分"],		"1625.08.01"=>["(日帯蝕一分)"],
    "1626.02.01"=>["(夜日蝕ニ分)"],		"1626.07.01"=>["日帯蝕七分"],		"1627.07.01"=>["日蝕ニ分"],
    "1627.12.01"=>["夜日蝕四分"],		"1628.06.01"=>["(日帯蝕四分)"],		"1628.12.01"=>["夜日蝕九分"],
    "1629.05.01"=>["(日蝕七分)"],		"1629.11.01"=>["(夜日蝕ニ分)"],		"1630.05.01"=>["夜日蝕八分"],
    "1630.11.01"=>["(日蝕四分)"],		"1631.04.01"=>["(日帯蝕五分)"],		"1631.10.01"=>["日蝕ニ分"],
    "1632.03.01"=>["(夜日蝕ニ分)"],		"1632.09.01"=>["夜日蝕三分"],		"1633.03.01"=>["夜日蝕九分"],
    "1633.09.01"=>["(日蝕九分半余)"],		"1634.03.01"=>["日蝕四分"],		"1635.01.01"=>["(夜日蝕七分)"],
    "1636.01.01"=>["(日帯蝕七分)"],		"1636.07.01"=>["日蝕八分"],		"1637.01.01"=>["日蝕八分"],
    "1637.06.01"=>["(夜日蝕一分)"],		"1637.12.01"=>["日蝕三分"],		"1639.05.01"=>["夜日蝕三分"],
    "1639.11.01"=>["(日帯蝕九分)"],		"1640.04.01"=>["(日帯蝕九分半余)"],	"1640.10.01"=>["夜日蝕九分"],
    "1641.04.01"=>["(日蝕一分)"],		"1641.10.01"=>["日蝕六分"],		"1642.09.01"=>["(日蝕ニ分)"],
    "1643.02.01"=>["日蝕八分"],			"1643.08.01"=>["(日蝕九分)"],		"1644.02.01"=>["(夜日蝕四分)"],
    "1644.08.01"=>["日蝕四分"],			"1645.02.01"=>["(日帯蝕六分)"],		"1645.07.01"=>["夜日蝕九分"],
    "1645.12.01"=>["日蝕四分"],			"1646.06.01"=>["(夜日蝕六分)"],		"1646.12.01"=>["日帯蝕八分"],
    "1647.06.01"=>["(日帯蝕六分)"],		"1647.12.01"=>["(日蝕九分)"],		"1648.05.01"=>["日蝕五分"],
    "1648.11.01"=>["(夜日蝕三分)"],		"1649.10.01"=>["夜日蝕七分"],		"1650.04.01"=>["(夜日蝕一分)"],
    "1650.10.01"=>["日蝕九分半余"],		"1651.03.01"=>["日蝕ニ分"],		"1651.09.01"=>["(夜日蝕五分)"],
    "1652.09.01"=>["(夜日蝕六分)"],		"1654.01.01"=>["(日蝕五分)"],		"1654.07.01"=>["夜日蝕一分にミたす"],
    "1655.01.01"=>["夜日蝕三分"],		"1655.07.01"=>["(日蝕ニ分)"],		"1656.06.01"=>["(夜日蝕五分)"],
    "1656.11.01"=>["(夜日蝕一分)"],		"1657.05.01"=>["日蝕六分"],		"1657.11.01"=>["(日帯蝕七分)"],
    "1658.05.01"=>["(日蝕六分)"],		"1658.11.01"=>["日帯蝕九分"],		"1659.04.01"=>["(日蝕八分)"],
    "1659.10.01"=>["夜日蝕六分"],		"1660.03.01"=>["日帯蝕五分"],		"1661.03.01"=>["夜日蝕四分"],
    "1661.08=01"=>["(夜日蝕ニ分)"],		"1662.02.01"=>["(日蝕九分)"],		"1662.08.01"=>["夜日蝕九分半余"],
    "1664.06.01"=>["(日蝕五分)"],		"1664.12.01"=>["日帯蝕七分"],		"1665.06.01"=>["(夜日蝕一分にミたす)"],
    "1665.12.01"=>["(夜日蝕ニ分)"],		"1666.06.01"=>["日蝕七分"],		"1666.12.01"=>["(夜日蝕四分)"],
    "1668.04.01"=>["(日蝕六分)"],		"1668.10.01"=>["夜日蝕一分"],		"1669.04.01"=>["日蝕九分"],
    "1669.10.01"=>["(夜日蝕六分)"],		"1670.03.01"=>["夜日蝕八分"],		"1670.09.01"=>["(日蝕六分)"],
    "1671.02.01"=>["(日蝕一分にミたす)"],	"1671.08.01"=>["日帯蝕四分"],		"1672.02.01"=>["(夜日蝕九分)"],
    "1672.07.01"=>["夜日蝕ニ分"],		"1673.01.01"=>["夜日蝕ニ分"],		"1673.07.01"=>["(日蝕八分)"],
    "1674.01.01"=>["日帯蝕七分"],		"1674.07.01"=>["(日蝕五分)"],		"1675.05.01"=>["日蝕ニ分"],
    "1675.11.01"=>["(夜日蝕九分)"],		"1676.05.01"=>["日帯蝕四分"],		"1676.11.01"=>["日蝕一分"],
    "1677.05.01"=>["(日帯蝕九分)"],		"1677.11.01"=>["日帯蝕ニ分"],		"1678.09.01"=>["(夜日蝕七分)"],
    "1679.03.01"=>["夜日蝕ニ分"],		"1679.09.01"=>["(夜日蝕一分)"],		"1680.03.01"=>["(夜日蝕六分)"],
    "1680.08=01"=>["日帯蝕八分"],		"1681.02.01"=>["(日蝕一分)"],		"1681.08.01"=>["日蝕四分"],
    "1682.01.01"=>["夜日蝕九分"],		"1683.01.01"=>["夜日蝕ニ分"],		"1683.06.01"=>["(日蝕ニ分)"],
    "1683.12.01"=>["(夜日蝕九分)"],		"1684.06.01"=>["夜日蝕六分"],		"1684.12.01"=>["(日蝕一分)"],

    # 暫定的な日食データ - 主に大崎正次編『近世日本天文史料』による(*付きは計算で補ったもの)
    "1688.04.01"=>["日そく五分"],		"1690.08.01"=>["日そく四分"],		"1691.02.01"=>["日そく四分"],
    "1692.01.01"=>["日そく六分"],		"1697.03.01"=>["日そく五分"],		"1700.01.01"=>["日そくニ分半"],
    "1701.01.01"=>["日帯そく五分"],		"1702.07.01"=>["日帯そく七分"],		"1704.11.01"=>["日そく九分"],
    "1709.08.01"=>["日そく四分"],		"1712.06.01"=>["日そく九分"],		"1716.03.01"=>["日そく五分"],
    "1719.01.01"=>["日帯そく三分"],		"1720.07.01"=>["日そく七分"],		"1721.07.01"=>["日帯そく"],
    "1723.05.01"=>["日そく三分"],		"1725.09.01"=>["日帯そく一分半"],
    "1731.12.01"=>["日そく九分餘"],		"1735.09.01"=>["日そく九分餘"],		"1740.11.01"=>["日帯そく五分"],
    "1744.09.01"=>["日そく四分"],		"1745.03.01"=>["日そく二分"],
    "1746.02.01"=>["日そく七分"],		"1747.07.01"=>["日帯そく"],             #"1751.05.01"=>["日そくニ分半"],
    "1755.02.01"=>["日帯そく"],			"1760.05.01"=>["日帯そく"],		"1767.01.01"=>["日そくニ分半"],
    "1768.12.01"=>["日そくニ分半"],		"1770.05.01"=>["日そく皆既"],		"1773.03.01"=>["日そく三分半"],
    "1774.08.01"=>["日そく五分"],		"1775.08.01"=>["日そく七分"],		"1775.12=01"=>["日蝕一分ミたす"],
    "1784.07.01"=>["日そくニ分"],		"1785.07.01"=>["日そく六分"],		"1788.05.01"=>["日帯そく"],
    "1789.10.01"=>["日そく六分"],		"1794.12.01"=>["日そく九分余"],		"1795.12.01"=>["日帯そく"],
    "1796.06.01"=>["日そく七分余"],		"1798.10.01"=>["日そく四分半"],		"1800.04.01"=>["日そく九分"],
    "1802.08.01"=>["日蝕九分"],			"1808.10.01"=>["日蝕ニ分半"],		"1810.03.01"=>["日そく一分半"],
    "1814.06.01"=>["日そく四分"],		"1815.06.01"=>["日蝕一分"],		"1817.04.01"=>["日そくニ分"],
    "1817.10.01"=>["日そく三分"],		"1818.04.01"=>["日帯そく四分"],		"1821.02.01"=>["日そく六分"],
    "1823.06.01"=>["日そく一分にミたす"],	"1824.06.01"=>["日そく八分半"],		"1828.09.01"=>["日そく一分半"],
    "1829.09.01"=>["日そく六分"],		"1833.06.01"=>["日蝕四分半"],		"1839.08.01"=>["日帯蝕"],
    "1840.02.01"=>["日蝕五分"],			"1842.06.01"=>["日蝕七分半"],		"1849.02.01"=>["日蝕九分半"],
    "1850.01.01"=>["日そく三分半"],		"1850.07.01"=>["日帯そく"],		"1854.05.01"=>["日帯そく"],
    "1856.09.01"=>["日そく四分半"],		"1857.08.01"=>["日そく京都にてハみへす"],"1861.06.01"=>["日そく三分"],
    "1862.11.01"=>["日そく一分半"],		"1864.04.01"=>["日そく五分"],		"1869.07.01"=>["日帯そく五分*"],
    "1872.05.01"=>["日そく七分*"],

    # 宣明暦の月食データ - http://www.wagoyomi.info/senmyoureki_gessyoku.html に基づく(半影食は“()”で囲んでいる)
    "0862.02.15"=>["(月蝕ニ分)"],		"0862.08.15"=>["(昼月蝕三分)"],		"0863.01.16"=>["月蝕五分"],
    "0863.06=15"=>["昼月蝕四分"],		"0863.12.15"=>["月蝕皆既"],		"0864.06.15"=>["月蝕皆既"],
    "0864.12.14"=>["月蝕皆既"],			"0865.06.16"=>["昼月蝕七分"],		"0865.12.15"=>["(月帯蝕ニ分)"],
    "0866.04.16"=>["(昼月蝕一分)"],		"0866.10.16"=>["月帯蝕三分"],		"0867.04.15"=>["昼月蝕八分"],
    "0867.10.16"=>["月蝕皆既"],			"0868.04.14"=>["月帯蝕皆既"],		"0868.10.16"=>["昼月蝕皆既"],
    "0869.03.14"=>["月蝕四分"],			"0869.09.16"=>["(昼月蝕三分)"],		"0870.02.16"=>["昼月蝕四分"],
    "0870.08.14"=>["(月蝕三分)"],		"0871.02.16"=>["月蝕皆既"],		"0871.08.15"=>["月帯蝕皆既"],
    "0872.01.16"=>["昼月蝕八分"],		"0872.07.15"=>["昼月蝕皆既"],		"0873.01.15"=>["(昼月蝕ニ分)"],
    "0873.07.16"=>["(昼月蝕ニ分)"],		"0873.12.14"=>["(月蝕四分)"],		"0874.05.16"=>["昼月蝕六分"],
    "0874.11.14"=>["月蝕皆既"],			"0875.05.16"=>["月蝕皆既"],		"0875.11.15"=>["月帯蝕皆既"],
    "0876.05.15"=>["月蝕六分"],			"0876.11.16"=>["昼月蝕三分"],		"0877.03.15"=>["(昼月蝕ニ分)"],
    "0877.09.16"=>["(昼月蝕三分)"],		"0878.03.14"=>["月蝕皆既"],		"0878.09.16"=>["昼月蝕皆既"],
    "0879.03.15"=>["月帯蝕皆既"],		"0879.09.15"=>["昼月蝕皆既"],		"0880.02.16"=>["昼月蝕ニ分"],
    "0880.08.15"=>["昼月蝕三分"],		"0881.01.16"=>["昼月蝕四分"],		"0881.07.15"=>["昼月蝕四分"],
    "0882.01.16"=>["昼月蝕皆既"],		"0882.07.16"=>["昼月蝕皆既"],		"0882.12.15"=>["昼月蝕皆既"],
    "0883.06.16"=>["月蝕七分"],			"0883.12.15"=>["(昼月蝕ニ分)"],		"0884.05.16"=>["(月蝕一分)"],
    "0884.06.16"=>["(昼月蝕一分にミたす)"],	"0884.11.15"=>["昼月蝕三分"],		"0885.04.15"=>["月蝕七分"],
    "0885.10.16"=>["月帯蝕皆既"],		"0886.04.14"=>["月蝕皆既"],		"0886.10.16"=>["月帯蝕皆既"],
    "0887.04.15"=>["昼月蝕四分"],		"0887.10.15"=>["(月蝕三分)"],		"0888.02.15"=>["月蝕三分"],
    "0888.08.15"=>["(昼月蝕三分)"],		"0889.02.16"=>["昼月蝕皆既"],		"0889.08.15"=>["昼月蝕皆既"],
    "0890.02.16"=>["月帯蝕皆既"],		"0890.08.15"=>["月蝕皆既"],		"0891.01.16"=>["(月蝕ニ分)"],
    "0891.07.16"=>["昼月蝕ニ分"],		"0891.12.14"=>["(月蝕四分)"],		"0892.06.16"=>["月蝕五分"],
    "0892.12.15"=>["昼月蝕皆既"],		"0893.05=16"=>["月帯蝕皆既"],		"0893.11.15"=>["月蝕皆既"],
    "0894.05.16"=>["昼月蝕六分"],		"0894.11.16"=>["月蝕三分"],		"0895.04.14"=>["(月蝕一分)"],
    "0895.10.16"=>["(昼月蝕三分)"],		"0896.03.15"=>["昼月蝕九分"],		"0896.09.15"=>["月帯蝕皆既"],
    "0897.03.15"=>["月蝕皆既"],			"0897.09.15"=>["月帯蝕皆既"],		"0898.03.16"=>["月蝕三分"],
    "0898.09.14"=>["月蝕四分"],			"0899.01.16"=>["(昼月蝕四分)"],		"0899.07.15"=>["月蝕三分"],
    "0900.01.15"=>["月帯蝕皆既"],		"0900.07.16"=>["月帯蝕皆既"],		"0901.01.15"=>["月帯蝕皆既"],
    "0901.06=16"=>["昼月蝕皆既"],		"0901.12.14"=>["(月蝕三分)"],		"0902.06.16"=>["(昼月蝕一分)"],
    "0902.11.15"=>["月蝕三分"],			"0903.05.14"=>["月帯蝕七分"],		"0903.11.16"=>["月帯蝕皆既"],
    "0904.04.15"=>["昼月蝕皆既"],		"0904.10.16"=>["月蝕皆既"],		"0905.04.15"=>["月帯蝕五分"],
    "0905.10.16"=>["(昼月蝕三分)"],		"0906.03.15"=>["(月蝕三分)"],		"0906.09.15"=>["(昼月蝕ニ分)"],
    "0907.02.16"=>["月帯蝕皆既"],		"0907.08.14"=>["月蝕九分"],		"0908.02.16"=>["月蝕皆既"],
    "0908.08.15"=>["昼月蝕皆既"],		"0909.02.16"=>["(月帯蝕ニ分)"],		"0909.08.15"=>["月蝕三分"],
    "0909.12.15"=>["(昼月蝕三分)"],		"0910.06.16"=>["月帯蝕五分"],		"0910.12.14"=>["月蝕皆既"],
    "0911.06.16"=>["昼月蝕皆既"],		"0911.12.15"=>["昼月蝕皆既"],		"0912.05=15"=>["昼月蝕七分"],
    "0912.11.15"=>["月蝕三分"],			"0913.04.15"=>["(昼月蝕一分)"],		"0913.05.15"=>["(昼月蝕一分にミたす)"],
    "0913.10.16"=>["(月蝕三分)"],		"0914.04.15"=>["月帯蝕八分"],		"0914.10.15"=>["月蝕九分"],
    "0915.03.16"=>["昼月蝕皆既"],		"0915.09.14"=>["月蝕皆既"],		"0916.03.16"=>["月蝕三分"],
    "0916.09.15"=>["月帯蝕四分"],		"0917.02.16"=>["(月蝕四分)"],		"0917.08.15"=>["(昼月蝕三分)"],
    "0918.01.15"=>["月蝕皆既"],			"0918.07.15"=>["月蝕皆既"],		"0919.01.14"=>["月蝕皆既"],
    "0919.07.16"=>["昼月蝕皆既"],		"0920.01.15"=>["(昼月蝕三分)"],		"0920.06=16"=>["(月蝕ニ分)"],
    "0920.11.16"=>["昼月蝕三分"],		"0921.05.15"=>["昼月蝕六分"],		"0921.11.16"=>["月蝕皆既"],
    "0922.05.15"=>["昼月蝕皆既"],		"0922.11.16"=>["昼月蝕皆既"],		"0923.04=14"=>["月蝕六分"],
    "0923.10.16"=>["(月帯蝕三分)"],		"0924.03.16"=>["(昼月蝕ニ分)"],		"0924.09.15"=>["(月蝕ニ分)"],
    "0925.03.16"=>["月蝕皆既"],			"0925.09.14"=>["月蝕八分"],		"0926.02.16"=>["昼月蝕皆既"],
    "0926.08.15"=>["月帯蝕皆既"],		"0927.02.16"=>["(昼月蝕三分)"],		"0927.08.16"=>["昼月蝕三分"],
    "0928.01.14"=>["(月蝕三分)"],		"0928.07.16"=>["昼月蝕四分"],		"0928.12.15"=>["月帯蝕皆既"],
    "0929.06.16"=>["月帯蝕皆既"],		"0929.12.15"=>["月蝕皆既"],		"0930.06.15"=>["月帯蝕八分"],
    "0930.12.16"=>["昼月蝕三分"],		"0931.05.15"=>["(昼月蝕一分にミたす)"],	"0931.05=14"=>["(月蝕一分)"],
    "0931.10.16"=>["(昼月蝕三分)"],		"0932.04.14"=>["月蝕七分"],		"0932.10.16"=>["昼月蝕九分"],
    "0933.04.15"=>["月帯蝕皆既"],		"0933.10.15"=>["昼月蝕皆既"],		"0934.03.16"=>["昼月蝕四分"],
    "0934.09.15"=>["昼月蝕四分"],		"0935.02.16"=>["(昼月蝕三分)"],		"0935.08.15"=>["(昼月蝕ニ分)"],
    "0936.02.16"=>["月帯蝕皆既"],		"0936.08.16"=>["昼月蝕皆既"],		"0937.01.16"=>["昼月蝕皆既"],
    "0937.07.16"=>["月蝕皆既"],			"0938.01.16"=>["月帯蝕三分"],		"0938.07.16"=>["(昼月蝕ニ分)"],
    "0938.12.15"=>["月蝕三分"],			"0939.06.15"=>["昼月蝕五分"],		"0939.11.16"=>["昼月蝕皆既"],
    "0940.05.14"=>["月蝕皆既"],			"0940.11.16"=>["月蝕皆既"],		"0941.05.15"=>["昼月蝕七分"],
    "0941.11.15"=>["(月蝕三分)"],		"0942.03=16"=>["(月帯蝕ニ分)"],		"0942.09.14"=>["(月蝕ニ分)"],
    "0943.03.16"=>["昼月蝕九分"],		"0943.09.15"=>["昼月蝕八分"],		"0944.03.16"=>["月帯蝕皆既"],
    "0944.09.15"=>["月蝕皆既"],			"0945.02.15"=>["(月蝕三分)"],		"0945.08.16"=>["月帯蝕四分"],
    "0946.01.15"=>["(月帯蝕三分)"],		"0946.07.16"=>["(月蝕四分)"],		"0947.01.15"=>["昼月蝕皆既"],
    "0947.07.15"=>["月蝕皆既"],			"0947.12.15"=>["月帯蝕皆既"],		"0948.06.15"=>["月帯蝕皆既"],
    "0948.12.16"=>["月蝕三分"],			"0949.06.15"=>["(昼月蝕ニ分)"],		"0949.11.16"=>["(月帯蝕三分)"],
    "0950.05.15"=>["昼月蝕六分"],		"0950.10.15"=>["月帯蝕九分"],		"0951.04.15"=>["月蝕皆既"],
    "0951.10.15"=>["月帯蝕皆既"],		"0952.04.16"=>["昼月蝕五分"],		"0952.10.14"=>["月蝕四分"],
    "0953.02.16"=>["(昼月蝕三分)"],		"0953.08.15"=>["(月蝕ニ分)"],		"0954.02.15"=>["昼月蝕九分"],
    "0954.08.16"=>["月帯蝕皆既"],		"0955.02.15"=>["月帯蝕皆既"],		"0955.08.17"=>["月帯蝕皆既"],
    "0956.01.14"=>["月蝕三分"],			"0956.07.16"=>["(昼月蝕三分)"],		"0956.12.15"=>["(月蝕三分)"],
    "0957.06.15"=>["(月蝕四分)"],		"0957.12.16"=>["月蝕皆既"],		"0958.06.15"=>["月帯蝕皆既"],
    "0958.11.16"=>["月帯蝕皆既"],		"0959.05.15"=>["昼月蝕七分"],		"0959.11.16"=>["(昼月蝕三分)"],
    "0960.04.15"=>["(月蝕一分)"],		"0960.10.15"=>["(昼月蝕ニ分)"],		"0961.03=16"=>["昼月蝕八分"],
    "0961.09.14"=>["月蝕八分"],			"0962.03.16"=>["月蝕皆既"],		"0962.09.15"=>["昼月蝕皆既"],
    "0963.03.15"=>["月蝕四分"],			"0963.09.15"=>["月蝕四分"],		"0964.01.15"=>["(昼月蝕三分)"],
    "0964.07.16"=>["(月蝕三分)"],		"0965.01.15"=>["月蝕皆既"],		"0965.07.16"=>["昼月蝕皆既"],
    "0966.01.15"=>["昼月蝕皆既"],		"0966.07.15"=>["昼月蝕皆既"],		"0966.12.15"=>["月蝕三分"],
    "0967.06.15"=>["(昼月蝕ニ分)"],		"0967.11.16"=>["(月蝕三分)"],		"0968.05.15"=>["昼月蝕六分"],
    "0968.11.15"=>["月蝕九分"],			"0969.05.16"=>["昼月蝕皆既"],		"0969.10.14"=>["月蝕皆既"],
    "0970.04.16"=>["月蝕五分"],			"0970.10.15"=>["月帯蝕五分"],		"0971.03.16"=>["(月蝕ニ分)"],
    "0971.09.16"=>["(昼月蝕ニ分)"],		"0972.02=15"=>["月蝕九分"],		"0972.08.15"=>["月蝕九分"],
    "0973.02.14"=>["月蝕皆既"],			"0973.08.16"=>["昼月蝕皆既"],		"0974.02.15"=>["昼月蝕三分"],
    "0974.08.16"=>["(月蝕三分)"],		"0974.12.16"=>["(昼月蝕三分)"],		"0975.06.15"=>["(昼月蝕三分)"],
    "0975.12.16"=>["月蝕皆既"],			"0976.06.15"=>["昼月蝕皆既"],		"0976.12.16"=>["昼月蝕皆既"],
    "0977.06.14"=>["月蝕皆既"],			"0977.11.16"=>["(月帯蝕三分)"],		"0978.04.16"=>["(昼月蝕一分にミたす)"],
    "0978.05.15"=>["(昼月蝕一分)"],		"0978.10.14"=>["(月蝕一分)"],		"0979.04.16"=>["月蝕八分"],
    "0979.10.14"=>["月蝕八分"],			"0980.03=16"=>["昼月蝕皆既"],		"0980.09.15"=>["月帯蝕皆既"],
    "0981.03.16"=>["昼月蝕四分"],		"0981.09.16"=>["昼月蝕四分"],		"0982.02.14"=>["(月蝕ニ分)"],
    "0982.08.16"=>["(昼月蝕ニ分)"],		"0983.01.15"=>["月帯蝕皆既"],		"0983.07.16"=>["月帯蝕九分"],
    "0984.01.15"=>["月蝕皆既"],			"0984.07.15"=>["月帯蝕皆既"],		"0985.01.16"=>["昼月蝕三分"],
    "0985.07.14"=>["月蝕三分"],			"0985.11.16"=>["(昼月蝕三分)"],		"0986.05.15"=>["月蝕五分"],
    "0986.11.16"=>["昼月蝕九分"],		"0987.05.16"=>["昼月蝕皆既"],		"0987.11.15"=>["昼月蝕皆既"],
    "0988.05.16"=>["昼月蝕六分"],		"0988.10.15"=>["月帯蝕五分"],		"0989.03.15"=>["(月蝕ニ分)"],
    "0989.09.15"=>["(昼月蝕一分)"],		"0990.03.16"=>["月帯蝕八分"],		"0990.09.16"=>["昼月蝕八分"],
    "0991.02=15"=>["昼月蝕皆既"],		"0991.08.16"=>["月蝕皆既"],		"0992.02.15"=>["昼月蝕四分"],
    "0992.08.15"=>["月蝕四分"],			"0993.01.15"=>["(月蝕三分)"],		"0993.07.15"=>["(昼月蝕三分)"],
    "0993.12.16"=>["昼月蝕皆既"],		"0994.06.15"=>["月帯蝕皆既"],		"0994.12.16"=>["月蝕皆既"],
    "0995.06.15"=>["月帯蝕皆既"],		"0995.12.15"=>["(月蝕三分)"],		"0996.06.15"=>["(月蝕ニ分)"],
    "0996.10.15"=>["(月蝕一分)"],		"0997.04.17"=>["昼月蝕七分"],		"0997.10.15"=>["昼月蝕八分"],
    "0998.04.16"=>["昼月蝕皆既"],		"0998.10.15"=>["月蝕皆既"],		"0999.03=15"=>["昼月蝕五分"],
    "0999.09.16"=>["月蝕四分"],			"1000.02.15"=>["(月蝕ニ分)"],		"1000.08.16"=>["(月蝕ニ分)"],
    "1001.02.15"=>["昼月蝕皆既"],		"1001.08.15"=>["月蝕四分"],		"1002.01.16"=>["月帯蝕皆既"],
    "1002.07.15"=>["月蝕皆既"],			"1003.01.16"=>["月蝕三分"],		"1003.07.14"=>["月蝕四分"],
    "1003.12.16"=>["(月蝕三分)"],		"1004.06.15"=>["昼月蝕四分"],		"1004.11.15"=>["月蝕九分"],
    "1005.05.15"=>["月蝕皆既"],			"1005.11.15"=>["月蝕皆既"],		"1006.05.16"=>["昼月蝕七分"],
    "1006.11.14"=>["月蝕五分"],			"1007.04.16"=>["(昼月蝕一分)"],		"1007.09.15"=>["(月蝕一分)"],
    "1008.03.15"=>["昼月蝕八分"],		"1008.09.16"=>["月帯蝕八分"],		"1009.03.15"=>["昼月蝕皆既"],
    "1009.09.16"=>["月帯蝕皆既"],		"1010.02=14"=>["月蝕四分"],		"1010.08.16"=>["昼月蝕四分"],
    "1011.01.16"=>["(月帯蝕三分)"],		"1011.07.15"=>["(月蝕ニ分)"],		"1012.01.16"=>["月蝕皆既"],
    "1012.07.14"=>["月蝕九分"],			"1012.12.16"=>["月帯蝕皆既"],		"1013.06.15"=>["昼月蝕皆既"],
    "1013.12.16"=>["(昼月蝕三分)"],		"1014.06.16"=>["昼月蝕ニ分"],		"1014.11.15"=>["(昼月蝕一分)"],
    "1015.05.16"=>["昼月蝕六分"],		"1015.10.14"=>["月蝕八分"],		"1016.04.16"=>["月帯蝕皆既"],
    "1016.10.15"=>["昼月蝕皆既"],		"1017.04.15"=>["月蝕六分"],		"1017.10.15"=>["月蝕五分"],
    "1018.03.15"=>["(昼月蝕ニ分)"],		"1018.08.16"=>["(月蝕ニ分)"],		"1019.02.14"=>["月蝕八分"],
    "1019.08.16"=>["月帯蝕八分"],		"1020.02.15"=>["昼月蝕皆既"],		"1020.08.15"=>["月帯蝕皆既"],
    "1021.01.16"=>["昼月蝕三分"],		"1021.07.15"=>["昼月蝕四分"],		"1021.12.16"=>["(月蝕三分)"],
    "1022.06.15"=>["昼月蝕三分"],		"1022.12.15"=>["月蝕九分"],		"1023.06.16"=>["昼月蝕皆既"],
    "1023.11.14"=>["月蝕皆既"],			"1024.05.16"=>["月蝕八分"],		"1024.11.15"=>["昼月蝕四分"],
    "1025.04.16"=>["(昼月蝕一分にミたす)"],	"1025.05.16"=>["(昼月蝕一分にミたす)"],	"1025.10.16"=>["(昼月蝕一分)"],
    "1026.04.15"=>["月帯蝕七分"],		"1026.09.15"=>["月蝕八分"],		"1027.03.14"=>["月蝕皆既"],
    "1027.09.16"=>["昼月蝕皆既"],		"1028.03.15"=>["昼月蝕五分"],		"1028.09.16"=>["月蝕五分"],
    "1029.02.16"=>["(昼月蝕ニ分)"],		"1029.07.14"=>["(月蝕ニ分)"],		"1030.01.16"=>["月帯蝕皆既"],
    "1030.07.15"=>["昼月蝕八分"],		"1031.01.16"=>["昼月蝕皆既"],		"1031.07.15"=>["月帯蝕皆既"],
    "1031.12.15"=>["(月帯蝕三分)"],		"1032.06.16"=>["昼月蝕三分"],		"1032.11.14"=>["(月蝕ニ分)"],
    "1033.05.16"=>["月蝕五分"],			"1033.11.15"=>["月帯蝕八分"],		"1034.05.15"=>["月帯蝕皆既"],
    "1034.10.15"=>["月蝕皆既"],			"1035.04.16"=>["月帯蝕六分"],		"1035.10.16"=>["昼月蝕五分"],
    "1036.03.14"=>["(月蝕一分)"],		"1036.09.16"=>["(昼月蝕一分)"],		"1037.03.15"=>["月帯蝕八分"],
    "1037.08.16"=>["昼月蝕八分"],		"1038.02.15"=>["月蝕皆既"],		"1038.08.15"=>["昼月蝕皆既"],
    "1039.02.16"=>["昼月蝕四分"],		"1039.08.14"=>["月帯蝕五分"],		"1039.12=16"=>["(昼月蝕三分)"],
    "1040.06.15"=>["(月蝕三分)"],		"1040.12.15"=>["昼月蝕九分"],		"1041.06.16"=>["昼月蝕皆既"],
    "1041.12.15"=>["昼月蝕皆既"],		"1042.06.16"=>["月帯蝕皆既"],		"1042.11.15"=>["月蝕四分"],
    "1043.05.16"=>["(昼月蝕一分)"],		"1043.10.15"=>["(月蝕一分)"],		"1044.04.15"=>["月蝕七分"],
    "1044.10.16"=>["昼月蝕八分"],		"1045.04.15"=>["月帯蝕皆既"],		"1045.09.16"=>["月蝕皆既"],
    "1046.03.15"=>["昼月蝕五分"],		"1046.09.15"=>["月蝕五分"],		"1047.02.15"=>["(月蝕ニ分)"],
    "1047.08.15"=>["(昼月蝕一分)"],		"1048.01=16"=>["昼月蝕九分"],		"1048.07.15"=>["昼月蝕八分"],
    "1049.01.16"=>["月蝕皆既"],			"1049.07.14"=>["月帯蝕皆既"],		"1050.01.15"=>["月蝕四分"],
    "1050.07.15"=>["月蝕三分"],			"1050.11.16"=>["(昼月蝕ニ分)"],		"1051.05.16"=>["月帯蝕五分"],
    "1051.11.15"=>["月帯蝕八分"],		"1052.05.16"=>["昼月蝕皆既"],		"1052.11.15"=>["月帯蝕皆既"],
    "1053.05.15"=>["昼月蝕七分"],		"1053.10.16"=>["月蝕五分"],		"1054.04.15"=>["(昼月蝕一分にミたす)"],
    "1054.09.16"=>["(月蝕一分)"],		"1055.03.15"=>["昼月蝕八分"],		"1055.09.15"=>["月蝕八分"],
    "1056.03.16"=>["月帯蝕皆既"],		"1056.08.15"=>["月蝕皆既"],		"1057.02.16"=>["月蝕四分"],
    "1057.08.14"=>["月蝕五分"],			"1058.01.16"=>["(月蝕ニ分)"],		"1058.07.15"=>["(月蝕ニ分)"],
    "1058.12=15"=>["月蝕九分"],			"1059.06.15"=>["月蝕皆既"],		"1059.12.15"=>["月蝕皆既"],
    "1060.06.16"=>["昼月蝕皆既"],		"1060.12.14"=>["月蝕四分"],		"1061.06.16"=>["(月蝕ニ分)"],
    "1061.10.15"=>["(月蝕一分)"],		"1062.04.15"=>["昼月蝕六分"],		"1062.10.16"=>["月蝕八分"],
    "1063.04.15"=>["昼月蝕皆既"],		"1063.10.15"=>["月帯蝕皆既"],		"1064.04.14"=>["月蝕六分"],
    "1064.09.16"=>["昼月蝕五分"],		"1065.02.16"=>["(昼月蝕ニ分)"],		"1065.08.15"=>["(昼月蝕一分)"],
    "1066.02.16"=>["月蝕九分"],			"1066.08.14"=>["月蝕七分"],		"1067.01=16"=>["月帯蝕皆既"],
    "1067.07.15"=>["昼月蝕皆既"],		"1068.01.16"=>["昼月蝕四分"],		"1068.07.16"=>["月帯蝕四分"],
    "1068.12.15"=>["(昼月蝕ニ分)"],		"1069.06.16"=>["(昼月蝕四分)"],		"1069.11.15"=>["月蝕八分"],
    "1070.01.14"=>["月蝕皆既"],			"1070.07.16"=>["月帯蝕皆既"],		"1071.05.15"=>["月帯蝕八分"],
    "1071.11.15"=>["月帯蝕五分"],		"1072.05.14"=>["(月蝕一分)"],		"1072.09.16"=>["(月蝕一分)"],
    "1073.03.15"=>["月蝕七分"],			"1073.09.15"=>["月帯蝕七分"],		"1074.03.16"=>["昼月蝕皆既"],
    "1074.09.14"=>["月帯蝕皆既"],		"1075.03.16"=>["昼月蝕四分"],		"1075.08.15"=>["昼月蝕六分"],
    "1076.01.15"=>["(月蝕ニ分)"],		"1076.07.15"=>["(昼月蝕ニ分)"],		"1077.01.15"=>["月蝕九分"],
    "1077.07.16"=>["月帯蝕皆既"],		"1077.12=14"=>["月帯蝕皆既"],		"1078.06.16"=>["月帯蝕皆既"],
    "1078.12.15"=>["昼月蝕四分"],		"1079.06.15"=>["(月蝕三分)"],		"1079.11.16"=>["(昼月蝕一分)"],
    "1080.05.15"=>["昼月蝕五分"],		"1080.10.16"=>["月蝕八分"],		"1081.04.14"=>["月帯蝕皆既"],
    "1081.10.16"=>["昼月蝕皆既"],		"1082.04.15"=>["昼月蝕七分"],		"1082.10.16"=>["月帯蝕五分"],
    "1083.03.16"=>["(昼月蝕一分)"],		"1083.08.14"=>["(月蝕一分にミたす)"],	"1084.02.17"=>["月帯蝕八分"],
    "1084.08.15"=>["昼月蝕七分"],		"1085.02.16"=>["昼月蝕皆既"],		"1085.08.15"=>["月帯蝕皆既"],
    "1086.02.15"=>["昼月蝕四分"],		"1086.07.16"=>["昼月蝕五分"],		"1086.12.14"=>["(月蝕ニ分)"],
    "1087.06.16"=>["(昼月蝕三分)"],		"1087.12.15"=>["昼月蝕八分"],		"1088.06.15"=>["月蝕皆既"],
    "1088.11.15"=>["月蝕皆既"],			"1089.05.15"=>["月蝕皆既"],		"1089.11.16"=>["昼月蝕五分"],
    "1090.05.15"=>["(月帯蝕ニ分)"],		"1090.10.16"=>["(昼月蝕一分)"],		"1091.04.15"=>["月帯蝕六分"],
    "1091.09.16"=>["昼月蝕七分"],		"1092.03.15"=>["月蝕皆既"],		"1092.09.15"=>["昼月蝕皆既"],
    "1093.03.16"=>["昼月蝕五分"],		"1093.09.15"=>["月帯蝕六分"],		"1094.02.16"=>["(昼月蝕ニ分)"],
    "1094.07.15"=>["(月蝕一分)"],		"1095.01.16"=>["昼月蝕八分"],		"1095.07.16"=>["昼月蝕八分"],
    "1096.01.15"=>["昼月蝕皆既"],		"1096.07.16"=>["月蝕皆既"],		"1097.01.15"=>["月蝕五分"],
    "1097.06.16"=>["(昼月蝕三分)"],		"1097.11.15"=>["(月蝕一分)"],		"1098.05.15"=>["(月蝕四分)"],
    "1098.11.16"=>["昼月蝕八分"],		"1099.05.14"=>["月帯蝕皆既"],		"1099.10.16"=>["月蝕皆既"],
    "1100.04.15"=>["昼月蝕七分"],		"1100.10.15"=>["月蝕五分"],		"1101.03.15"=>["(月蝕一分)"],
    "1101.09.15"=>["(昼月蝕一分にミたす)"],	"1102.03.16"=>["昼月蝕八分"],		"1102.08.15"=>["昼月蝕七分"],
    "1103.02.16"=>["月蝕皆既"],			"1103.08.14"=>["月帯蝕皆既"],		"1104.02.15"=>["月蝕五分"],
    "1104.08.15"=>["月蝕五分"],			"1105.01.15"=>["(昼月蝕一分)"],		"1105.06.16"=>["(月蝕三分)"],
    "1105.12.15"=>["月帯蝕八分"],		"1106.06.16"=>["月帯蝕九分"],		"1106.12.16"=>["月帯蝕皆既"],
    "1107.06.15"=>["昼月蝕皆既"],		"1107.11.16"=>["月蝕四分"],		"1108.05.15"=>["(昼月蝕三分)"],
    "1108.10.16"=>["(月蝕一分)"],		"1109.04.15"=>["昼月蝕六分"],		"1109.10.15"=>["月蝕七分"],
    "1110.04.16"=>["月帯蝕皆既"],		"1110.09.15"=>["月蝕皆既"],		"1111.03.16"=>["月蝕五分"],
    "1111.09.14"=>["月蝕六分"],			"1112.08.15"=>["(月蝕一分)"],		"1113.02.15"=>["月帯蝕八分"],
    "1113.07.16"=>["月蝕八分"],			"1114.01.14"=>["月蝕皆既"],		"1114.07.16"=>["昼月蝕皆既"],
    "1115.01.14"=>["月蝕五分"],			"1115.07.16"=>["昼月蝕四分"],		"1115.12.16"=>["(昼月蝕一分)"],
    "1116.05.14"=>["(月蝕三分)"],		"1116.11.16"=>["月蝕八分"],		"1117.05.15"=>["昼月蝕皆既"],
    "1117.11.16"=>["月帯蝕皆既"],		"1118.05.15"=>["月蝕皆既"],		"1118.10.16"=>["昼月蝕五分"],
    "1119.03.16"=>["(昼月蝕一分にミたす)"],	"1119.04.16"=>["(昼月蝕一分)"],		"1120.03.16"=>["月蝕七分"],
    "1120.09.14"=>["月蝕六分"],			"1121.03.15"=>["月帯蝕皆既"],		"1121.08.15"=>["昼月蝕皆既"],
    "1122.02.16"=>["月帯蝕五分"],		"1122.08.16"=>["月帯蝕五分"],		"1123.01.15"=>["(昼月蝕一分)"],
    "1123.07.16"=>["(昼月蝕ニ分)"],		"1124.01.14"=>["月蝕八分"],		"1124.06.16"=>["昼月蝕九分"],
    "1124.12.15"=>["月帯蝕皆既"],		"1125.06.15"=>["昼月蝕皆既"],		"1125.12.16"=>["昼月蝕四分"],
    "1126.06.14"=>["月帯蝕三分"],		"1126.10=16"=>["(月蝕一分)"],		"1127.04.15"=>["月蝕五分"],
    "1127.10.15"=>["月蝕七分"],			"1128.04.16"=>["昼月蝕皆既"],		"1128.10.14"=>["月帯蝕皆既"],
    "1129.04.16"=>["月帯蝕六分"],		"1129.09.15"=>["昼月蝕六分"],		"1130.08.15"=>["(昼月蝕一分にミたす)"],
    "1131.02.15"=>["月蝕八分"],			"1131.08.16"=>["月帯蝕七分"],		"1132.02.14"=>["月帯蝕皆既"],
    "1132.07.16"=>["月帯蝕皆既"],		"1133.01.15"=>["昼月蝕五分"],		"1133.07.15"=>["月蝕五分"],
    "1133.12.16"=>["(月帯蝕一分)"],		"1134.06.15"=>["(昼月蝕三分)"],		"1134.12.16"=>["月帯蝕八分"],
    "1135.05.15"=>["昼月蝕皆既"],		"1135.11.16"=>["月帯蝕皆既"],		"1136.05.15"=>["月帯蝕皆既"],
    "1136.11.15"=>["月蝕五分"],			"1137.05.15"=>["(月蝕一分)"],		"1138.03.16"=>["月帯蝕七分"],
    "1138.09.15"=>["月帯蝕六分"],		"1139.03.16"=>["昼月蝕皆既"],		"1139.09.15"=>["月蝕皆既"],
    "1140.03.15"=>["昼月蝕五分"],		"1140.08.16"=>["昼月蝕六分"],		"1141.01.14"=>["(月蝕一分)"],
    "1141.07.16"=>["(昼月蝕一分)"],		"1142.01.15"=>["昼月蝕八分"],		"1142.07.15"=>["月帯蝕八分"],
    "1143.01.15"=>["月蝕皆既"],			"1143.06.15"=>["月帯蝕皆既"],		"1143.12.16"=>["月帯蝕五分"],
    "1144.06.14"=>["月蝕四分"],			"1144.11.16"=>["(昼月蝕一分)"],		"1145.05.15"=>["月帯蝕四分"],
    "1145.10=16"=>["昼月蝕七分"],		"1146.04.15"=>["月蝕皆既"],		"1146.10.15"=>["昼月蝕皆既"],
    "1147.04.16"=>["昼月蝕七分"],		"1147.10.15"=>["月蝕六分"],		"1149.02.15"=>["昼月蝕七分"],
    "1149.08.16"=>["昼月蝕七分"],		"1150.02.15"=>["昼月蝕皆既"],		"1150.08.16"=>["月蝕皆既"],
    "1151.02.15"=>["月蝕五分"],			"1151.07.16"=>["昼月蝕五分"],		"1151.12.15"=>["(月蝕一分)"],
    "1152.06.15"=>["(昼月蝕ニ分)"],		"1152.12.16"=>["月帯蝕八分"],		"1153.06.14"=>["月蝕九分"],
    "1153.12.16"=>["月蝕皆既"],			"1154.05.15"=>["昼月蝕皆既"],		"1154.11.15"=>["月蝕五分"],
    "1155.05.16"=>["月帯蝕ニ分"],		"1156.04.16"=>["昼月蝕六分"],		"1156.09=15"=>["月帯蝕六分"],
    "1157.03.16"=>["月帯蝕皆既"],		"1157.09.15"=>["月帯蝕皆既"],		"1158.03.15"=>["月蝕六分"],
    "1158.09.16"=>["月蝕六分"],			"1159.02.15"=>["(昼月蝕一分)"],		"1159.07.16"=>["(月蝕一分)"],
    "1160.01.15"=>["月帯蝕七分"],		"1160.07.15"=>["月蝕七分"],		"1161.01.16"=>["昼月蝕皆既"],
    "1161.07.14"=>["月帯蝕皆既"],		"1162.01.16"=>["月蝕五分"],		"1162.06.15"=>["昼月蝕五分"],
    "1162.11.16"=>["(月蝕一分)"],		"1163.05.15"=>["昼月蝕三分"],		"1163.11.15"=>["月蝕七分"],
    "1164.05.16"=>["月帯蝕皆既"],		"1164.10=15"=>["月蝕皆既"],		"1165.04.16"=>["月帯蝕皆既"],
    "1165.10.14"=>["月蝕六分"],			"1166.04.16"=>["(月帯蝕一分にミたす)"],	"1167.03.15"=>["昼月蝕七分"],
    "1167.08.16"=>["月蝕七分"],			"1168.02.14"=>["月帯蝕皆既"],		"1168.08.16"=>["昼月蝕皆既"],
    "1169.02.14"=>["月蝕六分"],			"1169.08.16"=>["昼月蝕六分"],		"1170.01.16"=>["(昼月蝕一分)"],
    "1170.06.14"=>["(月蝕一分)"],		"1170.12.16"=>["月蝕八分"],		"1171.06.15"=>["月帯蝕八分"],
    "1171.12.16"=>["月帯蝕皆既"],		"1172.06.15"=>["月帯蝕皆既"],		"1172.12.16"=>["昼月蝕五分"],
    "1173.05.16"=>["昼月蝕三分"],		"1174.04.16"=>["月帯蝕五分"],		"1174.10.14"=>["月蝕六分"],
    "1175.04.15"=>["月蝕皆既"],			"1175.09=15"=>["昼月蝕皆既"],		"1176.03.15"=>["月蝕七分"],
    "1176.09.16"=>["月帯蝕六分"],		"1177.02.15"=>["(昼月蝕一分にミたす)"],	"1177.08.16"=>["(昼月蝕一分にミたす)"],
    "1178.02.14"=>["月蝕七分"],			"1178.07.16"=>["昼月蝕七分"],		"1179.01.15"=>["月帯蝕皆既"],
    "1179.07.15"=>["昼月蝕皆既"],		"1180.01.16"=>["昼月蝕五分"],		"1180.07.15"=>["昼月蝕五分"],
    "1180.12.15"=>["(月蝕一分)"],		"1181.05.15"=>["(月帯蝕三分)"],		"1181.11.15"=>["月蝕七分"],
    "1182.05.16"=>["昼月蝕皆既"],		"1182.11.14"=>["月帯蝕皆既"],		"1183.05.16"=>["月帯蝕皆既"],
    "1183.10=15"=>["昼月蝕六分"],		"1184.04.16"=>["(昼月蝕一分)"],		"1185.03.15"=>["月蝕六分"],
    "1185.09.15"=>["月帯蝕六分"],		"1186.03.14"=>["月蝕皆既"],		"1186.08.16"=>["月帯蝕皆既"],
    "1187.02.15"=>["昼月蝕六分"],		"1187.08.16"=>["月蝕六分"],		"1188.07.15"=>["(昼月蝕一分)"],
    "1189.01.16"=>["昼月蝕八分"],		"1189.06.15"=>["昼月蝕七分"],		"1189.12.16"=>["月帯蝕皆既"],
    "1190.06.14"=>["月蝕皆既"],			"1190.12.15"=>["月蝕五分"],		"1191.06.15"=>["月帯蝕四分"],
    "1192.04.16"=>["月蝕五分"],			"1192.10.15"=>["昼月蝕六分"],		"1193.04.16"=>["昼月蝕皆既"],
    "1193.10.15"=>["月蝕皆既"],			"1194.04.15"=>["昼月蝕七分"],		"1194.09.16"=>["昼月蝕六分"],
    "1195.03.15"=>["(昼月蝕一分にミたす)"],	"1196.02.15"=>["昼月蝕七分"],		"1196.08.15"=>["月帯蝕六分"],
    "1197.02.15"=>["月蝕皆既"],			"1197.07.15"=>["月帯蝕皆既"],		"1198.01.16"=>["月蝕五分"],
    "1198.07.14"=>["月蝕六分"],			"1198.12.16"=>["(昼月蝕一分)"],		"1199.06.15"=>["(月蝕ニ分)"],
    "1199.12.15"=>["昼月蝕七分"],		"1200.05.16"=>["月帯蝕皆既"],		"1200.11.15"=>["月帯蝕皆既"],
    "1201.05.16"=>["昼月蝕皆既"],		"1201.11.14"=>["月蝕六分"],		"1202.05.16"=>["(昼月蝕ニ分)"],
    "1203.03.15"=>["月帯蝕六分"],		"1203.09.16"=>["昼月蝕六分"],		"1204.03.15"=>["昼月蝕皆既"],
    "1204.09.16"=>["月蝕皆既"],			"1205.03.15"=>["月蝕六分"],		"1205.08.15"=>["月帯蝕六分"],
    "1206.07.15"=>["(昼月蝕一分にミたす)"],	"1206.08.15"=>["(月蝕一分にミたす)"],	"1207.01.16"=>["月帯蝕八分"],
    "1207.07.14"=>["月蝕七分"],			"1208.01.16"=>["月蝕皆既"],		"1208.06.15"=>["昼月蝕皆既"],
    "1208.12.15"=>["月蝕五分"],			"1209.06.15"=>["月蝕四分"],		"1210.05.16"=>["(昼月蝕四分)"],
    "1210.11.15"=>["月帯蝕六分"],		"1211.04.16"=>["昼月蝕皆既"],		"1211.10.16"=>["月帯蝕皆既"],
    "1212.04.15"=>["昼月蝕八分"],		"1212.10.16"=>["月蝕七分"],		"1213.04.14"=>["(月蝕一分)"],
    "1214.02.15"=>["月帯蝕六分"],		"1214.08.15"=>["月蝕六分"],		"1215.02.16"=>["昼月蝕皆既"],
    "1215.08.14"=>["月蝕皆既"],			"1216.02.16"=>["月蝕五分"],		"1216.07.15"=>["月帯蝕六分"],
    "1216.12.16"=>["(月蝕一分)"],		"1217.06.15"=>["(昼月蝕一分)"],		"1217.12.15"=>["月蝕七分"],
    "1218.06.15"=>["月帯蝕九分"],		"1218.12.14"=>["月蝕皆既"],		"1219.05.16"=>["月帯蝕皆既"],
    "1219.11.14"=>["月蝕六分"],			"1220.05.15"=>["(月蝕三分)"],		"1221.04.15"=>["(昼月蝕五分)"],
    "1221.10.16"=>["月蝕六分"],			"1222.03.15"=>["月帯蝕皆既"],		"1222.09.16"=>["月帯蝕皆既"],
    "1223.03.14"=>["月帯蝕七分"],		"1223.09.16"=>["昼月蝕七分"],		"1224.08.15"=>["(昼月蝕一分にミたす)"],
    "1225.01.16"=>["月蝕七分"],			"1225.07.14"=>["月帯蝕六分"],		"1226.01.16"=>["昼月蝕皆既"],
    "1226.07.15"=>["月帯蝕皆既"],		"1227.01.16"=>["昼月蝕六分"],		"1227.06.16"=>["昼月蝕五分"],
    "1228.05.16"=>["(昼月蝕三分)"],		"1228.11.14"=>["月蝕六分"],		"1229.05.15"=>["月蝕皆既"],
    "1229.11.15"=>["月帯蝕皆既"],		"1230.04.15"=>["月蝕皆既"],		"1230.10.16"=>["昼月蝕七分"],
    "1231.04.14"=>["(月蝕ニ分)"],		"1231.10.16"=>["(月蝕一分にミたす)"],	"1232.03.14"=>["月蝕六分"],
    "1232.09.16"=>["昼月蝕六分"],		"1233.02.15"=>["月帯蝕皆既"],		"1233.08.15"=>["月帯蝕皆既"],
    "1234.02.16"=>["昼月蝕六分"],		"1234.08.15"=>["昼月蝕七分"],		"1235.01.15"=>["(月蝕一分)"],
    "1235.06=15"=>["(昼月蝕一分)"],		"1235.07.14"=>["(月蝕一分にミたす)"],	"1235.12.15"=>["月帯蝕七分"],
    "1236.06.16"=>["昼月蝕八分"],		"1236.12.15"=>["月帯蝕皆既"],		"1237.06.16"=>["月蝕皆既"],
    "1237.12.15"=>["昼月蝕六分"],		"1238.05.16"=>["昼月蝕四分"],		"1239.04.15"=>["(月帯蝕四分)"],
    "1239.10.16"=>["月帯蝕六分"],		"1240.04.14"=>["月蝕皆既"],		"1240.10.16"=>["月帯蝕皆既"],
    "1241.03.15"=>["昼月蝕皆既"],		"1241.09.15"=>["月蝕七分"],		"1242.03.15"=>["(月蝕一分にミたす)"],
    "1242.09.15"=>["(月蝕一分)"],		"1243.02.16"=>["昼月蝕七分"],		"1243.07=15"=>["昼月蝕六分"],
    "1244.01.16"=>["月帯蝕皆既"],		"1244.07.15"=>["月蝕皆既"],		"1245.01.15"=>["月帯蝕六分"],
    "1245.07.16"=>["月帯蝕六分"],		"1246.05.16"=>["(月蝕ニ分)"],		"1246.11.15"=>["昼月蝕六分"],
    "1247.05.16"=>["月帯蝕九分"],		"1247.11.15"=>["月蝕皆既"],		"1248.05.15"=>["月帯蝕皆既"],
    "1248.11.16"=>["月帯蝕七分"],		"1249.04.15"=>["(昼月蝕三分)"],		"1249.10.16"=>["(月蝕一分にミたす)"],
    "1250.03.15"=>["昼月蝕五分"],		"1250.09.15"=>["昼月蝕五分"],		"1251.03.15"=>["月帯蝕皆既"],
    "1251.09.15"=>["月帯蝕皆既"],		"1252.02.16"=>["月帯蝕六分"],		"1252.08.14"=>["月蝕七分"],
    "1253.01.16"=>["(昼月蝕一分にミたす)"],	"1253.08.15"=>["(昼月蝕一分にミたす)"],	"1254.01.15"=>["昼月蝕七分"],
    "1254.06.16"=>["月帯蝕七分"],		"1254.12.15"=>["月帯蝕皆既"],		"1255.06.16"=>["月帯蝕皆既"],
    "1255.12.14"=>["月蝕六分"],			"1256.06.16"=>["昼月蝕四分"],		"1257.04.14"=>["(月蝕三分)"],
    "1257.10.16"=>["月帯蝕六分"],		"1258.04.15"=>["昼月蝕皆既"],		"1258.10.16"=>["月蝕皆既"],
    "1259.04.15"=>["月帯蝕皆既"],		"1259.10.15"=>["月蝕七分"],		"1260.03.16"=>["(昼月蝕一分)"],
    "1260.09.14"=>["(月蝕一分)"],		"1261.02.16"=>["月帯蝕七分"],		"1261.08.14"=>["月帯蝕五分"],
    "1262.02.16"=>["月蝕皆既"],			"1262.07=15"=>["昼月蝕皆既"],		"1263.01.15"=>["月蝕六分"],
    "1263.07.15"=>["月蝕六分"],			"1264.06.16"=>["(昼月蝕ニ分)"],		"1264.12.15"=>["月蝕六分"],
    "1265.05.16"=>["昼月蝕九分"],		"1265.11.16"=>["昼月蝕皆既"],		"1266.05.15"=>["昼月蝕皆既"],
    "1266.11.16"=>["月蝕七分"],			"1267.05.15"=>["昼月蝕三分"],		"1267.11.16"=>["(昼月蝕一分にミたす)"],
    "1268.03.15"=>["月帯蝕五分"],		"1268.09.15"=>["月蝕五分"],		"1269.03.16"=>["昼月蝕皆既"],
    "1269.09.14"=>["月蝕皆既"],			"1270.03.16"=>["月蝕七分"],		"1270.09.14"=>["月帯蝕八分"],
    "1271.01.16"=>["(月蝕一分にミたす)"],	"1271.08.15"=>["(昼月蝕一分)"],		"1272.01.15"=>["月蝕六分"],
    "1272.07.15"=>["月蝕七分"],			"1273.01.14"=>["月蝕皆既"],		"1273.06.16"=>["昼月蝕皆既"],
    "1273.12.15"=>["月帯蝕六分"],		"1274.06.16"=>["月蝕五分"],		"1275.05.15"=>["(昼月蝕三分)"],
    "1275.11.16"=>["月蝕六分"],			"1276.04.15"=>["昼月蝕皆既"],		"1276.10.16"=>["月帯蝕皆既"],
    "1277.04.14"=>["月蝕皆既"],			"1277.10.16"=>["昼月蝕七分"],		"1278.04.15"=>["(月蝕ニ分)"],
    "1278.10.15"=>["(昼月蝕一分)"],		"1279.02.16"=>["月蝕六分"],		"1279.08.14"=>["月蝕五分"],
    "1280.02.16"=>["月帯蝕皆既"],		"1280.08.15"=>["月帯蝕皆既"],		"1281.02.16"=>["昼月蝕六分"],
    "1281.07=16"=>["昼月蝕七分"],		"1282.06.16"=>["(昼月蝕一分)"],		"1282.12.14"=>["月蝕六分"],
    "1283.06.16"=>["月帯蝕八分"],		"1283.12.15"=>["月蝕皆既"],		"1284.05.15"=>["月帯蝕皆既"],
    "1284.11.16"=>["昼月蝕六分"],		"1285.05.14"=>["月蝕四分"],		"1285.11.16"=>["(月蝕一分にミたす)"],
    "1286.04.15"=>["月蝕四分"],			"1286.10.16"=>["月帯蝕五分"],		"1287.03.16"=>["月帯蝕皆既"],
    "1287.09.15"=>["月帯蝕皆既"],		"1288.03.16"=>["昼月蝕七分"],		"1288.09.15"=>["昼月蝕皆既"],
    "1289.03.16"=>["(月蝕一分にミたす)"],	"1289.09.15"=>["(月蝕一分)"],		"1290.01.15"=>["月蝕六分"],
    "1290.07.16"=>["昼月蝕六分"],		"1291.01.15"=>["月帯蝕皆既"],		"1291.07.16"=>["月蝕皆既"],
    "1292.01.15"=>["月帯蝕七分"],		"1292.06=15"=>["月帯蝕六分"],		"1293.05.15"=>["(昼月蝕ニ分)"],
    "1293.11.16"=>["昼月蝕六分"],		"1294.05.14"=>["月蝕九分"],		"1294.11.16"=>["月帯蝕皆既"],
    "1295.04.15"=>["昼月蝕皆既"],		"1295.10.15"=>["月蝕七分"],		"1296.04.15"=>["月蝕ニ分"],
    "1296.10.15"=>["(月蝕一分)"],		"1297.03.16"=>["昼月蝕六分"],		"1297.09.15"=>["昼月蝕五分"],
    "1298.02.16"=>["昼月蝕皆既"],		"1298.08.15"=>["月蝕皆既"],		"1299.02.15"=>["月帯蝕七分"],
    "1299.08.16"=>["月帯蝕七分"],		"1300.02.15"=>["(月蝕一分にミたす)"],	"1300.07.16"=>["(月蝕一分にミたす)"],
    "1300.07=16"=>["(昼月蝕一分にミたす)"],	"1300.12.15"=>["昼月蝕六分"],		"1301.06.15"=>["月蝕七分"],
    "1301.12.15"=>["月帯蝕皆既"],		"1302.06.14"=>["月蝕皆既"],		"1302.12.16"=>["月蝕六分"],
    "1303.05.15"=>["昼月蝕五分"],		"1304.04.15"=>["(昼月蝕三分)"],		"1304.10.15"=>["昼月蝕五分"],
    "1305.04.15"=>["月帯蝕皆既"],		"1305.10.15"=>["月帯蝕皆既"],		"1306.03.16"=>["月帯蝕皆既"],
    "1306.09.14"=>["月蝕皆既"],			"1307.03.16"=>["(月蝕一分)"],		"1307.09.15"=>["(昼月蝕一分)"],
    "1308.02.15"=>["昼月蝕六分"],		"1308.08.16"=>["月帯蝕六分"],		"1309.01.15"=>["月帯蝕皆既"],
    "1309.07.17"=>["月帯蝕皆既"],		"1310.01.14"=>["月蝕七分"],		"1310.07.16"=>["昼月蝕六分"],
    "1311.06.15"=>["(月蝕一分)"],		"1311.11.16"=>["月蝕六分"],		"1312.05.14"=>["月帯蝕八分"],
    "1312.11.16"=>["月蝕皆既"],			"1313.05.15"=>["昼月蝕皆既"],		"1313.11.15"=>["月蝕七分"],
    "1314.04.16"=>["昼月蝕三分"],		"1314.10.14"=>["(月蝕一分)"],		"1315.03.16"=>["昼月蝕五分"],
    "1315.09.15"=>["月蝕四分"],			"1316.03.16"=>["月蝕皆既"],		"1316.09.15"=>["昼月蝕皆既"],
    "1317.02.15"=>["月蝕七分"],			"1317.08.15"=>["月蝕皆既"],		"1318.02.14"=>["(月蝕一分)"],
    "1318.08.16"=>["(昼月蝕一分)"],		"1319.01.15"=>["月蝕六分"],		"1319.07.16"=>["月帯蝕六分"],
    "1319.12.16"=>["昼月蝕皆既"],		"1320.06.15"=>["昼月蝕皆既"],		"1320.12.15"=>["月帯蝕六分"],
    "1321.06.15"=>["昼月蝕六分"],		"1322.05.15"=>["(昼月蝕三分)"],		"1322.10.15"=>["(月蝕五分)"],
    "1323.04.16"=>["昼月蝕皆既"],		"1323.10.14"=>["月蝕皆既"],		"1324.04.16"=>["月蝕皆既"],
    "1324.10.14"=>["月帯蝕皆既"],		"1325.03.16"=>["(昼月蝕一分)"],		"1325.09.15"=>["(月蝕ニ分)"],
    "1326.02.15"=>["月帯蝕五分"],		"1326.08.16"=>["月蝕五分"],		"1327.02.14"=>["月蝕皆既"],
    "1327.08.16"=>["昼月蝕皆既"],		"1328.01.15"=>["昼月蝕七分"],		"1328.07.16"=>["月帯蝕七分"],
    "1329.01.15"=>["(月蝕一分にミたす)"],	"1329.06.14"=>["(月蝕一分にミたす)"],	"1329.07.15"=>["(昼月蝕一分にミたす)"],
    "1329.12.16"=>["月蝕六分"],			"1330.06.15"=>["昼月蝕七分"],		"1330.11.16"=>["昼月蝕皆既"],
    "1331.05.15"=>["月蝕皆既"],			"1331.11.16"=>["昼月蝕七分"],		"1332.05.16"=>["昼月蝕四分"],
    "1332.11.15"=>["(昼月蝕一分)"],		"1333.03.16"=>["月蝕五分"],		"1333.09.14"=>["月蝕四分"],
    "1334.03.16"=>["月帯蝕皆既"],		"1334.09.15"=>["月帯蝕皆既"],		"1335.03.15"=>["月帯蝕八分"],
    "1335.09.16"=>["昼月蝕皆既"],		"1336.02.15"=>["(昼月蝕一分)"],		"1336.08.16"=>["(月蝕一分)"],
    "1337.01.15"=>["月帯蝕六分"],		"1337.07.16"=>["昼月蝕六分"],		"1338.01.15"=>["月蝕皆既"],
    "1338.07.15"=>["昼月蝕皆既"],		"1338.12.16"=>["昼月蝕六分"],		"1339.06.14"=>["月帯蝕六分"],
    "1339.12.16"=>["(月蝕一分にミたす)"],	"1340.05.15"=>["(月蝕ニ分)"],		"1340.11.16"=>["(月帯蝕五分)"],
    "1341.04=16"=>["月帯蝕皆既"],		"1341.10.15"=>["月帯蝕皆既"],		"1342.04.16"=>["昼月蝕皆既"],
    "1342.10.15"=>["昼月蝕皆既"],		"1343.04.16"=>["(昼月蝕ニ分)"],		"1343.10.15"=>["(月蝕ニ分)"],
    "1344.02=14"=>["(月蝕五分)"],		"1344.08.16"=>["昼月蝕五分"],		"1345.02.15"=>["月帯蝕皆既"],
    "1345.08.16"=>["月帯蝕皆既"],		"1346.02.15"=>["月帯蝕七分"],		"1346.08.15"=>["月蝕七分"],
    "1347.01.16"=>["(昼月蝕一分にミたす)"],	"1347.07.15"=>["(月蝕一分)"],		"1347.12.16"=>["昼月蝕六分"],
    "1348.06.14"=>["月帯蝕七分"],		"1348.12.16"=>["月帯蝕皆既"],		"1349.06.15"=>["昼月蝕皆既"],
    "1349.11.15"=>["月蝕七分"],			"1350.05.15"=>["月蝕四分"],		"1350.11.15"=>["(月蝕一分)"],
    "1351.04.16"=>["(昼月蝕四分)"],		"1351.10.15"=>["昼月蝕四分"],		"1352.03.16"=>["昼月蝕皆既"],
    "1352.09.15"=>["月蝕皆既"],			"1353.03.15"=>["昼月蝕皆既"],		"1353.09.16"=>["月帯蝕皆既"],
    "1354.03.15"=>["(昼月蝕ニ分)"],		"1354.09.17"=>["(昼月蝕一分)"],		"1355.01.15"=>["昼月蝕五分"],
    "1355.07.15"=>["月蝕五分"],			"1356.01.16"=>["月帯蝕皆既"],		"1356.07.15"=>["月蝕皆既"],
    "1357.01.16"=>["月蝕七分"],			"1357.07.14"=>["月蝕七分"],		"1357.12.16"=>["(昼月蝕一分にミたす)"],
    "1358.05.15"=>["(昼月蝕一分)"],		"1358.11.15"=>["(昼月蝕五分)"],		"1359.05.15"=>["月蝕九分"],
    "1359.11.15"=>["月帯蝕皆既"],		"1360.04=16"=>["昼月蝕皆既"],		"1360.10.14"=>["月蝕皆既"],
    "1361.04.16"=>["(月蝕三分)"],		"1361.10.15"=>["(昼月蝕ニ分)"],		"1362.03.15"=>["(昼月蝕四分)"],
    "1362.09.16"=>["月帯蝕五分"],		"1363.02.15"=>["昼月蝕皆既"],		"1363.08.16"=>["月帯蝕皆既"],
    "1364.02.14"=>["月蝕皆既"],			"1364.08.16"=>["昼月蝕八分"],		"1365.02.15"=>["(昼月蝕一分)"],
    "1365.08.15"=>["(昼月蝕一分)"],		"1365.12.16"=>["月蝕六分"],		"1366.06.14"=>["月蝕六分"],
    "1366.12.16"=>["月蝕皆既"],			"1367.06.15"=>["昼月蝕皆既"],		"1367.12.15"=>["月蝕七分"],
    "1368.06.16"=>["昼月蝕五分"],		"1368.11.14"=>["(月蝕一分)"],		"1369.04.16"=>["(昼月蝕三分)"],
    "1369.10.15"=>["月蝕四分"],			"1370.04.16"=>["月帯蝕皆既"],		"1370.10.16"=>["昼月蝕皆既"],
    "1371.03=15"=>["月帯蝕皆既"],		"1371.09.15"=>["月蝕皆既"],		"1372.03.14"=>["(月蝕ニ分)"],
    "1372.09.16"=>["(昼月蝕ニ分)"],		"1373.02.15"=>["月蝕五分"],		"1373.08.15"=>["(月蝕五分)"],
    "1374.01.16"=>["月帯蝕皆既"],		"1374.07.14"=>["月帯蝕皆既"],		"1375.01.16"=>["月帯蝕七分"],
    "1375.07.15"=>["昼月蝕八分"],		"1376.01.16"=>["(昼月蝕一分にミたす)"],	"1376.06.15"=>["(昼月蝕一分にミたす)"],
    "1376.07.15"=>["(月蝕一分)"],		"1376.11.15"=>["(月蝕五分)"],		"1377.05.16"=>["昼月蝕八分"],
    "1377.11.14"=>["月蝕皆既"],			"1378.05.16"=>["月蝕皆既"],		"1378.11.15"=>["月帯蝕皆既"],
    "1379.04=16"=>["昼月蝕四分"],		"1379.10.15"=>["(月蝕ニ分)"],		"1380.03.15"=>["(昼月蝕四分)"],
    "1380.09.16"=>["月蝕四分"],			"1381.03.14"=>["月蝕皆既"],		"1381.09.16"=>["昼月蝕皆既"],
    "1382.02.15"=>["昼月蝕皆既"],		"1382.08.16"=>["昼月蝕皆既"],		"1383.02.15"=>["(月蝕一分)"],
    "1383.08.15"=>["(昼月蝕ニ分)"],		"1384.01.16"=>["月蝕六分"],		"1384.07.15"=>["昼月蝕五分"],
    "1384.12.16"=>["昼月蝕皆既"],		"1385.06.15"=>["月蝕皆既"],		"1385.12.15"=>["昼月蝕七分"],
    "1386.06.16"=>["昼月蝕六分"],		"1386.12.15"=>["(昼月蝕一分)"],		"1387.05.16"=>["(月蝕ニ分)"],
    "1387.10.14"=>["月蝕四分"],			"1388.04.15"=>["月蝕九分"],		"1388.10.15"=>["月蝕皆既"],
    "1389.04.14"=>["月帯蝕皆既"],		"1389.10.16"=>["昼月蝕皆既"],		"1390.03=15"=>["昼月蝕三分"],
    "1390.09.16"=>["(月蝕ニ分)"],		"1391.02.15"=>["昼月蝕五分"],		"1391.08.16"=>["(昼月蝕四分)"],
    "1392.02.15"=>["月蝕皆既"],			"1392.08.15"=>["昼月蝕皆既"],		"1390.01.16"=>["昼月蝕七分"],
    "1390.07.15"=>["月帯蝕皆既"],		"1394.01.16"=>["(月蝕一分にミたす)"],	"1394.07.15"=>["(昼月蝕一分)"],
    "1394.12.16"=>["(月帯蝕五分)"],		"1395.06.16"=>["昼月蝕七分"],		"1395.11.15"=>["月帯蝕皆既"],
    "1396.05.16"=>["月帯蝕皆既"],		"1396.11.15"=>["月帯蝕皆既"],		"1397.05.16"=>["昼月蝕四分"],
    "1397.11.15"=>["(月蝕ニ分)"],		"1398.04.15"=>["(月蝕三分)"],		"1398.09.16"=>["昼月蝕四分"],
    "1399.03.15"=>["月帯蝕皆既"],		"1399.09.16"=>["月帯蝕皆既"],		"1400.03.15"=>["昼月蝕皆既"],
    "1400.09.15"=>["月蝕皆既"],			"1401.02.16"=>["(昼月蝕一分)"],		"1401.08.15"=>["(月蝕ニ分)"],
    "1402.01.16"=>["昼月蝕五分"],		"1402.07.15"=>["昼月蝕五分"],		"1403.01.16"=>["月帯蝕皆既"],
    "1403.07.15"=>["月帯蝕皆既"],		"1403.12.15"=>["月蝕七分"],		"1404.06.15"=>["月蝕七分"],
    "1404.12.15"=>["(月蝕一分)"],		"1405.05.16"=>["(月蝕ニ分)"],		"1405.11.15"=>["昼月蝕四分"],
    "1406.05.16"=>["昼月蝕九分"],		"1406.10.15"=>["月帯蝕皆既"],		"1407.04.15"=>["昼月蝕皆既"],
    "1407.10.16"=>["月蝕皆既"],			"1408.04.15"=>["昼月蝕三分"],		"1408.10.16"=>["(月帯蝕ニ分)"],
    "1409.03.15"=>["昼月蝕四分"],		"1409.08.15"=>["(月帯蝕四分)"],		"1410.02.16"=>["昼月蝕皆既"],
    "1410.08.15"=>["月帯蝕皆既"],		"1411.02.16"=>["月蝕皆既"],		"1411.08.14"=>["月蝕皆既"],
    "1412.01.16"=>["(昼月蝕一分)"],		"1412.07.15"=>["(昼月蝕ニ分)"],		"1412.12.15"=>["(昼月蝕五分)"],
    "1413.06.16"=>["月蝕六分"],			"1413.12.15"=>["月帯蝕皆既"],		"1414.06.16"=>["昼月蝕皆既"],
    "1414.11.14"=>["月蝕皆既"],			"1415.05.16"=>["月帯蝕五分"],		"1415.11.15"=>["(昼月蝕ニ分)"],
    "1416.04.15"=>["(昼月蝕三分)"],		"1416.10.16"=>["月蝕四分"],		"1417.04.15"=>["昼月蝕皆既"],
    "1417.09.16"=>["月蝕皆既"],			"1418.03.15"=>["月蝕皆既"],		"1418.09.16"=>["月帯蝕皆既"],
    "1419.03.15"=>["昼月蝕ニ分"],		"1419.09.15"=>["(月帯蝕三分)"],		"1420.01=16"=>["月蝕五分"],
    "1420.07.14"=>["月蝕四分"],			"1421.01.15"=>["月蝕皆既"],		"1421.07.15"=>["昼月蝕皆既"],
    "1422.01.15"=>["月蝕八分"],			"1422.07.16"=>["昼月蝕七分"],		"1422.12.14"=>["(月蝕一分)"],
    "1423.05.16"=>["(昼月蝕一分)"],		"1423.06.16"=>["(月蝕一分にミたす)"],	"1423.11.15"=>["月蝕四分"],
    "1424.05.16"=>["昼月蝕八分"],		"1424.11.16"=>["昼月蝕皆既"],		"1425.05.15"=>["昼月蝕皆既"],
    "1425.10.15"=>["月帯蝕皆既"],		"1426.04.14"=>["月蝕四分"],		"1426.10.16"=>["(昼月蝕ニ分)"],
    "1427.03.15"=>["月蝕四分"],			"1427.09.15"=>["(月蝕四分)"],		"1428.03.16"=>["月帯蝕皆既"],
    "1428.08.14"=>["月蝕皆既"],			"1429.02.16"=>["月帯蝕皆既"],		"1429.08.15"=>["昼月蝕皆既"],
    "1430.02.16"=>["(昼月蝕一分)"],		"1430.08.15"=>["月蝕ニ分"],		"1430.12.15"=>["(月蝕五分)"],
    "1431.06.16"=>["昼月蝕六分"],		"1431.12.14"=>["月蝕皆既"],		"1432.06.16"=>["月帯蝕皆既"],
    "1432.12.15"=>["昼月蝕皆既"],		"1433.06.15"=>["月蝕六分"],		"1433.11.15"=>["(月蝕ニ分)"],
    "1434.04.15"=>["(昼月蝕ニ分)"],		"1434.10.16"=>["(月蝕四分)"],		"1435.04.14"=>["月帯蝕九分"],
    "1435.10.16"=>["昼月蝕皆既"],		"1436.04.15"=>["月帯蝕皆既"],		"1436.09.16"=>["昼月蝕皆既"],
    "1437.03.15"=>["月蝕ニ分"],			"1437.09.15"=>["(昼月蝕三分)"],		"1438.02.16"=>["月蝕五分"],
    "1438.08.15"=>["(昼月蝕四分)"],		"1439.01=16"=>["昼月蝕皆既"],		"1439.07.15"=>["月蝕皆既"],
    "1440.01.15"=>["昼月蝕八分"],		"1440.07.16"=>["昼月蝕皆既"],		"1441.01.15"=>["(昼月蝕ニ分)"],
    "1441.06.16"=>["(昼月蝕一分にミたす)"],	"1441.07.16"=>["(月蝕一分)"],		"1441.11.16"=>["昼月蝕四分"],
    "1442.05.15"=>["月蝕七分"],			"1442.11.15"=>["月蝕皆既"],		"1443.05.15"=>["月蝕皆既"],
    "1443.11.16"=>["昼月蝕皆既"],		"1444.05.14"=>["月帯蝕五分"],		"1444.10.16"=>["(月蝕ニ分)"],
    "1445.03.15"=>["(昼月蝕三分)"],		"1445.09.16"=>["(昼月蝕三分)"],		"1446.03.15"=>["月蝕皆既"],
    "1446.09.15"=>["昼月蝕九分半余"],		"1447.02=16"=>["昼月蝕皆既"],		"1447.08.15"=>["月帯蝕皆既"],
    "1448.02.16"=>["(月蝕一分)"],		"1448.08.15"=>["月帯蝕三分"],		"1449.01.15"=>["(月帯蝕五分)"],
    "1449.07.16"=>["昼月蝕五分"],		"1449.12.16"=>["昼月蝕皆既"],		"1450.06.16"=>["月蝕皆既"],
    "1450.12.15"=>["月蝕皆既"],			"1451.06.16"=>["昼月蝕七分"],		"1451.12.16"=>["(昼月蝕ニ分)"],
    "1452.05.15"=>["(月帯蝕一分)"],		"1452.10.16"=>["(昼月蝕四分)"],		"1453.04.14"=>["月蝕八分"],
    "1453.10.16"=>["月蝕皆既"],			"1454.04.15"=>["昼月蝕皆既"],		"1454.10.15"=>["月蝕皆既"],
    "1455.04.16"=>["昼月蝕三分"],		"1455.09.15"=>["(月蝕三分)"],		"1456.02.16"=>["昼月蝕四分"],
    "1456.08.15"=>["(昼月蝕三分)"],		"1457.02.16"=>["月帯蝕皆既"],		"1457.08.16"=>["月帯蝕皆既"],
    "1458.01=15"=>["月帯蝕皆既"],		"1458.07.16"=>["月蝕皆既"],		"1459.01.14"=>["(月蝕ニ分)"],
    "1459.07.16"=>["(昼月蝕一分)"],		"1459.12.15"=>["月帯蝕四分"],		"1460.06.15"=>["月帯蝕六分"],
    "1460.11.16"=>["月帯蝕皆既"],		"1461.05.15"=>["月帯蝕皆既"],		"1461.11.16"=>["月蝕皆既"],
    "1462.05.15"=>["昼月蝕六分"],		"1462.11.16"=>["(昼月蝕ニ分)"],		"1463.04.15"=>["(昼月蝕三分)"],
    "1463.09.15"=>["(月帯蝕三分)"],		"1464.03.16"=>["昼月蝕皆既"],		"1464.09.15"=>["月帯蝕九分"],
    "1465.03.16"=>["月蝕皆既"],			"1465.09.14"=>["月蝕皆既"],		"1466.02=16"=>["(月帯蝕ニ分)"],
    "1466.08.15"=>["昼月蝕三分"],		"1467.01.15"=>["(昼月蝕四分)"],		"1467.07.16"=>["月蝕四分"],
    "1468.01.15"=>["月帯蝕皆既"],		"1468.07.16"=>["昼月蝕皆既"],		"1468.12.15"=>["月蝕皆既"],
    "1469.06.16"=>["昼月蝕七分"],		"1469.12.15"=>["月蝕ニ分"],		"1470.05.14"=>["(月蝕一分にミたす)"],
    "1470.06.15"=>["(昼月蝕一分)"],		"1470.11.16"=>["(月蝕四分)"],		"1471.05.15"=>["昼月蝕七分"],
    "1471.10.16"=>["月蝕皆既"],			"1472.04.15"=>["月蝕皆既"],		"1472.10.15"=>["月帯蝕皆既"],
    "1473.04.16"=>["昼月蝕四分"],		"1473.10.14"=>["月帯蝕三分"],		"1474.03.16"=>["(月蝕四分)"],
    "1474.08.14"=>["(月蝕三分)"],		"1475.02.15"=>["月蝕皆既"],		"1475.08.15"=>["昼月蝕皆既"],
    "1476.02.15"=>["月蝕皆既"],			"1476.08.16"=>["月帯蝕皆既"],		"1477.01=14"=>["(月蝕ニ分)"],
    "1477.07.16"=>["(月帯蝕ニ分)"],		"1477.12.15"=>["月蝕四分"],		"1478.06.16"=>["昼月蝕五分"],
    "1478.12.16"=>["月帯蝕皆既"],		"1479.06.15"=>["昼月蝕皆既"],		"1479.11.17"=>["月帯蝕皆既"],
    "1480.05.14"=>["月帯蝕六分"],		"1480.11.16"=>["(月帯蝕ニ分)"],		"1481.04.15"=>["(月蝕ニ分)"],
    "1481.10.15"=>["(月蝕三分)"],		"1482.04.16"=>["昼月蝕皆既"],		"1482.09.14"=>["月蝕九分"],
    "1483.03.17"=>["月帯蝕皆既"],		"1483.09.15"=>["月帯蝕皆既"],		"1484.03.16"=>["(昼月蝕ニ分)"],
    "1484.09.15"=>["月蝕三分"],			"1485.02.15"=>["(月蝕四分)"],		"1485.07.16"=>["月帯蝕四分"],
    "1486.01.14"=>["月蝕皆既"],			"1486.07.16"=>["月帯蝕皆既"],		"1487.01.15"=>["昼月蝕皆既"],
    "1487.07.15"=>["月蝕八分"],			"1487.12.15"=>["月蝕ニ分"],		"1488.06.15"=>["(月蝕一分)"],
    "1488.11.16"=>["(月蝕四分)"],		"1489.05.15"=>["昼月蝕七分"],		"1489.11.16"=>["昼月蝕皆既"],
    "1490.05.15"=>["月帯蝕皆既"],		"1490.10.15"=>["昼月蝕皆既"],		"1491.04.15"=>["月蝕四分"],
    "1491.10.15"=>["昼月蝕三分"],		"1492.03.16"=>["(月蝕四分)"],		"1492.09.15"=>["(月帯蝕三分)"],
    "1493.03.16"=>["昼月蝕皆既"],		"1493.08.15"=>["月蝕皆既"],		"1494.02.15"=>["昼月蝕皆既"],
    "1494.08.16"=>["昼月蝕皆既"],		"1495.02.15"=>["(昼月蝕ニ分)"],		"1495.08.16"=>["(月蝕ニ分)"],
    "1496.01.15"=>["昼月蝕四分"],		"1496.06.15"=>["(昼月蝕五分)"],		"1496.12.15"=>["月蝕皆既"],
    "1497.06.15"=>["月帯蝕皆既"],		"1497.12.16"=>["月帯蝕皆既"],		"1498.06.14"=>["月蝕七分"],
    "1498.11.16"=>["(月蝕ニ分)"],		"1499.04.16"=>["(昼月蝕一分)"],		"1499.10.16"=>["(昼月蝕三分)"],
    "1500.04.15"=>["月蝕九分"],			"1500.10.15"=>["昼月蝕九分"],		"1501.04.16"=>["昼月蝕皆既"],
    "1501.09.15"=>["月帯蝕皆既"],		"1502.03.16"=>["(月蝕三分)"],		"1502.09.16"=>["月帯蝕三分"],
    "1503.02.15"=>["(月帯蝕四分)"],		"1503.08.16"=>["(昼月蝕四分)"],		"1504.02.15"=>["昼月蝕皆既"],
    "1504.07.16"=>["月蝕皆既"],			"1505.01.15"=>["月蝕皆既"],		"1505.07.15"=>["月帯蝕皆既"],
    "1506.01.16"=>["昼月蝕ニ分"],		"1506.07.15"=>["(月帯蝕ニ分)"],		"1506.11=16"=>["(昼月蝕四分)"],
    "1507.05.14"=>["月蝕六分"],			"1507.11.16"=>["月蝕皆既"],		"1508.05.15"=>["昼月蝕皆既"],
    "1508.11.15"=>["月蝕皆既"],			"1509.05.16"=>["昼月蝕五分"],		"1509.10.15"=>["月蝕三分"],
    "1510.03.16"=>["(昼月蝕三分)"],		"1510.09.15"=>["(昼月蝕ニ分)"],		"1511.03.16"=>["月帯蝕九分半余"],
    "1511.09.16"=>["月帯蝕皆既"],		"1512.03.15"=>["月帯蝕皆既"],		"1512.08.16"=>["月蝕皆既"],
    "1513.02.14"=>["月蝕三分"],			"1513.08.16"=>["昼月蝕三分"],		"1514.01.15"=>["(月蝕四分)"],
    "1514.07.15"=>["(月蝕四分)"],		"1515.01.16"=>["昼月蝕皆既"],		"1515.06.14"=>["月蝕皆既"],
    "1515.12.16"=>["月蝕皆既"],			"1516.06.15"=>["昼月蝕皆既"],		"1516.12.16"=>["(昼月蝕ニ分)"],
    "1517.05.15"=>["(昼月蝕一分にミたす)"],	"1517.06.15"=>["(月蝕一分)"],		"1517.10=15"=>["(月帯蝕三分)"],
    "1518.04.16"=>["昼月蝕八分"],		"1518.10.15"=>["月帯蝕九分"],		"1519.04.16"=>["月帯蝕皆既"],
    "1519.10.14"=>["月蝕皆既"],			"1520.04.15"=>["月蝕四分"],		"1520.09.15"=>["昼月蝕四分"],
    "1521.02.15"=>["(昼月蝕三分)"],		"1521.08.16"=>["(月蝕三分)"],		"1522.02.15"=>["月帯蝕皆既"],
    "1522.08.16"=>["月帯蝕皆既"],		"1523.02.14"=>["月帯蝕皆既"],		"1523.07.16"=>["昼月蝕皆既"],
    "1524.01.15"=>["月蝕ニ分"],			"1524.07.15"=>["(昼月蝕三分)"],		"1524.12.16"=>["(月蝕四分)"],
    "1525.06.15"=>["月帯蝕五分"],		"1525.11=15"=>["月帯蝕皆既"],		"1526.05.15"=>["月帯蝕皆既"],
    "1526.11.15"=>["月帯蝕皆既"],		"1527.05.16"=>["昼月蝕六分"],		"1527.11.15"=>["月帯蝕三分"],
    "1528.04.16"=>["(昼月蝕ニ分)"],		"1528.09=14"=>["(月蝕ニ分)"],		"1529.03.15"=>["月蝕九分"],
    "1529.09.15"=>["昼月蝕皆既"],		"1530.03.15"=>["月蝕皆既"],		"1530.09.16"=>["月帯蝕皆既"],
    "1531.03.14"=>["月蝕三分"],			"1531.08.16"=>["月帯蝕三分"],		"1532.01.15"=>["(月蝕四分)"],
    "1532.07.16"=>["(昼月蝕四分)"],		"1533.01.16"=>["月蝕皆既"],		"1533.07.15"=>["昼月蝕皆既"],
    "1534.01.16"=>["昼月蝕皆既"],		"1534.06.15"=>["昼月蝕皆既"],		"1534.12.16"=>["(月蝕ニ分)"],
    "1535.06.14"=>["(月蝕一分)"],		"1535.11.15"=>["(月蝕三分)"],		"1536.05.16"=>["昼月蝕七分"],
    "1536.10=14"=>["月蝕九分"],			"1537.04.16"=>["月帯蝕皆既"],		"1537.10.15"=>["昼月蝕皆既"],
    "1538.04.16"=>["昼月蝕四分"],		"1538.10.15"=>["月蝕四分"],		"1539.03.15"=>["(月蝕三分)"],
    "1539.08.15"=>["(月蝕三分)"],		"1540.02.14"=>["月蝕九分"],		"1540.08.16"=>["昼月蝕九分"],
    "1541.02.15"=>["昼月蝕皆既"],		"1541.08.15"=>["月帯蝕皆既"],		"1542.02.15"=>["月蝕ニ分"],
    "1542.07.15"=>["(月帯蝕三分)"],		"1542.12.16"=>["(月帯蝕四分)"],		"1543.06.15"=>["昼月蝕四分"],
    "1543.12.16"=>["昼月蝕皆既"],		"1544.06.15"=>["月帯蝕皆既"],		"1544.11=15"=>["月帯蝕皆既"],
    "1545.05.16"=>["月蝕七分"],			"1545.11.15"=>["昼月蝕三分"],		"1546.04.16"=>["(月蝕ニ分)"],
    "1546.10.15"=>["(昼月蝕ニ分)"],		"1547.04.16"=>["月帯蝕八分"],		"1547.09.15"=>["月蝕九分"],
    "1548.03.15"=>["月帯蝕皆既"],		"1548.09.16"=>["月帯蝕皆既"],		"1549.03.15"=>["昼月蝕四分"],
    "1549.09.16"=>["月蝕三分"],			"1550.02.15"=>["(昼月蝕三分)"],		"1550.07.15"=>["(昼月蝕三分)"],
    "1551.01.15"=>["月帯蝕皆既"],		"1551.07.15"=>["昼月蝕九分半余"],	"1552.01.16"=>["月帯蝕皆既"],
    "1552.07.14"=>["月蝕皆既"],			"1553.01.16"=>["(月蝕ニ分)"],		"1553.06.15"=>["昼月蝕ニ分"],
    "1553.11.15"=>["(昼月蝕三分)"],		"1554.05.16"=>["月蝕六分"],		"1554.11.15"=>["昼月蝕九分"],
    "1555.05.16"=>["昼月蝕皆既"],		"1555.10=15"=>["月帯蝕皆既"],		"1556.04.16"=>["昼月蝕五分"],
    "1556.10.16"=>["昼月蝕四分"],		"1557.03.14"=>["(月蝕ニ分)"],		"1557.09.16"=>["(昼月蝕ニ分)"],
    "1558.03.15"=>["昼月蝕九分"],		"1558.08.16"=>["月蝕九分"],		"1559.02.15"=>["月帯蝕皆既"],
    "1559.08.15"=>["月蝕皆既"],			"1560.02.16"=>["昼月蝕三分"],		"1560.08.14"=>["月蝕四分"],
    "1561.01.16"=>["(昼月蝕四分)"],		"1561.06.14"=>["(月蝕四分)"],		"1561.12.16"=>["月蝕皆既"],
    "1562.06.15"=>["昼月蝕皆既"],		"1562.12.15"=>["月蝕皆既"],		"1563.06.16"=>["月帯蝕皆既"],
    "1563.12.14"=>["月蝕三分"],			"1564.04.16"=>["(昼月蝕一分)"],		"1564.05.16"=>["(昼月蝕一分にミたす)"],
    "1564.10.15"=>["(月帯蝕ニ分)"],		"1565.04.16"=>["昼月蝕八分"],		"1565.10.16"=>["月帯蝕九分"],
    "1566.04.15"=>["昼月蝕皆既"],		"1566.09.16"=>["月蝕皆既"],		"1567.03.14"=>["月帯蝕四分"],
    "1567.09.16"=>["昼月蝕四分"],		"1568.02.15"=>["(月蝕三分)"],		"1568.08.15"=>["(月蝕三分)"],
    "1569.02.16"=>["昼月蝕皆既"],		"1569.07.14"=>["月蝕九分"],		"1570.01.16"=>["月蝕皆既"],
    "1570.07.15"=>["月帯蝕皆既"],		"1571.01.16"=>["(昼月蝕ニ分)"],		"1571.07.15"=>["月帯蝕三分"],
    "1571.12.15"=>["(月帯蝕三分)"],		"1572.05.16"=>["月帯蝕六分"],		"1572.11.15"=>["月蝕九分"],
    "1573.05.16"=>["月帯蝕皆既"],		"1573.11.14"=>["月蝕皆既"],		"1574.05.15"=>["月蝕六分"],
    "1574.11.15"=>["月蝕四分"],			"1575.03.15"=>["(昼月蝕ニ分)"],		"1575.09.16"=>["(月蝕ニ分)"],
    "1576.03.15"=>["昼月蝕八分"],		"1576.09.16"=>["月帯蝕九分"],		"1577.03.14"=>["月帯蝕皆既"],
    "1577.08.16"=>["昼月蝕皆既"],		"1578.02.15"=>["月蝕三分"],		"1578.08.15"=>["昼月蝕四分"],
    "1579.01.16"=>["(月蝕四分)"],		"1579.07.14"=>["(月蝕三分)"],		"1580.01.16"=>["月帯蝕皆既"],
    "1580.06.15"=>["月帯蝕皆既"],		"1580.12.16"=>["月帯蝕皆既"],		"1581.06.16"=>["昼月蝕皆既"],
    "1581.12.15"=>["昼月蝕三分"],		"1582.05.16"=>["(昼月蝕一分にミたす)"],	"1582.06.16"=>["(月蝕一分)"],
    "1582.11.14"=>["(月蝕ニ分)"],		"1583.04.15"=>["月帯蝕七分"],		"1583.10.15"=>["月帯蝕九分"],
    "1584.04.15"=>["月帯蝕皆既"],		"1584.10.16"=>["月帯蝕皆既"],		"1585.04.14"=>["月蝕五分"],
    "1585.09.16"=>["月帯蝕四分"],		"1586.02.15"=>["(月蝕三分)"],		"1586.08.15"=>["(月蝕ニ分)"],
    "1587.02.16"=>["月蝕皆既"],			"1587.08.15"=>["月帯蝕九分"],		"1588.02.16"=>["昼月蝕皆既"],
    "1588.07.15"=>["昼月蝕皆既"],		"1589.01.16"=>["(月蝕三分)"],		"1589.07.15"=>["月蝕三分"],
    "1589.12.15"=>["(月蝕三分)"],		"1590.06.16"=>["昼月蝕五分"],		"1590.12.14"=>["月蝕九分"],
    "1591.05.16"=>["月蝕皆既"],			"1591.11.15"=>["昼月蝕皆既"],		"1592.05.16"=>["月帯蝕七分"],
    "1592.11.15"=>["月蝕四分"],			"1593.04.15"=>["(昼月蝕一分)"],		"1593.09=17"=>["(月蝕ニ分)"],
    "1594.03.14"=>["月蝕八分"],			"1594.09.16"=>["昼月蝕九分"],		"1595.03.15"=>["昼月蝕皆既"],
    "1595.09.15"=>["月帯蝕皆既"],		"1596.03.16"=>["月帯蝕四分"],		"1596.08.15"=>["月帯蝕四分"],
    "1597.01.16"=>["(昼月蝕三分)"],		"1597.07.15"=>["(昼月蝕ニ分)"],		"1598.01.16"=>["昼月蝕九分半余"],
    "1598.07.15"=>["月蝕皆既"],			"1599.01.15"=>["昼月蝕皆既"],		"1599.06.16"=>["月帯蝕皆既"],
    "1599.12.15"=>["月帯蝕三分"],		"1600.06.16"=>["(昼月蝕ニ分)"],		"1600.11.15"=>["(昼月蝕ニ分)"],
    "1601.05.15"=>["月蝕六分"],			"1601.11.15"=>["月蝕九分"],		"1602.04.14"=>["月帯蝕皆既"],
    "1602.10.16"=>["月帯蝕皆既"],		"1603.04.15"=>["昼月蝕六分"],		"1603.10.16"=>["月蝕四分"],
    "1604.03.15"=>["(昼月蝕ニ分)"],		"1604.08=15"=>["(昼月蝕ニ分)"],		"1605.02.16"=>["月帯蝕九分"],
    "1605.08.15"=>["昼月蝕八分"],		"1606.02.16"=>["月帯蝕皆既"],		"1606.08.14"=>["月帯蝕皆既"],
    "1607.02.16"=>["(月蝕三分)"],		"1607.07.15"=>["昼月蝕四分"],		"1607.12.15"=>["(昼月蝕三分)"],
    "1608.06.16"=>["月帯蝕四分"],		"1608.12.15"=>["昼月蝕九分"],		"1609.06.16"=>["月帯蝕皆既"],
    "1609.12.15"=>["月蝕皆既"],			"1610.05.16"=>["昼月蝕七分"],		"1610.11.16"=>["昼月蝕四分"],
    "1611.04.14"=>["(月蝕一分にミたす)"],	"1611.05.15"=>["(昼月蝕一分)"],		"1611.10.16"=>["(昼月蝕ニ分)"],
    "1612.04.15"=>["昼月蝕七分"],		"1612.10.16"=>["月蝕八分"],		"1613.03.15"=>["月帯蝕皆既"],
    "1613.09.15"=>["月蝕皆既"],			"1614.03.16"=>["昼月蝕四分"],		"1614.09.14"=>["月蝕五分"],
    "1615.02.16"=>["(昼月蝕三分)"],		"1615.07.15"=>["(月帯蝕ニ分)"],		"1616.01.16"=>["月蝕九分半余"],
    "1616.07.15"=>["昼月蝕皆既"],		"1617.01.15"=>["月蝕皆既"],		"1617.07.15"=>["月帯蝕皆既"],
    "1618.01.14"=>["月蝕三分"],			"1618.06.16"=>["(昼月蝕ニ分)"],		"1618.11.15"=>["(月蝕ニ分)"],
    "1619.05.16"=>["昼月蝕五分"],		"1619.11.16"=>["昼月蝕九分"],		"1620.05.15"=>["昼月蝕皆既"],
    "1620.11.16"=>["月蝕皆既"],			"1621.04.15"=>["昼月蝕六分"],		"1621.10.16"=>["昼月蝕四分"],
    "1622.03.15"=>["(月蝕ニ分)"],		"1622.09.15"=>["(月蝕一分)"],		"1623.03.16"=>["昼月蝕九分"],
    "1623.08=14"=>["月蝕八分"],			"1624.02.16"=>["月蝕皆既"],		"1624.08.14"=>["月帯蝕皆既"],
    "1625.02.16"=>["昼月蝕三分"],		"1625.08.15"=>["月帯蝕四分"],		"1626.01.15"=>["(月帯蝕三分)"],
    "1626.06.16"=>["(月蝕三分)"],		"1626.12.14"=>["月蝕九分"],		"1627.06.16"=>["昼月蝕皆既"],
    "1627.12.14"=>["月帯蝕皆既"],		"1628.06.15"=>["月帯蝕皆既"],		"1628.12.15"=>["月蝕四分"],
    "1629.05.15"=>["(月蝕一分)"],		"1629.10.16"=>["(月蝕ニ分)"],		"1630.04.15"=>["昼月蝕七分"],
    "1630.10.16"=>["月帯蝕八分"],		"1631.04.14"=>["月帯蝕皆既"],		"1631.10.16"=>["月帯蝕皆既"],
    "1632.03.15"=>["月蝕五分"],			"1632.09.15"=>["昼月蝕五分"],		"1633.02.16"=>["(月蝕三分)"],
    "1633.08.14"=>["(月蝕ニ分)"],		"1634.02.15"=>["月帯蝕九分"],		"1634.07=15"=>["月帯蝕九分"],
    "1635.01.14"=>["月帯蝕皆既"],		"1635.07.16"=>["昼月蝕皆既"],		"1636.01.15"=>["昼月蝕四分"],
    "1636.07.16"=>["月蝕三分"],			"1636.12.14"=>["(月蝕ニ分)"],		"1637.05.15"=>["(昼月蝕五分)"],
    "1637.11.15"=>["月蝕九分"],			"1638.05.15"=>["昼月蝕皆既"],		"1638.11.16"=>["昼月蝕皆既"],
    "1639.05.14"=>["月蝕七分"],			"1639.11.16"=>["月蝕四分"],		"1640.03.15"=>["(月蝕一分)"],
    "1640.09.15"=>["(月蝕一分)"],		"1641.03.16"=>["月蝕八分"],		"1641.09.14"=>["月蝕八分"],
    "1642.03.16"=>["昼月蝕皆既"],		"1642.09.15"=>["昼月蝕皆既"],		"1643.02.16"=>["昼月蝕四分"],
    "1643.08.15"=>["月蝕五分"],			"1644.01.15"=>["(月蝕三分)"],		"1644.07.16"=>["(昼月蝕三分)"],
    "1645.01.14"=>["月帯蝕九分"],		"1645.06.16"=>["月帯蝕皆既"],		"1645.12.15"=>["月帯蝕皆既"],
    "1646.06.15"=>["月蝕皆既"],			"1646.12.15"=>["月蝕四分"],		"1647.06.14"=>["(月蝕ニ分)"],
    "1647.11.16"=>["(昼月蝕ニ分)"],		"1648.04.14"=>["月蝕六分"],		"1648.10.16"=>["昼月蝕八分"],
    "1649.04.15"=>["昼月蝕皆既"],		"1649.10.15"=>["月帯蝕皆既"],		"1650.04.16"=>["月帯蝕五分"],
    "1650.10.15"=>["月帯蝕五分"],		"1651.02.16"=>["(昼月蝕ニ分)"],		"1651.08.15"=>["(昼月蝕一分)"],
    "1652.02.16"=>["昼月蝕九分"],		"1652.08.15"=>["月蝕八分"],		"1653.02.15"=>["昼月蝕皆既"],
    "1653.07.16"=>["月帯蝕皆既"],		"1654.01.15"=>["昼月蝕四分"],		"1654.07.16"=>["昼月蝕四分"],
    "1654.12.15"=>["(昼月蝕ニ分)"],		"1655.06.15"=>["(月蝕四分)"],		"1655.12.15"=>["月帯蝕九分"],
    "1656.05.14"=>["月蝕皆既"],			"1656.11.16"=>["月蝕皆既"],		"1657.05.15"=>["月帯蝕皆既"],
    "1657.11.16"=>["月蝕四分"],			"1658.04.15"=>["(昼月蝕一分にミたす)"],	"1658.05.15"=>["(昼月蝕一分)"],
    "1658.10.15"=>["(昼月蝕一分)"],		"1659.03.16"=>["月帯蝕八分"],		"1659.09.15"=>["昼月蝕七分"],
    "1660.03.16"=>["月帯蝕皆既"],		"1660.09.14"=>["月蝕皆既"],		"1661.03.15"=>["月蝕四分"],
    "1661.08=15"=>["昼月蝕五分"],		"1662.01.15"=>["(昼月蝕ニ分)"],		"1662.07.16"=>["(昼月蝕ニ分)"],
    "1663.01.15"=>["昼月蝕九分"],		"1663.07.16"=>["月帯蝕九分"],		"1664.01.15"=>["月蝕皆既"],
    "1664.06.16"=>["昼月蝕皆既"],		"1664.12.16"=>["昼月蝕四分"],		"1665.06.15"=>["(昼月蝕三分)"],
    "1665.11.16"=>["(月帯蝕ニ分)"],		"1666.05.14"=>["月帯蝕五分"],		"1666.11.16"=>["月蝕八分"],
    "1667.04.15"=>["月帯蝕皆既"],		"1667.10.15"=>["月蝕皆既"],		"1668.04.16"=>["昼月蝕六分"],
    "1668.10.14"=>["月蝕五分"],			"1669.03.16"=>["(昼月蝕ニ分)"],		"1669.09.15"=>["(月蝕一分)"],
    "1670.02.16"=>["月帯蝕九分"],		"1670.08.16"=>["昼月蝕八分"],		"1671.02.15"=>["月帯蝕皆既"],
    "1671.08.16"=>["月蝕皆既"],			"1672.02.14"=>["月蝕四分"],		"1672.07.16"=>["昼月蝕四分"],
    "1672.12.15"=>["(月蝕ニ分)"],		"1673.06.15"=>["(月蝕三分)"],		"1673.12.16"=>["昼月蝕九分"],
    "1674.06.15"=>["月帯蝕皆既"],		"1674.12.16"=>["月蝕皆既"],		"1675.05.15"=>["昼月蝕皆既"],
    "1675.11.16"=>["昼月蝕四分"],		"1676.05.14"=>["(月蝕一分)"],		"1676.10.15"=>["(月蝕一分)"],
    "1677.04.16"=>["昼月蝕七分"],		"1677.10.14"=>["月蝕七分"],		"1678.03.16"=>["月蝕皆既"],
    "1678.09.14"=>["月帯蝕皆既"],		"1679.03.16"=>["昼月蝕五分"],		"1679.09.15"=>["月帯蝕五分"],
    "1680.02.15"=>["(昼月蝕ニ分)"],		"1680.08.16"=>["(月蝕ニ分)"],		"1681.01.14"=>["月蝕八分"],
    "1681.07.16"=>["昼月蝕九分"],		"1682.01.15"=>["月帯蝕皆既"],		"1682.07.16"=>["昼月蝕皆既"],
    "1683.01.15"=>["月蝕四分"],			"1683.06.15"=>["昼月蝕四分"],		"1683.11.16"=>["(月蝕ニ分)"],

    # 暫定的な月食データ - 大崎正次編『近世日本天文史料』による(*付きは計算で補ったもの)
    "1685.05.15"=>["月そく皆つくる"],		"1685.11.15"=>["月帯そく"],		"1686.04.16"=>["月そく"],
    "1688.03.15"=>["月そく六分"],		"1688.09.16"=>["月蝕六分"],		"1689.02.15"=>["月そく皆つくる"],
    "1690.02.14"=>["月帯そく"],			"1690.08.16"=>["月そく四分"],		"1691.12.16"=>["月そく七分"],
    "1693.06.15"=>["月帯そく皆既"],		"1693.12.16"=>["月そく四分"],		"1695.04.16"=>["月そく四分"],
    "1695.10.14"=>["月そく五分"],		"1697.09.15"=>["月そく六分"],		"1699.02.14"=>["月そく六分"],
    "1699.08.16"=>["月そく七分"],		"1700.01.15"=>["月帯そく六分半"],	"1700.07.15"=>["月そく"],
    "1701.07.15"=>["月そく四分"],		"1702.11.16"=>["月帯そく三分"],		"1703.11.15"=>["月帯そく"],
    "1704.05.16"=>["月そく九分"],		"1704.11.15"=>["月帯そく四分"],		"1706.09.15"=>["月そく五分"],
    "1707.09.16"=>["月そく皆つくる"],		"1708.08.16"=>["月そく五分"],		"1710.07.15"=>["月帯そく八分"],
    "1710.12.16"=>["月そく皆つくる"],		"1711.06.14"=>["月そく皆つくる"],	"1711.12.16"=>["月そく四分"],
    "1713.05.16"=>["月帯蝕三分"],		"1714.10.15"=>["月そく皆つくる"],	"1715.04.16"=>["月そく七分"],
    "1717.08.16"=>["月そく六分"],		"1718.02.15"=>["月そく皆つくる"],	"1718.08.15"=>["月帯そく皆つくる"],
    "1719.01.16"=>["月帯そく五分"],		"1719.07.14"=>["月帯蝕五分"],		"1720.12.16"=>["月そく六分"],
    "1721.06.15"=>["月帯そく"],			"1721.11.15"=>["月そく皆つくる*"],	"1722.11.15"=>["月そく六分"],
    "1725.03.15"=>["月帯そく"],			"1725.09.15"=>["月そく皆既"],		"1726.03.15"=>["月そく六分"],
    "1728.01.16"=>["月帯そく"],			"1728.07.14"=>["月そく七分"],		"1729.01.16"=>["月入帯そく皆既"],
    "1730.06.15"=>["月そく三分"],		"1731.11.15"=>["月そく五分"],		"1732.05.16"=>["月そく皆既"],
    "1732.10.14"=>["月帯そく皆既"],		"1733.04.15"=>["月そく八分"],		"1733.10.15"=>["月そく六分半"],
    "1735.03.15"=>["月そく五分"],		"1737.02.16"=>["月そく五分"],		"1738.12.15"=>["月帯そく"],
    "1739.06.15"=>["月そく八分"],		"1739.12.15"=>["月帯そく"],		"1740.06.16"=>["月帯そく皆つきて出"],
    "1742.04.15"=>["月そく四分"],		"1742.10.16"=>["月そく五分"],		"1743.04.15"=>["月そく皆つくる"],
    "1744.03.14"=>["月帯そく"],			"1744.09.16"=>["月そく六分"],		"1746.01.16"=>["月そく六分"],
    "1747.07.15"=>["月帯そく"],			"1748.01.16"=>["月そく五分"],		"1749.11.14"=>["月そく五分"],
    "1750.05.16"=>["月帯そく"],			"1750.11.15"=>["月帯そく"],		"1751.10.15"=>["月帯そく"],
    "1753.03.14"=>["月そく四分"],		"1753.09.16"=>["月帯そく"],		"1754.02=15"=>["月帯そく"],
    "1754.08.15"=>["月帯そく"],			"1755.08.15"=>["月そく六分"],		"1756.12.16"=>["月帯そく"],
    "1758.06.16"=>["月そく皆つくる"],		"1760.10.15"=>["月帯そく"],		"1761.10.16"=>["月そく皆つくる*"],
    "1762.09.16"=>["月帯そく七分"],		"1765.01.16"=>["凡そく皆既"],		"1765.07.14"=>["凡そく皆既"],
    "1766.01.16"=>["凡そく五分"],		"1767.06.16"=>["月そく五十秒"],		"1768.11.15"=>["月そく皆既"],
    "1769.11.16"=>["月帯そく"],			"1771.09.16"=>["月そく五分"],		"1772.03.15"=>["月そく皆既"],
    "1772.09.15"=>["月そく皆既"],		"1773.03.16"=>["月帯そく"],		"1773.08.14"=>["月そく八分"],
    "1775.01.16"=>["月そく七分"],		"1775.12=15"=>["月そく皆既*"],		"1776.12.14"=>["月そく六分半"],
    "1777.06.16"=>["月そく三分"],		"1779.10.16"=>["月そく皆既"],		"1780.04.15"=>["月帯蝕九分"],
    "1782.08.15"=>["月そく四分半"],		"1783.08.15"=>["月そく皆既*"],		"1784.07.15"=>["月そく七分"],
    "1785.12.15"=>["月そく五分"],		"1786.06.16"=>["月帯そく九分"],		"1787.05.15"=>["月そく皆既"],
    "1787.11.15"=>["月そく八分*"],		"1789.04.15"=>["月帯そく"],		"1791.03.16"=>["月そく七分半"],
    "1793.01.16"=>["月帯そく"],			"1793.07.15"=>["月そく六分半"],		"1794.01.16"=>["月帯そく"],
    "1795.06.16"=>["月帯そく四分"],		"1796.05.15"=>["月帯そく"],		"1796.11.16"=>["月そく四分半"],
    "1797.05.15"=>["月帯そく皆既"],		"1798.04.14"=>["月そく皆既"],		"1798.10.16"=>["月そく六分半"],
    "1800.03.16"=>["月そく五分半"],		"1800.08.15"=>["月帯そく"],		"1801.08.15"=>["月帯そく"],
    "1802.02.16"=>["月蝕四分半"],		"1804.06.16"=>["月蝕九分"],		"1804.12.15"=>["月帯蝕皆既"],
    "1805.06.16"=>["月帯食"],			"1807.04.14"=>["月蝕一分"],		"1807.10.16"=>["月帯食二分半"],
    "1808.09.15"=>["月帯蝕皆既"],		"1809.09.15"=>["月帯そく八分半"],	"1812.07.16"=>["月そく皆既"],
    "1813.01.15"=>["月帯食六分出"],		"1814.11.16"=>["月帯そく"],		"1815.05.14"=>["月蝕皆既"],
    "1815.11.16"=>["月蝕皆既"],			"1816.10.16"=>["月帯そく七分"],		"1819.03.16"=>["月そく皆既"],
    "1819.08.15"=>["月そく皆既"],		"1820.02.16"=>["月そく五分半"],		"1822.12.15"=>["月そく皆既"],
    "1823.12.16"=>["月帯そく八分"],		"1825.10.16"=>["月そくニ分"],		"1826.04.15"=>["月そく皆既"],
    "1826.10.15"=>["月そく皆既"],		"1827.04.16"=>["月帯そく"],		"1827.09.14"=>["月そく九分"],
    "1829.02.16"=>["月蝕三分半"],		"1830.02.15"=>["月そく皆既"],		"1830.07.17"=>["月帯そく"],
    "1831.01.14"=>["月帯そく五分"],		"1832.11=16"=>["月帯食四分半"],		"1833.11.16"=>["月帯食皆既*"],
    "1834.05.16"=>["月帯食"],			"1836.09.15"=>["月食一分"],		"1837.03.17"=>["月帯食"],
    "1837.09.15"=>["月帯食"],			"1838.08.15"=>["月食九分半"],		"1840.01.15"=>["月そく三分半"],
    "1841.06.16"=>["月帯食"],			"1841.12.15"=>["月食八分"],		"1842.06.15"=>["月帯食三分"],
    "1847.02.16"=>["月帯食"],			"1847.08.15"=>["月食四分半"],		"1848.02.16"=>["月帯食"],
    "1849.07.16"=>["月食六分"],			"1850.12.16"=>["月そく五分"],		"1851.12.16"=>["月帯そく"],
    "1852.05.14"=>["月そく皆既"],		"1852.11.16"=>["月そく六分半"],		"1854.04.16"=>["月そくニ分半"],
    "1854.09.14"=>["月帯そく一分にミたす"],	"1855.09.15"=>["月帯そく皆既"],		"1856.03.16"=>["月帯そく"],
    "1858.01.15"=>["月帯そく"],			"1858.07.16"=>["月そく五分"],		"1859.01.15"=>["月そく皆既"],
    "1859.07.15"=>["月蝕皆既*"],		"1860.06.15"=>["月そく四分半"],		"1861.11.16"=>["月帯そく"],
    "1862.10.15"=>["月帯そく皆既"],		"1863.10.15"=>["月帯そく九分半"],	"1866.08.16"=>["月そく皆既"],
    "1867.02.15"=>["月帯そく"],			"1869.06.15"=>["月そく六分*"],		"1869.12.16"=>["月そく皆既*"],
    "1870.11.16"=>["月帯そく七分*"],		"1871.05.15"=>["月そく三分*"],
  }

  # @private
  Mag = %w(一分にミたす 一分 ニ分 三分 四分 五分 六分 七分 八分 九分 九分半余)

  # @private
  Category = {'T'=>'皆既', 'A'=>'金環'}

  # 日月食の概要を文字列化する
  #
  # @param [Array<String, Numeric, Array>] info 食の情報
  #
  #   [ String  - 食の種類 'T' 皆既食, 'A' 金環食, 'P' 部分食, 'B' 帯食]
  #   [ Numeric - 最大食分                                             ]
  #   [ Array   - 食の詳細                                             ]
  #
  # @return String 食の概要
  #
  def self.eclipse_summary(info)
    category, mag = info
    (category =~ /B/i ? '帯蝕' : '蝕') +
    (Category[category[0..0]] || Mag[(mag * 10).round])
  end
end
